@@ -128,6 +128,8 @@ mm_support (void)
             rval |= MM_MMX;
         if (edx & (1 << 24))
             rval |= MM_MMXEXT;
+        if (rval==0)
+            goto inteltest;
         return rval;
     }
     else if (ebx == 0x69727943 && edx == 0x736e4978 && ecx == 0x64616574)
@@ -150,6 +152,17 @@ mm_support (void)
         if (eax & 0x01000000)
             rval |= MM_MMXEXT;
         return rval;
+    } 
+    else if (ebx == 0x756e6547 && edx == 0x54656e69 && ecx == 0x3638784d) 
+    {
+        /* Tranmeta Crusoe */
+        cpuid(0x80000000, eax, ebx, ecx, edx);
+        if ((unsigned)eax < 0x80000001)
+            return 0;
+        cpuid(0x80000001, eax, ebx, ecx, edx);
+        if ((edx & 0x00800000) == 0)
+            return 0;
+        return MM_MMX;
     }
     else
     {
@@ -157,6 +170,7 @@ mm_support (void)
     }
 }
 #else
+#define emms()
 int mm_support(void) { return 0; }
 #endif // i386
 
