@@ -14,6 +14,9 @@ int VIA_field_start(MpegEncContext*s, AVCodecContext *avctx)
     slicestate = (via_slice_state_t *)s->current_picture.data[0];
     assert(slicestate != NULL);
 
+    slicestate->progressive_sequence = s->progressive_sequence;
+    slicestate->top_field_first = s->top_field_first;
+
     VIAMPGSurface.dwPictureType = s->pict_type;
     VIAMPGSurface.dwDecodePictStruct = s->picture_structure;
     VIAMPGSurface.dwDecodeBuffIndex = slicestate->image_number;
@@ -74,7 +77,7 @@ int VIA_field_start(MpegEncContext*s, AVCodecContext *avctx)
     VIAMPGSurface.dwConcealMV = s->concealment_motion_vectors;
 
     // XXX: not sure on this
-    VIAMPGSurface.dwSecondField = s->first_field;
+    VIAMPGSurface.dwSecondField = s->first_field?0:1;
 
     // FIXME: fill in from last, next picture_ptr
     VIAMPGSurface.dwOldRefFrame = 0;
@@ -99,6 +102,9 @@ int VIA_field_start(MpegEncContext*s, AVCodecContext *avctx)
     VIAMPGSurface.BVMVRange = s->mpeg_f_code[1][1] - 1;
     VIAMPGSurface.BHMVRange = s->mpeg_f_code[1][0] - 1;
 
+    VIAMPGSurface.dwMPEGProgressiveMode = s->progressive_sequence?
+                                          VIA_PROGRESSIVE:VIA_NON_PROGRESSIVE;
+
     VIABeginPicture(&VIAMPGSurface);
 
     return 0;
