@@ -1,7 +1,7 @@
 /*
  * MOV, 3GP, MP4 encoder.
  * Copyright (c) 2003 Thomas Raivio.
- * Enhancements by Gildas Bazin <gbazin@netcourrier.com>
+ * Copyright (c) 2004 Gildas Bazin <gbazin at videolan dot org>.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,10 @@
 #define MOV_INDEX_CLUSTER_SIZE 16384
 #define globalTimescale 1000
 
+#define MODE_MP4 0
+#define MODE_MOV 1
+#define MODE_3GP 2
+
 typedef struct MOVIentry {
     unsigned int flags, pos, size;
     unsigned int samplesInChunk;
@@ -35,6 +39,7 @@ typedef struct MOVIentry {
 } MOVIentry;
 
 typedef struct MOVIndex {
+    int         mode;
     int         entry;
     int         mdat_size;
     int         ents_allocated;
@@ -53,11 +58,11 @@ typedef struct MOVIndex {
 } MOVTrack;
 
 typedef struct {
+    int     mode;
     long    time;
     int     nb_streams;
     int     mdat_written;
     offset_t mdat_pos;
-    offset_t movi_list;
     long    timescale;
     MOVTrack tracks[MAX_STREAMS];
 } MOVContext;
@@ -204,33 +209,72 @@ static int mov_write_damr_tag(ByteIOContext *pb)
     return 0x11;
 }
 
+static int mov_write_wave_tag(ByteIOContext *pb, MOVTrack* track)
+{
+    int pos = url_ftell(pb);
+
+    put_be32(pb, 0);     /* size */
+    put_tag(pb, "wave");
+
+    put_be32(pb, 12);    /* size */
+    put_tag(pb, "frma");
+    put_tag(pb, "mp4a");
+
+    put_be32(pb, 12);    /* size */
+    put_tag(pb, "mp4a");
+    put_be32(pb, 0);
+
+    mov_write_esds_tag(pb, track);
+
+    put_be32(pb, 12);    /* size */
+    put_tag(pb, "srcq");
+    put_be32(pb, 0x40);
+
+    put_be32(pb, 8);     /* size */
+    put_be32(pb, 0);     /* null tag */
+
+    return updateSize (pb, pos);
+}
+
+const CodecTag codec_movaudio_tags[] = {
+    { CODEC_ID_PCM_MULAW, MKTAG('u', 'l', 'a', 'w') },
+    { CODEC_ID_PCM_ALAW, MKTAG('a', 'l', 'a', 'w') },
+    { CODEC_ID_ADPCM_IMA_QT, MKTAG('i', 'm', 'a', '4') },
+    { CODEC_ID_MACE3, MKTAG('M', 'A', 'C', '3') },
+    { CODEC_ID_MACE6, MKTAG('M', 'A', 'C', '6') },
+    { CODEC_ID_AAC, MKTAG('m', 'p', '4', 'a') },
+    { CODEC_ID_AMR_NB, MKTAG('s', 'a', 'm', 'r') },
+    { CODEC_ID_PCM_S16BE, MKTAG('t', 'w', 'o', 's') },
+    { CODEC_ID_PCM_S16LE, MKTAG('s', 'o', 'w', 't') },
+    { CODEC_ID_MP3, MKTAG('.', 'm', 'p', '3') },
+    { 0, 0 },
+};
+
 static int mov_write_audio_tag(ByteIOContext *pb, MOVTrack* track)
 {
     int pos = url_ftell(pb);
+    int tag;
+    
     put_be32(pb, 0); /* size */
 
-    if(track->enc->codec_id == CODEC_ID_PCM_MULAW)
-      put_tag(pb, "ulaw");
-    else if(track->enc->codec_id == CODEC_ID_PCM_ALAW)
-      put_tag(pb, "alaw");
-    else if(track->enc->codec_id == CODEC_ID_ADPCM_IMA_QT)
-      put_tag(pb, "ima4");
-    else if(track->enc->codec_id == CODEC_ID_MACE3)
-      put_tag(pb, "MAC3");
-    else if(track->enc->codec_id == CODEC_ID_MACE6)
-      put_tag(pb, "MAC6");
-    else if(track->enc->codec_id == CODEC_ID_AAC)
-      put_tag(pb, "mp4a");
-    else if(track->enc->codec_id == CODEC_ID_AMR_NB)
-      put_tag(pb, "samr");
-    else
-      put_tag(pb, "    ");
+    tag = codec_get_tag(codec_movaudio_tags, track->enc->codec_id);
+    // if no mac fcc found, try with Microsoft tags
+    if (!tag)
+    {
+	int tmp = codec_get_tag(codec_wav_tags, track->enc->codec_id);
+        tag = MKTAG('m', 's', ((tmp >> 8) & 0xff), (tmp & 0xff));
+    }
+    put_le32(pb, tag); // store it byteswapped
 
     put_be32(pb, 0); /* Reserved */
     put_be16(pb, 0); /* Reserved */
     put_be16(pb, 1); /* Data-reference index, XXX  == 1 */
+
     /* SoundDescription */
-    put_be16(pb, 0); /* Version */
+    if(track->mode == MODE_MOV && track->enc->codec_id == CODEC_ID_AAC)
+        put_be16(pb, 1); /* Version 1 */
+    else
+        put_be16(pb, 0); /* Version 0 */
     put_be16(pb, 0); /* Revision level */
     put_be32(pb, 0); /* Reserved */
 
@@ -238,13 +282,33 @@ static int mov_write_audio_tag(ByteIOContext *pb, MOVTrack* track)
     /* TODO: Currently hard-coded to 16-bit, there doesn't seem
                  to be a good way to get number of bits of audio */
     put_be16(pb, 0x10); /* Reserved */
-    put_be16(pb, 0); /* compression ID (= 0) */
+
+    if(track->enc->codec_id == CODEC_ID_AAC ||
+       track->enc->codec_id == CODEC_ID_MP3)
+    {
+        put_be16(pb, 0xfffe); /* compression ID (vbr)*/
+    }
+    else
+    {
+        put_be16(pb, 0); /* compression ID (= 0) */
+    }
     put_be16(pb, 0); /* packet size (= 0) */
     put_be16(pb, track->timescale); /* Time scale */
     put_be16(pb, 0); /* Reserved */
 
-    if(track->enc->codec_id == CODEC_ID_AAC)
-        mov_write_esds_tag(pb, track);
+    if(track->mode == MODE_MOV && track->enc->codec_id == CODEC_ID_AAC)
+    {
+        /* SoundDescription V1 extended info */
+        put_be32(pb, track->enc->frame_size); /* Samples per packet  */
+        put_be32(pb, 1536); /* Bytes per packet */
+        put_be32(pb, 2); /* Bytes per frame */
+        put_be32(pb, 2); /* Bytes per sample */
+    }
+
+    if(track->enc->codec_id == CODEC_ID_AAC) {
+        if( track->mode == MODE_MOV ) mov_write_wave_tag(pb, track);
+        else mov_write_esds_tag(pb, track);
+    }
     if(track->enc->codec_id == CODEC_ID_AMR_NB)
         mov_write_damr_tag(pb);
     return updateSize (pb, pos);
@@ -368,25 +432,32 @@ static int mov_write_esds_tag(ByteIOContext *pb, MOVTrack* track) // Basic
     }
 
     // SL descriptor
-    putDescr(pb, 0x06, descrLength(1));
+    putDescr(pb, 0x06, 1);
     put_byte(pb, 0x02);
     return updateSize (pb, pos);
 }
 
+const CodecTag codec_movvideo_tags[] = {
+    { CODEC_ID_SVQ1, MKTAG('S', 'V', 'Q', '1') },
+    { CODEC_ID_SVQ3, MKTAG('S', 'V', 'Q', '3') },
+    { CODEC_ID_MPEG4, MKTAG('m', 'p', '4', 'v') },
+    { CODEC_ID_H263, MKTAG('s', '2', '6', '3') },
+    { CODEC_ID_DVVIDEO, MKTAG('d', 'v', 'c', ' ') },
+    { 0, 0 },
+};
+
 static int mov_write_video_tag(ByteIOContext *pb, MOVTrack* track)
 {
     int pos = url_ftell(pb);
+    int tag;
+
     put_be32(pb, 0); /* size */
-    if(track->enc->codec_id == CODEC_ID_SVQ1)
-      put_tag(pb, "SVQ1");
-    else if(track->enc->codec_id == CODEC_ID_SVQ3)
-      put_tag(pb, "SVQ3");
-    else if(track->enc->codec_id == CODEC_ID_MPEG4)
-      put_tag(pb, "mp4v");
-    else if(track->enc->codec_id == CODEC_ID_H263)
-      put_tag(pb, "s263");
-    else
-      put_tag(pb, "    "); /* Unknown tag */
+
+    tag = codec_get_tag(codec_movvideo_tags, track->enc->codec_id);
+    // if no mac fcc found, try with Microsoft tags
+    if (!tag)
+	tag = codec_get_tag(codec_bmp_tags, track->enc->codec_id);
+    put_le32(pb, tag); // store it byteswapped
 
     put_be32(pb, 0); /* Reserved */
     put_be16(pb, 0); /* Reserved */
@@ -400,17 +471,18 @@ static int mov_write_video_tag(ByteIOContext *pb, MOVTrack* track)
     put_be16(pb, track->enc->height); /* Video height */
     put_be32(pb, 0x00480000); /* Reserved */
     put_be32(pb, 0x00480000); /* Reserved */
+    put_be32(pb, 0); /* Data size (= 0) */
+    put_be16(pb, 1); /* Frame count (= 1) */
+    
     put_be32(pb, 0); /* Reserved */
     put_be32(pb, 0); /* Reserved */
     put_be32(pb, 0); /* Reserved */
     put_be32(pb, 0); /* Reserved */
     put_be32(pb, 0); /* Reserved */
     put_be32(pb, 0); /* Reserved */
-
-    put_be16(pb, 0); /* Reserved */
-    put_be32(pb, 0); /* Reserved */
     put_be32(pb, 0); /* Reserved */
     put_be32(pb, 0); /* Reserved */
+    
     put_be16(pb, 0x18); /* Reserved */
     put_be16(pb, 0xffff); /* Reserved */
     if(track->enc->codec_id == CODEC_ID_MPEG4)
@@ -509,6 +581,39 @@ static int mov_write_vmhd_tag(ByteIOContext *pb)
     return 0x14;
 }
 
+static int mov_write_hdlr_tag(ByteIOContext *pb, MOVTrack* track)
+{
+    char *descr, *hdlr, *hdlr_type;
+    int pos = url_ftell(pb);
+    
+    if (!track) { /* no media --> data handler */
+	hdlr = "dhlr";
+	hdlr_type = "url ";
+	descr = "DataHandler";
+    } else {
+	hdlr = (track->mode == MODE_MOV) ? "mhlr" : "\0\0\0\0";
+	if (track->enc->codec_type == CODEC_TYPE_VIDEO) {
+	    hdlr_type = "vide";
+	    descr = "VideoHandler";
+	} else {
+	    hdlr_type = "soun";
+	    descr = "SoundHandler";
+	}
+    }
+    
+    put_be32(pb, 0); /* size */
+    put_tag(pb, "hdlr");
+    put_be32(pb, 0); /* Version & flags */
+    put_buffer(pb, hdlr, 4); /* handler */
+    put_tag(pb, hdlr_type); /* handler type */
+    put_be32(pb ,0); /* reserved */
+    put_be32(pb ,0); /* reserved */
+    put_be32(pb ,0); /* reserved */
+    put_byte(pb, strlen(descr)); /* string counter */
+    put_buffer(pb, descr, strlen(descr)); /* handler description */
+    return updateSize(pb, pos);
+}
+
 static int mov_write_minf_tag(ByteIOContext *pb, MOVTrack* track)
 {
     int pos = url_ftell(pb);
@@ -518,35 +623,13 @@ static int mov_write_minf_tag(ByteIOContext *pb, MOVTrack* track)
         mov_write_vmhd_tag(pb);
     else
         mov_write_smhd_tag(pb);
+    if (track->mode == MODE_MOV) /* FIXME: Why do it for MODE_MOV only ? */
+        mov_write_hdlr_tag(pb, NULL);
     mov_write_dinf_tag(pb);
     mov_write_stbl_tag(pb, track);
     return updateSize(pb, pos);
 }
 
-static int mov_write_hdlr_tag(ByteIOContext *pb, MOVTrack* track)
-{
-    char *str;
-    int pos = url_ftell(pb);
-    put_be32(pb, 0); /* size */
-    put_tag(pb, "hdlr");
-    put_be32(pb, 0); /* Version & flags */
-    put_be32(pb, 0); /* reserved */
-    if(track->enc->codec_type == CODEC_TYPE_VIDEO)
-        put_tag(pb, "vide"); /* handler type */
-    else
-        put_tag(pb, "soun"); /* handler type */
-    put_be32(pb ,0); /* reserved */
-    put_be32(pb ,0); /* reserved */
-    put_be32(pb ,0); /* reserved */
-    if(track->enc->codec_type == CODEC_TYPE_VIDEO)
-        str = "VideoHandler";
-    else
-        str = "SoundHandler";
-    put_byte(pb, strlen(str)); /* string counter */
-    put_buffer(pb, str, strlen(str));
-    return updateSize(pb, pos);
-}
-
 static int mov_write_mdhd_tag(ByteIOContext *pb, MOVTrack* track)
 {
     put_be32(pb, 32); /* size */
@@ -608,7 +691,9 @@ static int mov_write_tkhd_tag(ByteIOContext *pb, MOVTrack* track)
 
     /* Track width and height, for visual only */
     if(track->enc->codec_type == CODEC_TYPE_VIDEO) {
-        put_be32(pb, track->enc->width*0x10000);
+        double sample_aspect_ratio = av_q2d(track->enc->sample_aspect_ratio);
+        if( !sample_aspect_ratio ) sample_aspect_ratio = 1;
+        put_be32(pb, sample_aspect_ratio * track->enc->width*0x10000);
         put_be32(pb, track->enc->height*0x10000);
     }
     else {
@@ -691,7 +776,298 @@ static int mov_write_mvhd_tag(ByteIOContext *pb, MOVContext *mov)
     return 0x6c;
 }
 
-static int mov_write_moov_tag(ByteIOContext *pb, MOVContext *mov)
+static int mov_write_itunes_hdlr_tag(ByteIOContext *pb, MOVContext* mov,
+                                     AVFormatContext *s)
+{
+    int pos = url_ftell(pb);
+    put_be32(pb, 0); /* size */
+    put_tag(pb, "hdlr");
+    put_be32(pb, 0);
+    put_be32(pb, 0);
+    put_tag(pb, "mdir");
+    put_tag(pb, "appl");
+    put_be32(pb, 0);
+    put_be32(pb, 0);
+    put_be16(pb, 0);
+    return updateSize(pb, pos);
+}
+
+/* helper function to write a data tag with the specified string as data */
+static int mov_write_string_data_tag(ByteIOContext *pb, MOVContext* mov,
+                                     AVFormatContext *s, const char *data)
+{
+    int pos = url_ftell(pb);
+    put_be32(pb, 0); /* size */
+    put_tag(pb, "data");
+    put_be32(pb, 1);
+    put_be32(pb, 0);
+    put_buffer(pb, data, strlen(data));
+    return updateSize(pb, pos);
+}
+
+/* iTunes name of the song/movie */
+static int mov_write_nam_tag(ByteIOContext *pb, MOVContext* mov,
+                             AVFormatContext *s)
+{
+    int size = 0;
+    if ( s->title[0] ) {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, "\251nam");
+        mov_write_string_data_tag(pb, mov, s, s->title);
+        size = updateSize(pb, pos);
+    }
+    return size;
+}
+
+/* iTunes name of the artist/performer */
+static int mov_write_ART_tag(ByteIOContext *pb, MOVContext* mov,
+                             AVFormatContext *s)
+{
+    int size = 0;
+    if ( s->author[0] ) {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, "\251ART");
+        // we use the author here as this is the only thing that we have...
+        mov_write_string_data_tag(pb, mov, s, s->author);
+        size = updateSize(pb, pos);
+    }
+    return size;
+}
+
+/* iTunes name of the writer */
+static int mov_write_wrt_tag(ByteIOContext *pb, MOVContext* mov,
+                             AVFormatContext *s)
+{
+    int size = 0;
+    if ( s->author[0] ) {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, "\251wrt");
+        mov_write_string_data_tag(pb, mov, s, s->author);
+        size = updateSize(pb, pos);
+    }
+    return size;
+}
+
+/* iTunes name of the album */
+static int mov_write_alb_tag(ByteIOContext *pb, MOVContext* mov,
+                             AVFormatContext *s)
+{
+    int size = 0;
+    if ( s->album[0] ) {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, "\251alb");
+        mov_write_string_data_tag(pb, mov, s, s->album);
+        size = updateSize(pb, pos);
+    }
+    return size;
+}
+
+/* iTunes year */
+static int mov_write_day_tag(ByteIOContext *pb, MOVContext* mov,
+                             AVFormatContext *s)
+{
+    char year[5];
+    int size = 0;
+    if ( s->year ) {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, "\251day");
+        snprintf(year, 5, "%04d", s->year);
+        mov_write_string_data_tag(pb, mov, s, year);
+        size = updateSize(pb, pos);
+    }
+    return size;
+}
+
+/* iTunes tool used to create the file */
+static int mov_write_too_tag(ByteIOContext *pb, MOVContext* mov,
+                             AVFormatContext *s)
+{
+    int pos = url_ftell(pb);
+    put_be32(pb, 0); /* size */
+    put_tag(pb, "\251too");
+    mov_write_string_data_tag(pb, mov, s, LIBAVFORMAT_IDENT);
+    return updateSize(pb, pos);
+}
+
+/* iTunes comment */
+static int mov_write_cmt_tag(ByteIOContext *pb, MOVContext* mov,
+                             AVFormatContext *s)
+{
+    int size = 0;
+    if ( s->comment[0] ) {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, "\251cmt");
+        mov_write_string_data_tag(pb, mov, s, s->comment);
+        size = updateSize(pb, pos);
+    }
+    return size;
+}
+
+/* iTunes custom genre */
+static int mov_write_gen_tag(ByteIOContext *pb, MOVContext* mov,
+                             AVFormatContext *s)
+{
+    int size = 0;
+    if ( s->genre[0] ) {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, "\251gen");
+        mov_write_string_data_tag(pb, mov, s, s->genre);
+        size = updateSize(pb, pos);
+    }
+    return size;
+}
+
+/* iTunes track number */
+static int mov_write_trkn_tag(ByteIOContext *pb, MOVContext* mov,
+                              AVFormatContext *s)
+{
+    int size = 0;
+    if ( s->track ) {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, "trkn");
+        {
+            int pos = url_ftell(pb);
+            put_be32(pb, 0); /* size */
+            put_tag(pb, "data");
+            put_be32(pb, 0);        // 8 bytes empty
+            put_be32(pb, 0);
+            put_be16(pb, 0);        // empty
+            put_be16(pb, s->track); // track number
+            put_be16(pb, 0);        // total track number
+            put_be16(pb, 0);        // empty
+            updateSize(pb, pos);
+        }
+        size = updateSize(pb, pos);
+    }
+    return size;
+}
+
+/* iTunes meta data list */
+static int mov_write_ilst_tag(ByteIOContext *pb, MOVContext* mov,
+                              AVFormatContext *s)
+{
+    int pos = url_ftell(pb);
+    put_be32(pb, 0); /* size */
+    put_tag(pb, "ilst");
+    mov_write_nam_tag(pb, mov, s);
+    mov_write_ART_tag(pb, mov, s);
+    mov_write_wrt_tag(pb, mov, s);
+    mov_write_alb_tag(pb, mov, s);
+    mov_write_day_tag(pb, mov, s);
+    mov_write_too_tag(pb, mov, s);
+    mov_write_cmt_tag(pb, mov, s);
+    mov_write_gen_tag(pb, mov, s);
+    mov_write_trkn_tag(pb, mov, s);
+    return updateSize(pb, pos);
+}
+
+/* iTunes meta data tag */
+static int mov_write_meta_tag(ByteIOContext *pb, MOVContext* mov,
+                              AVFormatContext *s)
+{
+    int size = 0;
+
+    // only save meta tag if required
+    if ( s->title[0] || s->author[0] || s->album[0] || s->year || 
+         s->comment[0] || s->genre[0] || s->track ) {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, "meta");
+        put_be32(pb, 0);
+        mov_write_itunes_hdlr_tag(pb, mov, s);
+        mov_write_ilst_tag(pb, mov, s);
+        size = updateSize(pb, pos);
+    }
+    return size;
+}
+    
+static int mov_write_udta_tag(ByteIOContext *pb, MOVContext* mov,
+                              AVFormatContext *s)
+{
+    int pos = url_ftell(pb);
+    int i;
+
+    put_be32(pb, 0); /* size */
+    put_tag(pb, "udta");
+
+    /* iTunes meta data */
+    mov_write_meta_tag(pb, mov, s);
+
+    /* Requirements */
+    for (i=0; i<MAX_STREAMS; i++) {
+        if(mov->tracks[i].entry <= 0) continue;
+        if (mov->tracks[i].enc->codec_id == CODEC_ID_AAC ||
+            mov->tracks[i].enc->codec_id == CODEC_ID_MPEG4) {
+            int pos = url_ftell(pb);
+            put_be32(pb, 0); /* size */
+            put_tag(pb, "\251req");
+            put_be16(pb, sizeof("QuickTime 6.0 or greater") - 1);
+            put_be16(pb, 0);
+            put_buffer(pb, "QuickTime 6.0 or greater",
+                       sizeof("QuickTime 6.0 or greater") - 1);
+            updateSize(pb, pos);
+            break;
+        }
+    }
+
+    /* Encoder */
+    if(!(mov->tracks[0].enc->flags & CODEC_FLAG_BITEXACT))
+    {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, "\251enc");
+        put_be16(pb, sizeof(LIBAVFORMAT_IDENT) - 1); /* string length */
+        put_be16(pb, 0);
+        put_buffer(pb, LIBAVFORMAT_IDENT, sizeof(LIBAVFORMAT_IDENT) - 1);
+        updateSize(pb, pos);
+    }
+
+    if( s->title[0] )
+    {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, "\251nam");
+        put_be16(pb, strlen(s->title)); /* string length */
+        put_be16(pb, 0);
+        put_buffer(pb, s->title, strlen(s->title));
+        updateSize(pb, pos);
+    }
+
+    if( s->author[0] )
+    {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, /*"\251aut"*/ "\251day" );
+        put_be16(pb, strlen(s->author)); /* string length */
+        put_be16(pb, 0);
+        put_buffer(pb, s->author, strlen(s->author));
+        updateSize(pb, pos);
+    }
+
+    if( s->comment[0] )
+    {
+        int pos = url_ftell(pb);
+        put_be32(pb, 0); /* size */
+        put_tag(pb, "\251des");
+        put_be16(pb, strlen(s->comment)); /* string length */
+        put_be16(pb, 0);
+        put_buffer(pb, s->comment, strlen(s->comment));
+        updateSize(pb, pos);
+    }
+
+    return updateSize(pb, pos);
+}
+
+static int mov_write_moov_tag(ByteIOContext *pb, MOVContext *mov,
+                              AVFormatContext *s)
 {
     int pos, i;
     pos = url_ftell(pb);
@@ -732,6 +1108,8 @@ static int mov_write_moov_tag(ByteIOContext *pb, MOVContext *mov)
         }
     }
 
+    mov_write_udta_tag(pb, mov, s);
+
     return updateSize(pb, pos);
 }
 
@@ -746,17 +1124,19 @@ int mov_write_mdat_tag(ByteIOContext *pb, MOVContext* mov)
 /* TODO: This needs to be more general */
 int mov_write_ftyp_tag(ByteIOContext *pb, AVFormatContext *s)
 {
+    MOVContext *mov = s->priv_data;
+
     put_be32(pb, 0x14 ); /* size */
     put_tag(pb, "ftyp");
 
-    if (!strcmp("3gp", s->oformat->name))
+    if ( mov->mode == MODE_3GP )
         put_tag(pb, "3gp4");
     else
         put_tag(pb, "isom");
 
     put_be32(pb, 0x200 );
 
-    if (!strcmp("3gp", s->oformat->name))
+    if ( mov->mode == MODE_3GP )
         put_tag(pb, "3gp4");
     else
         put_tag(pb, "mp41");
@@ -767,10 +1147,42 @@ int mov_write_ftyp_tag(ByteIOContext *pb, AVFormatContext *s)
 static int mov_write_header(AVFormatContext *s)
 {
     ByteIOContext *pb = &s->pb;
+    MOVContext *mov = s->priv_data;
+    int i;
 
-    if(s->oformat != NULL) {
-    if(!strcmp("3gp", s->oformat->name) || !strcmp("mp4", s->oformat->name))
-        mov_write_ftyp_tag(pb,s);
+    for(i=0; i<s->nb_streams; i++){
+        AVCodecContext *c= &s->streams[i]->codec;
+
+        if      (c->codec_type == CODEC_TYPE_VIDEO){
+            if (!codec_get_tag(codec_movvideo_tags, c->codec_id)){
+                if(!codec_get_tag(codec_bmp_tags, c->codec_id))
+                    return -1;
+                else
+                    av_log(s, AV_LOG_INFO, "Warning, using MS style video codec tag, the file may be unplayable!\n");
+            }
+        }else if(c->codec_type == CODEC_TYPE_AUDIO){
+            if (!codec_get_tag(codec_movaudio_tags, c->codec_id)){
+                if(!codec_get_tag(codec_wav_tags, c->codec_id))
+                    return -1;
+                else
+                    av_log(s, AV_LOG_INFO, "Warning, using MS style audio codec tag, the file may be unplayable!\n");
+            }
+        }
+    }
+
+    /* Default mode == MP4 */
+    mov->mode = MODE_MP4;
+
+    if (s->oformat != NULL) {
+        if (!strcmp("3gp", s->oformat->name)) mov->mode = MODE_3GP;
+        else if (!strcmp("mov", s->oformat->name)) mov->mode = MODE_MOV;
+
+        if ( mov->mode == MODE_3GP || mov->mode == MODE_MP4 )
+            mov_write_ftyp_tag(pb,s);
+    }
+
+    for (i=0; i<MAX_STREAMS; i++) {
+        mov->tracks[i].mode = mov->mode;
     }
 
     put_flush_packet(pb);
@@ -778,19 +1190,15 @@ static int mov_write_header(AVFormatContext *s)
     return 0;
 }
 
-static int Timestamp() {
-    return 1067949799U+(24107*86400); //its the modification time of this line :)
-}
-
-static int mov_write_packet(AVFormatContext *s, int stream_index,
-                            const uint8_t *buf, int size, int64_t pts)
+static int mov_write_packet(AVFormatContext *s, AVPacket *pkt)
 {
     MOVContext *mov = s->priv_data;
     ByteIOContext *pb = &s->pb;
-    AVCodecContext *enc = &s->streams[stream_index]->codec;
-    MOVTrack* trk = &mov->tracks[stream_index];
+    AVCodecContext *enc = &s->streams[pkt->stream_index]->codec;
+    MOVTrack* trk = &mov->tracks[pkt->stream_index];
     int cl, id;
     unsigned int samplesInChunk = 0;
+    int size= pkt->size;
 
     if (url_is_streamed(&s->pb)) return 0; /* Can't handle that */
     if (!size) return 0; /* Discard 0 sized packets */
@@ -806,13 +1214,16 @@ static int mov_write_packet(AVFormatContext *s, int stream_index,
             int len = 0;
 
             while (len < size && samplesInChunk < 100) {
-                len += packed_size[(buf[len] >> 3) & 0x0F];
+                len += packed_size[(pkt->data[len] >> 3) & 0x0F];
                 samplesInChunk++;
             }
         }
         else if(enc->codec_id == CODEC_ID_PCM_ALAW) {
             samplesInChunk = size/enc->channels;
         }
+	else if(enc->codec_id == CODEC_ID_PCM_S16BE || enc->codec_id == CODEC_ID_PCM_S16LE) {
+	    samplesInChunk = size/(2*enc->channels);
+        }	    
         else {
             samplesInChunk = 1;
         }
@@ -820,7 +1231,7 @@ static int mov_write_packet(AVFormatContext *s, int stream_index,
 
     if ((enc->codec_id == CODEC_ID_MPEG4 || enc->codec_id == CODEC_ID_AAC)
         && trk->vosLen == 0) {
-        assert(enc->extradata_size);
+//        assert(enc->extradata_size);
 
         trk->vosLen = enc->extradata_size;
         trk->vosData = av_malloc(trk->vosLen);
@@ -842,16 +1253,16 @@ static int mov_write_packet(AVFormatContext *s, int stream_index,
     if (mov->mdat_written == 0) {
         mov_write_mdat_tag(pb, mov);
         mov->mdat_written = 1;
-        mov->time = Timestamp();
+        mov->time = s->timestamp;
     }
 
-    trk->cluster[cl][id].pos = url_ftell(pb) - mov->movi_list;
+    trk->cluster[cl][id].pos = url_ftell(pb);
     trk->cluster[cl][id].samplesInChunk = samplesInChunk;
     trk->cluster[cl][id].size = size;
     trk->cluster[cl][id].entries = samplesInChunk;
     if(enc->codec_type == CODEC_TYPE_VIDEO) {
-        trk->cluster[cl][id].key_frame = enc->coded_frame->key_frame;
-        if(enc->coded_frame->pict_type == FF_I_TYPE)
+        trk->cluster[cl][id].key_frame = !!(pkt->flags & PKT_FLAG_KEY);
+        if(trk->cluster[cl][id].key_frame)
             trk->hasKeyframes = 1;
     }
     trk->enc = enc;
@@ -859,7 +1270,7 @@ static int mov_write_packet(AVFormatContext *s, int stream_index,
     trk->sampleCount += samplesInChunk;
     trk->mdat_size += size;
 
-    put_buffer(pb, buf, size);
+    put_buffer(pb, pkt->data, size);
 
     put_flush_packet(pb);
     return 0;
@@ -871,22 +1282,20 @@ static int mov_write_trailer(AVFormatContext *s)
     ByteIOContext *pb = &s->pb;
     int res = 0;
     int i, j;
-    offset_t file_size;
 
-    file_size = url_ftell(pb);
-    j = 0;
+    offset_t moov_pos = url_ftell(pb);
 
     /* Write size of mdat tag */
-    for (i=0; i<MAX_STREAMS; i++) {
+    for (i=0, j=0; i<MAX_STREAMS; i++) {
         if(mov->tracks[i].ents_allocated > 0) {
             j += mov->tracks[i].mdat_size;
         }
     }
     url_fseek(pb, mov->mdat_pos, SEEK_SET);
     put_be32(pb, j+8);
-    url_fseek(pb, file_size, SEEK_SET);
+    url_fseek(pb, moov_pos, SEEK_SET);
 
-    mov_write_moov_tag(pb, mov);
+    mov_write_moov_tag(pb, mov, s);
 
     for (i=0; i<MAX_STREAMS; i++) {
         for (j=0; j<mov->tracks[i].ents_allocated/MOV_INDEX_CLUSTER_SIZE; j++) {
@@ -898,6 +1307,7 @@ static int mov_write_trailer(AVFormatContext *s)
         mov->tracks[i].cluster = NULL;
         mov->tracks[i].ents_allocated = mov->tracks[i].entry = 0;
     }
+
     put_flush_packet(pb);
 
     return res;
@@ -909,7 +1319,7 @@ static AVOutputFormat mov_oformat = {
     NULL,
     "mov",
     sizeof(MOVContext),
-    CODEC_ID_PCM_ALAW,
+    CODEC_ID_AAC,
     CODEC_ID_MPEG4,
     mov_write_header,
     mov_write_packet,
