@@ -432,6 +432,7 @@ static void pmt_cb(void *opaque, const uint8_t *section, int section_len)
         case STREAM_TYPE_VIDEO_H264:
         case STREAM_TYPE_AUDIO_AAC:
         case STREAM_TYPE_AUDIO_AC3:
+        case STREAM_TYPE_AUDIO_DTS:
             add_pes_stream(ts, pid, stream_type);
             break;
         default:
@@ -757,6 +758,10 @@ static void mpegts_push_data(void *opaque,
                             codec_type = CODEC_TYPE_AUDIO;
                             codec_id = CODEC_ID_AC3;
                             break;
+                        case STREAM_TYPE_AUDIO_DTS:
+                            codec_type = CODEC_TYPE_AUDIO;
+                            codec_id = CODEC_ID_DTS;
+                            break;
                         default:
                             if (code >= 0x1c0 && code <= 0x1df) {
                                 codec_type = CODEC_TYPE_AUDIO;
@@ -772,6 +777,7 @@ static void mpegts_push_data(void *opaque,
                         }
                         st = av_new_stream(pes->stream, pes->pid);
                         if (st) {
+                            av_set_pts_info(st, 60, 1, 90000);
                             st->priv_data = pes;
                             st->codec.codec_type = codec_type;
                             st->codec.codec_id = codec_id;
@@ -1086,7 +1092,7 @@ static int mpegts_read_header(AVFormatContext *s,
     MpegTSContext *ts = s->priv_data;
     ByteIOContext *pb = &s->pb;
     uint8_t buf[1024];
-    int len, sid;
+    int len, sid, i;
     int64_t pos;
     MpegTSService *service;
     
@@ -1141,19 +1147,21 @@ static int mpegts_read_header(AVFormatContext *s,
            }
             
             /* tune to first service found */
-            service = ts->services[0];
-            sid = service->sid;
+            for(i=0; i<ts->nb_services && ts->set_service_ret; i++){
+                service = ts->services[0];
+                sid = service->sid;
 #ifdef DEBUG_SI
-            printf("tuning to '%s'\n", service->name);
+                printf("tuning to '%s'\n", service->name);
 #endif
             
-            /* now find the info for the first service if we found any,
-               otherwise try to filter all PATs */
+                /* now find the info for the first service if we found any,
+                otherwise try to filter all PATs */
             
-            url_fseek(pb, pos, SEEK_SET);
-            mpegts_set_service(ts, sid, set_service_cb, ts);
+                url_fseek(pb, pos, SEEK_SET);
+                mpegts_set_service(ts, sid, set_service_cb, ts);
             
-            handle_packets(ts, MAX_SCAN_PACKETS);
+                handle_packets(ts, MAX_SCAN_PACKETS);
+            }
             
             /* if could not find service, exit */
             if (ts->set_service_ret != 0)
@@ -1172,14 +1180,11 @@ static int mpegts_read_header(AVFormatContext *s,
         uint8_t packet[TS_PACKET_SIZE];
         
         /* only read packets */
-
-        s->pts_num = 1;
-        s->pts_den = 27000000;
-       
     do_pcr: 
         st = av_new_stream(s, 0);
         if (!st)
             goto fail;
+        av_set_pts_info(st, 60, 1, 27000000);
         st->codec.codec_type = CODEC_TYPE_DATA;
         st->codec.codec_id = CODEC_ID_MPEG2TS;
         
@@ -1292,14 +1297,15 @@ static int mpegts_read_close(AVFormatContext *s)
 }
 
 static int64_t mpegts_get_pcr(AVFormatContext *s, int stream_index, 
-                              int64_t *ppos, int find_next)
+                              int64_t *ppos, int64_t pos_limit)
 {
     MpegTSContext *ts = s->priv_data;
     int64_t pos, timestamp;
     uint8_t buf[TS_PACKET_SIZE];
     int pcr_l, pid;
 
-    pos = *ppos;
+    const int find_next= 1;
+    pos = ((*ppos  + ts->raw_packet_size - 1) / ts->raw_packet_size) * ts->raw_packet_size;
     if (find_next) {
         for(;;) {
             url_fseek(&s->pb, pos, SEEK_SET);
@@ -1331,114 +1337,27 @@ static int64_t mpegts_get_pcr(AVFormatContext *s, int stream_index,
     return timestamp;
 }
 
-typedef int64_t ReadTimestampFunc(AVFormatContext *s, int stream_index, 
-                                  int64_t *ppos, int find_next);
-
-static int64_t do_block_align(int64_t val, int block_align)
-{
-    return (val / block_align) * block_align;
-}
-
-/* XXX: use it in other formats */
-static int timestamp_read_seek(AVFormatContext *s, 
-                               int stream_index, int64_t timestamp,
-                               ReadTimestampFunc *read_timestamp,
-                               int block_align)
-{
-    int64_t pos_min, pos_max, pos;
-    int64_t dts_min, dts_max, dts;
-
-#ifdef DEBUG_SEEK
-    printf("read_seek: %d %0.3f\n", stream_index, timestamp / 90000.0);
-#endif
-
-    pos_min = 0;
-    dts_min = read_timestamp(s, stream_index, &pos_min, 1);
-    if (dts_min == AV_NOPTS_VALUE) {
-        /* we can reach this case only if no PTS are present in
-           the whole stream */
-        return -1;
-    }
-    pos_max = do_block_align(url_filesize(url_fileno(&s->pb)), block_align) - 
-        block_align;
-    dts_max = read_timestamp(s, stream_index, &pos_max, 0);
-    
-    while (pos_min <= pos_max) {
-#ifdef DEBUG_SEEK
-        printf("pos_min=0x%llx pos_max=0x%llx dts_min=%0.3f dts_max=%0.3f\n", 
-               pos_min, pos_max,
-               dts_min / 90000.0, dts_max / 90000.0);
-#endif
-        if (timestamp <= dts_min) {
-            pos = pos_min;
-            goto found;
-        } else if (timestamp >= dts_max) {
-            pos = pos_max;
-            goto found;
-        } else {
-            /* interpolate position (better than dichotomy) */
-            pos = (int64_t)((double)(pos_max - pos_min) * 
-                            (double)(timestamp - dts_min) /
-                            (double)(dts_max - dts_min)) + pos_min;
-            pos = do_block_align(pos, block_align);
-        }
-#ifdef DEBUG_SEEK
-        printf("pos=0x%llx\n", pos);
-#endif
-        /* read the next timestamp */
-        dts = read_timestamp(s, stream_index, &pos, 1);
-        /* check if we are lucky */
-        if (dts == AV_NOPTS_VALUE) {
-            /* should never happen */
-            pos = pos_min;
-            goto found;
-        } else if (timestamp == dts) {
-            goto found;
-        } else if (timestamp < dts) {
-            pos_max = pos;
-            dts_max = read_timestamp(s, stream_index, &pos_max, 0);
-            if (dts_max == AV_NOPTS_VALUE) {
-                /* should never happen */
-                break;
-            } else if (timestamp >= dts_max) {
-                pos = pos_max;
-                goto found;
-            }
-        } else {
-            pos_min = pos + block_align;
-            dts_min = read_timestamp(s, stream_index, &pos_min, 1);
-            if (dts_min == AV_NOPTS_VALUE) {
-                /* should never happen */
-                goto found;
-            } else if (timestamp <= dts_min) {
-                goto found;
-            }
-        }
-    }
-    pos = pos_min;
- found:
-#ifdef DEBUG_SEEK
-    pos_min = pos;
-    dts_min = read_timestamp(s, stream_index, &pos_min, 1);
-    pos_min += block_align;
-    dts_max = read_timestamp(s, stream_index, &pos_min, 1);
-    printf("pos=0x%llx %0.3f<=%0.3f<=%0.3f\n", 
-           pos, dts_min / 90000.0, timestamp / 90000.0, dts_max / 90000.0);
-#endif
-    /* do the seek */
-    url_fseek(&s->pb, pos, SEEK_SET);
-    return 0;
-}
-
-static int mpegts_read_seek(AVFormatContext *s, 
-                            int stream_index, int64_t timestamp)
-{
+static int read_seek(AVFormatContext *s, int stream_index, int64_t target_ts){
     MpegTSContext *ts = s->priv_data;
+    uint8_t buf[TS_PACKET_SIZE];
+    int64_t pos;
 
-    timestamp = (timestamp * 90000) / AV_TIME_BASE;
+    if(av_seek_frame_binary(s, stream_index, target_ts) < 0)
+        return -1;
 
-    return timestamp_read_seek(s, stream_index, timestamp, 
-                               mpegts_get_pcr, ts->raw_packet_size);
+    pos= url_ftell(&s->pb);
+
+    for(;;) {
+        url_fseek(&s->pb, pos, SEEK_SET);
+        if (get_buffer(&s->pb, buf, TS_PACKET_SIZE) != TS_PACKET_SIZE)
+            return -1;
+//        pid = ((buf[1] & 0x1f) << 8) | buf[2];
+        if(buf[1] & 0x40) break;
+        pos += ts->raw_packet_size;
+    }
+    url_fseek(&s->pb, pos, SEEK_SET);
+
+    return 0;
 }
 
 /**************************************************************/
@@ -1503,7 +1422,8 @@ AVInputFormat mpegts_demux = {
     mpegts_read_header,
     mpegts_read_packet,
     mpegts_read_close,
-    mpegts_read_seek,
+    read_seek,
+    mpegts_get_pcr,
     .flags = AVFMT_SHOW_IDS,
 };
 
