@@ -29,13 +29,18 @@ const CodecTag codec_wav_tags[] = {
     { CODEC_ID_PCM_MULAW, 0x07 },
     { CODEC_ID_ADPCM_MS, 0x02 },
     { CODEC_ID_ADPCM_IMA_WAV, 0x11 },
+    { CODEC_ID_ADPCM_G726, 0x45 },
     { CODEC_ID_ADPCM_IMA_DK4, 0x61 },  /* rogue format number */
     { CODEC_ID_ADPCM_IMA_DK3, 0x62 },  /* rogue format number */
     { CODEC_ID_WMAV1, 0x160 },
     { CODEC_ID_WMAV2, 0x161 },
+    { CODEC_ID_VORBIS, ('V'<<8)+'o' }, //HACK/FIXME, does vorbis in WAV/AVI have an (in)official id?
+    { CODEC_ID_SONIC, 0x2048 },
+    { CODEC_ID_SONIC_LS, 0x2048 },
     { 0, 0 },
 };
 
+#ifdef CONFIG_ENCODERS
 /* WAVEFORMATEX header */
 /* returns the size or -1 on error */
 int put_wav_header(ByteIOContext *pb, AVCodecContext *enc)
@@ -100,12 +105,21 @@ int put_wav_header(ByteIOContext *pb, AVCodecContext *enc)
         put_le32(pb, 0);  /* dwPTSHigh */
     } else if (enc->codec_id == CODEC_ID_ADPCM_IMA_WAV) {
         put_le16(pb, 2); /* wav_extra_size */
+        hdrsize += 2;
         put_le16(pb, ((enc->block_align - 4 * enc->channels) / (4 * enc->channels)) * 8 + 1); /* wSamplesPerBlock */
-    } else
-        put_le16(pb, 0); /* wav_extra_size */
+    } else {
+        put_le16(pb, enc->extradata_size);
+        put_buffer(pb, enc->extradata, enc->extradata_size);
+        hdrsize += enc->extradata_size;
+        if(hdrsize&1){
+            hdrsize++;
+            put_byte(pb, 0);
+        }
+    }
 
     return hdrsize;
 }
+#endif //CONFIG_ENCODERS
 
 /* We could be given one of the three possible structures here:
  * WAVEFORMAT, PCMWAVEFORMAT or WAVEFORMATEX. Each structure
@@ -130,7 +144,7 @@ void get_wav_header(ByteIOContext *pb, AVCodecContext *codec, int size)
     }else
         codec->bits_per_sample = get_le16(pb);
     codec->codec_id = wav_codec_get_id(id, codec->bits_per_sample);
-
+    
     if (size > 16) {  /* We're obviously dealing with WAVEFORMATEX */
 	codec->extradata_size = get_le16(pb);
 	if (codec->extradata_size > 0) {
@@ -160,6 +174,7 @@ int wav_codec_get_id(unsigned int tag, int bps)
     return id;
 }
 
+#ifdef CONFIG_ENCODERS
 typedef struct {
     offset_t data;
 } WAVContext;
@@ -190,11 +205,10 @@ static int wav_write_header(AVFormatContext *s)
     return 0;
 }
 
-static int wav_write_packet(AVFormatContext *s, int stream_index_ptr,
-                            const uint8_t *buf, int size, int64_t pts)
+static int wav_write_packet(AVFormatContext *s, AVPacket *pkt)
 {
     ByteIOContext *pb = &s->pb;
-    put_buffer(pb, buf, size);
+    put_buffer(pb, pkt->data, pkt->size);
     return 0;
 }
 
@@ -217,6 +231,7 @@ static int wav_write_trailer(AVFormatContext *s)
     }
     return 0;
 }
+#endif //CONFIG_ENCODERS
 
 /* return the size of the found tag */
 /* XXX: > 2GB ? */
@@ -281,6 +296,7 @@ static int wav_read_header(AVFormatContext *s,
         return AVERROR_NOMEM;
 
     get_wav_header(pb, &st->codec, size);
+    st->need_parsing = 1;
     
     size = find_tag(pb, MKTAG('d', 'a', 't', 'a'));
     if (size < 0)
@@ -293,12 +309,21 @@ static int wav_read_header(AVFormatContext *s,
 static int wav_read_packet(AVFormatContext *s,
                            AVPacket *pkt)
 {
-    int ret;
+    int ret, size;
+    AVStream *st;
 
     if (url_feof(&s->pb))
-        return -EIO;
-    if (av_new_packet(pkt, MAX_SIZE, 0))
-        return -EIO;
+        return AVERROR_IO;
+    st = s->streams[0];
+
+    size = MAX_SIZE;
+    if (st->codec.block_align > 1) {
+        if (size < st->codec.block_align)
+            size = st->codec.block_align;
+        size = (size / st->codec.block_align) * st->codec.block_align;
+    }
+    if (av_new_packet(pkt, size))
+        return AVERROR_IO;
     pkt->stream_index = 0;
 
     ret = get_buffer(&s->pb, pkt->data, pkt->size);
@@ -315,6 +340,26 @@ static int wav_read_close(AVFormatContext *s)
     return 0;
 }
 
+static int wav_read_seek(AVFormatContext *s, 
+                         int stream_index, int64_t timestamp)
+{
+    AVStream *st;
+
+    st = s->streams[0];
+    switch(st->codec.codec_id) {
+    case CODEC_ID_MP2:
+    case CODEC_ID_MP3:
+    case CODEC_ID_AC3:
+    case CODEC_ID_DTS:
+        /* use generic seeking with dynamically generated indexes */
+        return -1;
+    default:
+        break;
+    }
+    return pcm_read_seek(s, stream_index, timestamp);
+}
+
+
 static AVInputFormat wav_iformat = {
     "wav",
     "wav format",
@@ -323,8 +368,10 @@ static AVInputFormat wav_iformat = {
     wav_read_header,
     wav_read_packet,
     wav_read_close,
+    wav_read_seek,
 };
 
+#ifdef CONFIG_ENCODERS
 static AVOutputFormat wav_oformat = {
     "wav",
     "wav format",
@@ -337,10 +384,13 @@ static AVOutputFormat wav_oformat = {
     wav_write_packet,
     wav_write_trailer,
 };
+#endif //CONFIG_ENCODERS
 
-int wav_init(void)
+int ff_wav_init(void)
 {
     av_register_input_format(&wav_iformat);
+#ifdef CONFIG_ENCODERS
     av_register_output_format(&wav_oformat);
+#endif //CONFIG_ENCODERS
     return 0;
 }
