@@ -63,7 +63,7 @@ void AudioOutputOSS::Reconfigure(int laudio_bits, int laudio_channels,
     audio_channels = laudio_channels;
     audio_bits = laudio_bits;
     audio_samplerate = laudio_samplerate;
-    if(audio_bits != 8 && audio_bits != 16)
+    if (audio_bits != 8 && audio_bits != 16)
     {
         Error("AudioOutputOSS only supports 8 or 16bit audio.");
         return;
@@ -150,7 +150,7 @@ void AudioOutputOSS::Reconfigure(int laudio_bits, int laudio_channels,
                                  .arg(fragment_size));
 
     audio_buffer_unused = info.bytes - (fragment_size * 4);
-    if(audio_buffer_unused < 0)
+    if (audio_buffer_unused < 0)
        audio_buffer_unused = 0;
 
     if (!gContext->GetNumSetting("AggressiveSoundcardBuffer", 0))
@@ -424,6 +424,9 @@ void AudioOutputOSS::AddSamples(char *buffers[], int samples,
         {
             VERBOSE(VB_IMPORTANT, "Audio buffer overflow, audio data lost!");
             samples = afree / audio_bytes_per_sample;
+            pthread_mutex_unlock(&audio_buflock);
+            Reset();
+            return;
         }
     }
     
@@ -432,7 +435,7 @@ void AudioOutputOSS::AddSamples(char *buffers[], int samples,
         for(int chan = 0; chan < audio_channels; chan++)
         {
             audiobuffer[waud++] = buffers[chan][itemp];
-            if(audio_bits == 16)
+            if (audio_bits == 16)
                 audiobuffer[waud++] = buffers[chan][itemp+1];
             
             if (waud >= AUDBUFSIZE)
@@ -474,6 +477,9 @@ void AudioOutputOSS::AddSamples(char *buffer, int samples, long long timecode)
         {
             VERBOSE(VB_IMPORTANT, "Audio buffer overflow, audio data lost!");
             len = afree;
+            pthread_mutex_unlock(&audio_buflock);
+            Reset();
+            return;
         }
     }
 
@@ -547,7 +553,7 @@ void AudioOutputOSS::OutputAudioLoop(void)
             // should this use ioctl(audio_fd, SNDCTL_DSP_POST, 0) instead ?
             
             space_on_soundcard = getSpaceOnSoundcard();
-            if (fragment_size < space_on_soundcard)
+            if (fragment_size <= space_on_soundcard)
             {
                 WriteAudio(zeros, fragment_size);
             }
@@ -567,7 +573,7 @@ void AudioOutputOSS::OutputAudioLoop(void)
         /* do audio output */
         
         // wait for the buffer to fill with enough to play
-        if (fragment_size >= audiolen(true))
+        if (fragment_size > audiolen(true))
         {
             VERBOSE(VB_AUDIO, QString("audio thread waiting for buffer to fill"
                                       " fragment_size=%1, audiolen=%2")
@@ -601,7 +607,7 @@ void AudioOutputOSS::OutputAudioLoop(void)
 
         // re-check audiolen() in case things changed.
         // for example, ClearAfterSeek() might have run
-        if (fragment_size < audiolen(false))
+        if (fragment_size <= audiolen(false))
         {
             int bdiff = AUDBUFSIZE - raud;
             if (fragment_size > bdiff)
