@@ -134,7 +134,7 @@ void LCD::sendToServer(const QString &someText)
     // Check the socket, make sure the connection is still up
     if (socket->state() == QSocket::Idle)
     {
-        if(!lcd_ready)
+        if (!lcd_ready)
             return;
 
         lcd_ready = false;
@@ -154,7 +154,7 @@ void LCD::sendToServer(const QString &someText)
    
     last_command = someText;
  
-    if(connected)
+    if (connected)
     {
 #ifdef LCD_DEVICE_DEBUG
         cout << "lcddevice: Sending to Server: " << someText << endl ;
@@ -313,6 +313,7 @@ void LCD::init()
     lcd_showgeneric=(gContext->GetSetting("LCDShowGeneric", "1")=="1");
     lcd_showvolume=(gContext->GetSetting("LCDShowVolume", "1")=="1");
     lcd_showmenu=(gContext->GetSetting("LCDShowMenu", "1")=="1");
+    lcd_backlighton=(gContext->GetSetting("LCDBacklightOn", "1")=="1");
 
     connected = TRUE;
 
@@ -329,6 +330,10 @@ void LCD::init()
     sendToServer("screen_add Time");
     sendToServer("widget_del Time heartbeat");
     sendToServer("screen_set Time priority 255");
+    if (lcd_backlighton)
+        sendToServer("screen_set Time backlight on");
+    else
+        sendToServer("screen_set Time backlight off");
     sendToServer("widget_add Time timeWidget string");
     sendToServer("widget_add Time topWidget string");
 
@@ -339,6 +344,7 @@ void LCD::init()
     sendToServer("screen_add Menu");
     sendToServer("widget_del Menu heartbeat");
     sendToServer("screen_set Menu priority 255");
+    sendToServer("screen_set Menu backlight on");
     sendToServer("widget_add Menu topWidget string");
     sendToServer("widget_add Menu menuWidget1 string");
     sendToServer("widget_add Menu menuWidget2 string");
@@ -351,6 +357,7 @@ void LCD::init()
     sendToServer("screen_add Music");
     sendToServer("widget_del Music heartbeat");
     sendToServer("screen_set Music priority 255");
+    sendToServer("screen_set Music backlight on");
     sendToServer("widget_add Music topWidget string");
     
     // Have to put in 10 bars for equalizer
@@ -368,7 +375,9 @@ void LCD::init()
     sendToServer("screen_add Channel");
     sendToServer("widget_del Channel heartbeat");
     sendToServer("screen_set Channel priority 255");
+    sendToServer("screen_set Channel backlight on");
     sendToServer("widget_add Channel topWidget string");
+    sendToServer("widget_add Channel botWidget string");
     sendToServer("widget_add Channel progressBar hbar");
    
     // The Generic Screen
@@ -376,6 +385,7 @@ void LCD::init()
     sendToServer("screen_add Generic");
     sendToServer("widget_del Generic heartbeat");
     sendToServer("screen_set Generic priority 255");
+    sendToServer("screen_set Generic backlight on");
     sendToServer("widget_add Generic textWidget1 string");
     sendToServer("widget_add Generic textWidget2 string");
     sendToServer("widget_add Generic textWidget3 string");
@@ -386,21 +396,18 @@ void LCD::init()
     sendToServer("screen_add Volume");
     sendToServer("widget_del Volume heartbeat");
     sendToServer("screen_set Volume priority 255");
+    sendToServer("screen_set Volume backlight on");
     sendToServer("widget_add Volume topWidget string");
     sendToServer("widget_add Volume botWidget string");
     sendToServer("widget_add Volume progressBar hbar");
 
-    // Turn on the backlight
-
-    sendToServer("backlight 255");
-    
     lcd_ready = true;
  
     switchToTime();    // clock is on by default
     
     // send buffer if there's anything in there
     
-    if(send_buffer.length() > 0)
+    if (send_buffer.length() > 0)
     {
         sendToServer(send_buffer);
         send_buffer = "";
@@ -511,8 +518,7 @@ void LCD::stopAll()
 
 void LCD::startTime()
 {
-    if (lcd_showtime)
-        sendToServer("screen_set Time priority 128");
+    sendToServer("screen_set Time priority 128");
     timeTimer->start(1000, FALSE);
     outputTime();
     theMode = 0;    
@@ -687,20 +693,23 @@ void LCD::startChannel(QString channum, QString title, QString subtitle)
     QString aString;
     if (lcd_showchannel)
       sendToServer("screen_set Channel priority 64");
-    channelTimer->start(500, FALSE);
     aString = channum;
-    aString += ": ";
-    aString += title;
-    if(subtitle.length() > 0)
-    {
-        aString += " (";
-        aString += subtitle;
-        aString += ")";
-    }
     theMode = 2;
-    assignScrollingText(aString);
-    progress = 0.0;
-    outputChannel();
+    assignScrollingText(aString, "topWidget", 1);
+    aString = title;
+    if (subtitle.length() > 0)
+    {
+        aString += " - '";
+        aString += subtitle;
+        aString += "'";
+    }
+    assignScrollingText(aString, "botWidget", 2);
+    if (lcdHeight > 2)
+    {
+        progress = 0.0;
+        channelTimer->start(500, FALSE);
+        outputChannel();
+    }
 }
 
 void LCD::startGeneric(QPtrList<LCDTextItem> * textItems)
@@ -1252,15 +1261,18 @@ void LCD::outputTime()
     aString += " ";
     aString += QString::number(y);
     aString += " \"";
-    aString += QTime::currentTime().toString(timeformat).left(8) + "\"";
-    if (timeFlash)
-    {
-        aString = aString.replace(QRegExp(":"), " ");
-        timeFlash = false;
+    if (lcd_showtime) {
+        aString += QTime::currentTime().toString(timeformat).left(8) + "\"";
+        if (timeFlash)
+        {
+            aString = aString.replace(QRegExp(":"), " ");
+            timeFlash = false;
+        }
+        else
+            timeFlash = true;
     }
     else
-        timeFlash = true;
-
+        aString += " \"";
     sendToServer(aString);
 }
 
@@ -1435,7 +1447,7 @@ LCD::~LCD()
     cout << "lcddevice: An LCD device is being snuffed out of existence "
             "(~LCD() was called)" << endl ;
 #endif
-    if(socket)
+    if (socket)
     {
         delete socket;
         lcd_ready = false;
