@@ -13,6 +13,7 @@
 #include <qhostaddress.h>
 
 #include <cmath>
+#include <queue>
 
 #include "mythcontext.h"
 #include "oldsettings.h"
@@ -24,6 +25,7 @@
 #include "mythdialogs.h"
 #include "mythplugin.h"
 #include "screensaver.h"
+#include "DisplayRes.h"
 
 MythContext *gContext = NULL;
 
@@ -91,6 +93,13 @@ class MythContextPrivate
     int m_logenable, m_logmaxcount, m_logprintlevel;
     QMap<QString,int> lastLogCounts;
     QMap<QString,QString> lastLogStrings;
+
+    bool screensaverEnabled;
+
+    DisplayRes *display_res;
+
+    QMutex *m_priv_mutex;
+    queue<MythPrivRequest> m_priv_requests;
 };
 
 MythContextPrivate::MythContextPrivate(MythContext *lparent)
@@ -122,13 +131,14 @@ void MythContextPrivate::Init(bool gui, bool lcd)
         cerr << "Unable to read configuration file mysql.txt" << endl;
 
     m_localhostname = m_settings->GetSetting("LocalHostName", NULL);
-    if(m_localhostname == NULL)
+    if (m_localhostname == NULL)
     {
         char localhostname[1024];
         if (gethostname(localhostname, 1024))
         {
             perror("gethostname");
-            exit(1);
+            VERBOSE(VB_IMPORTANT, "MCP: Could not determine host name, exiting");
+            exit(-29);
         }
         m_localhostname = localhostname;
     }
@@ -168,6 +178,10 @@ void MythContextPrivate::Init(bool gui, bool lcd)
         lcd_device = NULL;
 
     disablelibrarypopup = false;
+
+    display_res = NULL;
+
+    m_priv_mutex = new QMutex(true);
 }
 
 MythContextPrivate::~MythContextPrivate()
@@ -184,6 +198,8 @@ MythContextPrivate::~MythContextPrivate()
         delete eventSock;
     if (lcd_device)
         delete lcd_device;
+    if (m_priv_mutex)
+        delete m_priv_mutex;
 }
 
 void MythContextPrivate::LoadLogSettings(void)
@@ -200,12 +216,12 @@ MythContext::MythContext(const QString &binversion, bool gui, bool lcd)
 
     if (binversion != MYTH_BINARY_VERSION)
     {
-        cerr << "This app was compiled against libmyth version: " << binversion
-             << "\nbut the library is version: " << MYTH_BINARY_VERSION << endl;
-        cerr << "You probably want to recompile everything, and do a\n"
-             << "'make distclean' first.\n";
-        cerr << "exiting\n";
-        exit(1);
+        VERBOSE(VB_IMPORTANT, QString("This app was compiled against libmyth version: %1")
+                .arg(binversion));
+        VERBOSE(VB_IMPORTANT, QString("but the library is version: %1").arg(MYTH_BINARY_VERSION));
+        VERBOSE(VB_IMPORTANT, "You probably want to recompile everything, and do a");
+        VERBOSE(VB_IMPORTANT, "'make distclean' first. exiting.");
+        exit(-30);
     }
 
     d = new MythContextPrivate(this);
@@ -372,7 +388,18 @@ void MythContext::LoadQtConfig(void)
     d->m_width = GetNumSetting("GuiWidth", 0);
     d->m_height = GetNumSetting("GuiHeight", 0);
 
-    if (d->m_width <= 0 || d->m_height <= 0)
+    if ((d->display_res = DisplayRes::getDisplayRes()))
+    {
+        // Make sure DisplayRes has current context info
+        d->display_res->Initialize();
+
+        // Switch to desired GUI resolution
+        d->display_res->switchToGUI();
+
+        d->m_width = d->display_res->Width();
+        d->m_height = d->display_res->Height();
+    }
+    else if (d->m_width <= 0 || d->m_height <= 0)
     {
         d->m_height = QApplication::desktop()->height();
         d->m_width = QApplication::desktop()->width();
@@ -633,7 +660,7 @@ void MythContext::InitializeScreenSettings()
 {
     int x = 0, y = 0, w = 0, h = 0;
 
-#ifndef QWS
+#if defined(Q_WS_X11)
     GetMythTVGeometry(qt_xdisplay(), qt_xscreen(), &x, &y, &w, &h);
 #endif
 
@@ -643,18 +670,28 @@ void MythContext::InitializeScreenSettings()
     int height = GetNumSetting("GuiHeight", d->m_height);
     int width = GetNumSetting("GuiWidth", d->m_width);
 
-    if (w != 0 && h != 0 && height == 0 && width == 0)
+    if (width == 0 && height == 0 && d->display_res)
     {
-        width = w;
-        height = h;
+        // If using custom, full-screen resolution, note the size
+        height = d->display_res->Height();
+        width  = d->display_res->Width();
     }
     else
     {
-        if (height == 0)
-            height = d->m_height;
-        if (width == 0)
-            width = d->m_width;
+        if (w != 0 && h != 0 && height == 0 && width == 0)
+        {
+            width = w;
+            height = h;
+        }
+        else
+        {
+            if (height == 0)
+                height = d->m_height;
+            if (width == 0)
+                width = d->m_width;
+        }
     }
+
     
     if (height < 160 || width < 160)
     {
@@ -699,7 +736,7 @@ QString MythContext::FindThemeDir(const QString &themename)
 
 
     // Don't complain about the "default" theme being missing
-    if(themename == QObject::tr("Default"))
+    if (themename == QObject::tr("Default"))
     {
         testdir = d->m_installprefix + "/share/mythtv/";
         dir.setPath(testdir);
@@ -770,7 +807,7 @@ void MythContext::KickDatabase(QSqlDatabase *db)
 
 void MythContext::DBError(const QString &where, const QSqlQuery& query) 
 {
-    if(query.lastError().type())
+    if (query.lastError().type())
     {
         cerr << "DB Error (" << where << "):" << endl;
     }
@@ -981,7 +1018,9 @@ void MythContext::ThemeWidget(QWidget *widget)
     {
         widget->setPalette(d->m_palette);
         if (d->m_backgroundimage && d->m_backgroundimage->width() > 0)
+        {
             widget->setPaletteBackgroundPixmap(*(d->m_backgroundimage));
+        }
         return;
     }
 
@@ -998,6 +1037,7 @@ void MythContext::ThemeWidget(QWidget *widget)
         bgpixmap = LoadScalePixmap(pmapname);
         if (bgpixmap)
         {
+            widget->setBackgroundOrigin(QWidget::AncestorOrigin);
             widget->setPaletteBackgroundPixmap(*bgpixmap);
             d->m_backgroundimage = new QPixmap(*bgpixmap);
         }
@@ -1022,6 +1062,7 @@ void MythContext::ThemeWidget(QWidget *widget)
             tmp.end();
 
             d->m_backgroundimage = new QPixmap(background);
+            widget->setBackgroundOrigin(QWidget::AncestorOrigin);
             widget->setPaletteBackgroundPixmap(background);
         }
     }
@@ -1226,6 +1267,8 @@ void MythContext::SetSetting(const QString &key, const QString &newValue)
     d->m_settings->SetSetting(key, newValue);
 }
 
+
+
 bool MythContext::SendReceiveStringList(QStringList &strlist, bool quickTimeout)
 {
     d->serverSockLock.lock();
@@ -1499,18 +1542,27 @@ void MythContext::ResetScreensaver(void)
 void MythContext::DoDisableScreensaver(void)
 {
     d->screensaver->Disable();
+    d->screensaverEnabled = false;
 }
 
 void MythContext::DoRestoreScreensaver(void)
 {
     d->screensaver->Restore();
+    d->screensaverEnabled = true;
 }
 
 void MythContext::DoResetScreensaver(void)
 {
     d->screensaver->Reset();
+    d->screensaverEnabled = false;
 }
 
+bool MythContext::GetScreensaverEnabled(void)
+{
+    return d->screensaverEnabled;
+}
+
+
 void MythContext::LogEntry(const QString &module, int priority,
                            const QString &message, const QString &details)
 {
@@ -1586,9 +1638,30 @@ void MythContext::LogEntry(const QString &module, int priority,
     
         d->dbLock.unlock();
 
-	if (priority <= d->m_logprintlevel)
+        if (priority <= d->m_logprintlevel)
             VERBOSE(VB_ALL, module + ": " + message);
     }
 }
 
+void MythContext::addPrivRequest(MythPrivRequest::Type t, void *data)
+{
+    QMutexLocker lockit(d->m_priv_mutex);
+    d->m_priv_requests.push(MythPrivRequest(t, data));
+}
 
+bool MythContext::hasPrivRequest() const
+{
+    QMutexLocker lockit(d->m_priv_mutex);
+    return !d->m_priv_requests.empty();
+}
+
+MythPrivRequest MythContext::popPrivRequest()
+{
+    QMutexLocker lockit(d->m_priv_mutex);
+    MythPrivRequest ret_val(MythPrivRequest::PrivEnd, NULL);
+    if (!d->m_priv_requests.empty()) {
+        ret_val = d->m_priv_requests.front();
+        d->m_priv_requests.pop();
+    }
+    return ret_val;
+}
