@@ -11,9 +11,13 @@
 #include <qfocusdata.h>
 #include <qdict.h>
 #include <qsqldatabase.h>
+
 #ifdef QWS
 #include <qwindowsystem_qws.h>
 #endif
+#ifdef Q_WS_MACX
+#import <HIToolbox/Menus.h>   // For HideMenuBar()
+#endif
 
 #include <iostream>
 using namespace std;
@@ -76,7 +80,7 @@ struct JumpData
 
 struct MHData
 {
-    void (*callback)(void);
+    void (*callback)(MythMediaDevice *mediadevice);
     int MediaType;
     QString destination;
     QString description;
@@ -110,6 +114,9 @@ class MythMainWindowPrivate
 
     void (*exitmenucallback)(void);
 
+    void (*exitmenumediadevicecallback)(MythMediaDevice* mediadevice);
+    MythMediaDevice * mediadeviceforcallback;
+
     int escapekey;
 };
 
@@ -125,7 +132,8 @@ int MythMainWindowPrivate::TranslateKeyNum(QKeyEvent* e)
         // if modifiers have been pressed, rebuild keynum
         if ((modifiers = e->state()) != 0)
         {
-            int modnum = ((modifiers & Qt::ShiftButton) ? Qt::SHIFT : 0) |
+            int modnum = (((modifiers & Qt::ShiftButton) &&
+                           keynum > 0x7f) ? Qt::SHIFT : 0) |
                          ((modifiers & Qt::ControlButton) ? Qt::CTRL : 0) |
                          ((modifiers & Qt::MetaButton) ? Qt::META : 0) |
                          ((modifiers & Qt::AltButton) ? Qt::ALT : 0);
@@ -146,6 +154,8 @@ MythMainWindow::MythMainWindow(QWidget *parent, const char *name, bool modal)
     d->ignore_lirc_keys = false;
     d->exitingtomain = false;
     d->exitmenucallback = false;
+    d->exitmenumediadevicecallback = false;
+    d->mediadeviceforcallback = NULL;
     d->escapekey = Key_Escape;
 
 #ifdef USE_LIRC
@@ -229,7 +239,13 @@ void MythMainWindow::Show(void)
     if (gContext->GetNumSetting("RunFrontendInWindow", 0))
         show();
     else
+    {
         showFullScreen();
+#ifdef Q_WS_MACX
+        HideMenuBar();
+#endif
+    }
+
     setActiveWindow();
 }
 
@@ -247,7 +263,7 @@ void MythMainWindow::detach(QWidget *child)
 {
     if (d->widgetList.back() != child)
     {
-        cerr << "Not removing top-most widget\n";
+        //cerr << "Not removing top-most widget\n";
         return;
     }
 
@@ -305,6 +321,13 @@ void MythMainWindow::ExitToMainMenu(void)
 
                 callback();
             }
+            else if (d->exitmenumediadevicecallback)
+            {
+                void (*callback)(MythMediaDevice*) = d->exitmenumediadevicecallback;
+                MythMediaDevice * mediadevice = d->mediadeviceforcallback;
+                d->mediadeviceforcallback = NULL;
+                callback(mediadevice);
+            }
         }
     }
 }
@@ -498,7 +521,7 @@ bool MythMainWindow::DestinationExists(const QString& destination) const
 void MythMainWindow::RegisterMediaHandler(const QString &destination,
                                           const QString &description,
                                           const QString &key, 
-                                          void (*callback)(void),
+                              void (*callback)(MythMediaDevice*mediadevice),
                                           int mediaType)
 {
     (void)key;
@@ -537,7 +560,9 @@ void MythMainWindow::RegisterMediaPlugin(const QString &name,
     }
 }
 
-bool MythMainWindow::HandleMedia(QString &handler, const QString &mrl)
+
+bool MythMainWindow::HandleMedia(QString& handler, const QString &mrl, const QString& plot,
+                                 const QString& title, const QString& director, int lenMins, const QString& year)
 {
     if (handler.length() < 1)
         handler = "Internal";
@@ -545,7 +570,7 @@ bool MythMainWindow::HandleMedia(QString &handler, const QString &mrl)
     // Let's see if we have a plugin that matches the handler name...
     if (d->mediaPluginMap.count(handler)) 
     {
-        d->mediaPluginMap[handler].playFn(mrl);
+        d->mediaPluginMap[handler].playFn(mrl, plot, title, director, lenMins, year);
         return true;
     }
 
@@ -590,17 +615,32 @@ void MythMainWindow::customEvent(QCustomEvent *ce)
 
         if (pDev) 
         {
-            while (itr != d->mediaHandlerMap.end())
+            QWidget * activewidget = qApp->focusWidget();
+            MythDialog * activedialog = NULL;
+            bool iscatched = false;
+            while (activewidget && !activedialog)
             {
-                if ((itr.data().MediaType & (int)pDev->getMediaType()))
+                activedialog = dynamic_cast<MythDialog*>(activewidget);
+                if (!activedialog)
+                       activewidget = activewidget->parentWidget();
+            }
+            if (activedialog)
+                iscatched = activedialog->onMediaEvent(pDev);
+            if (!iscatched)
+            {
+                while (itr != d->mediaHandlerMap.end())
                 {
-                    cout << "Found a handler" << endl;
-                    d->exitingtomain = true;
-                    d->exitmenucallback = itr.data().callback;
-                    QApplication::postEvent(this, new ExitToMainMenuEvent());
-                    break;
+                   if ((itr.data().MediaType & (int)pDev->getMediaType()))
+                   {
+                       VERBOSE(VB_ALL, "Found a handler");
+                       d->exitingtomain = true;
+                       d->exitmenumediadevicecallback = itr.data().callback;
+                       d->mediadeviceforcallback = pDev;
+                       QApplication::postEvent(this, new ExitToMainMenuEvent());
+                       break;
+                   }
+                   itr++;
                 }
-                itr++;
             }
         }
     }
@@ -726,6 +766,7 @@ MythDialog::MythDialog(MythMainWindow *parent, const char *name, bool setsize)
 
     if (setsize)
     {
+        move(0, 0);
         setFixedSize(QSize(screenwidth, screenheight));
         gContext->ThemeWidget(this);
     }
@@ -749,6 +790,13 @@ void MythDialog::setNoErase(void)
     setWFlags(flags);
 }
 
+bool MythDialog::onMediaEvent(MythMediaDevice*)
+{
+    return false;
+}
+
+
+
 void MythDialog::Show(void)
 {
     show();
@@ -854,7 +902,12 @@ MythPopupBox::MythPopupBox(MythMainWindow *parent, const char *name)
             : MythDialog(parent, name, false)
 {
     float wmult, hmult;
-    arrowAccel = true;
+
+    if (gContext->GetNumSetting("UseArrowAccels", 1))
+        arrowAccel = true;
+    else
+        arrowAccel = false;
+
     gContext->GetScreenSettings(wmult, hmult);
 
     setLineWidth(3);
@@ -878,7 +931,12 @@ MythPopupBox::MythPopupBox(MythMainWindow *parent, bool graphicPopup,
             : MythDialog(parent, name, false)
 {
     float wmult, hmult;
-    arrowAccel = true;
+
+    if (gContext->GetNumSetting("UseArrowAccels", 1))
+        arrowAccel = true;
+    else
+        arrowAccel = false;
+
     gContext->GetScreenSettings(wmult, hmult);
 
     setLineWidth(3);
@@ -1083,7 +1141,7 @@ void MythPopupBox::keyPressEvent(QKeyEvent *e)
     {
         QString action = actions[i];
 
-        if((action == "ESCAPE") || (arrowAccel && action == "LEFT"))
+        if ((action == "ESCAPE") || (arrowAccel && action == "LEFT"))
         {
             emit popupDone();
             handled = true;
@@ -1157,6 +1215,31 @@ bool MythPopupBox::showOkCancelPopup(MythMainWindow *parent, QString title,
     return (popup.ExecPopup() == 0);
 }
 
+bool MythPopupBox::showGetTextPopup(MythMainWindow *parent, QString title,
+                                    QString message, QString& text)
+{
+    MythPopupBox popup(parent, title);
+    popup.addLabel(message, Medium, true);
+    
+    MythRemoteLineEdit* textEdit = new MythRemoteLineEdit(&popup, "chooseEdit");
+    textEdit->setText(text);
+    popup.addWidget(textEdit);
+    
+    popup.addButton(tr("OK"));
+    popup.addButton(tr("Cancel"));
+    
+    textEdit->setFocus();
+    
+    if (popup.ExecPopup() == 0)
+    {
+        text = textEdit->text();
+        return true;
+    }
+    
+    return false;
+}
+
+
 int MythPopupBox::show2ButtonPopup(MythMainWindow *parent, QString title,
                                    QString message, QString button1msg,
                                    QString button2msg, int defvalue)
@@ -1299,11 +1382,11 @@ MythThemedDialog::MythThemedDialog(MythMainWindow *parent, QString window_name,
     theme = new XMLParse();
     theme->SetWMult(wmult);
     theme->SetHMult(hmult);
-    if(!theme->LoadTheme(xmldata, window_name, theme_filename))
+    if (!theme->LoadTheme(xmldata, window_name, theme_filename))
     {
         cerr << "dialogbox.o: Couldn't find your theme. I'm outta here" << endl;
         cerr << window_name << " - " <<  theme_filename << endl;
-        exit(0);
+        exit(-31);
     }
 
     loadWindow(xmldata);
@@ -1366,7 +1449,7 @@ bool MythThemedDialog::buildFocusList()
         }
         ++another_it;
     }
-    if(focus_taking_widgets.count() > 0)
+    if (focus_taking_widgets.count() > 0)
     {
         return true;
     }
@@ -1406,7 +1489,7 @@ void MythThemedDialog::loadWindow(QDomElement &element)
             {
                 cerr << "dialogbox.o: I don't understand this DOM Element:" 
                      << e.tagName() << endl;
-                exit(0);
+                exit(-32);
             }
         }
     }
@@ -1428,7 +1511,7 @@ void MythThemedDialog::parseContainer(QDomElement &element)
     {
         cerr << "dialogbox.o: I told an object to parse a container and it "
                 "didn't give me a name back\n";
-        exit(0);
+        exit(-33);
     }
 
     LayerSet *container_reference = theme->GetSet(name);
@@ -1507,7 +1590,7 @@ void MythThemedDialog::updateForeground()
 void MythThemedDialog::updateForeground(const QRect &r)
 {
     QRect rect_to_update = r;
-    if(r.width() == 0 || r.height() == 0)
+    if (r.width() == 0 || r.height() == 0)
     {
         cerr << "MythThemedDialog.o: something is requesting a screen update of zero size. "
              << "A widget probably has not done a calculateScreeArea(). Will redraw "
@@ -1525,7 +1608,7 @@ void MythThemedDialog::updateForeground(const QRect &r)
 void MythThemedDialog::ReallyUpdateForeground(const QRect &r)
 {
     QRect rect_to_update = r;
-    if(r.width() == 0 || r.height() == 0)
+    if (r.width() == 0 || r.height() == 0)
     {
         cerr << "MythThemedDialog.o: something is requesting a screen update of zero size. "
              << "A widget probably has not done a calculateScreeArea(). Will redraw "
@@ -1679,7 +1762,7 @@ bool MythThemedDialog::assignFirstFocus()
 
     while ((looper = an_it.current()) != 0)
     {
-        if(looper->canTakeFocus())
+        if (looper->canTakeFocus())
         {
             widget_with_current_focus = looper;
             widget_with_current_focus->takeFocus();
@@ -1750,7 +1833,7 @@ bool MythThemedDialog::nextPrevWidgetFocus(bool up_or_down)
             an_it.toLast();
             while ((looper = an_it.current()) != 0)
             {
-                if(looper->canTakeFocus())
+                if (looper->canTakeFocus())
                 {
                     widget_with_current_focus->looseFocus();
                     widget_with_current_focus = looper;
@@ -1805,7 +1888,7 @@ UIType* MythThemedDialog::getUIObject(const QString &name)
 
 UIType* MythThemedDialog::getCurrentFocusWidget()
 {
-    if(widget_with_current_focus)
+    if (widget_with_current_focus)
     {
         return widget_with_current_focus;
     }
@@ -1820,10 +1903,10 @@ UIManagedTreeListType* MythThemedDialog::getUIManagedTreeListType(const QString
     while( (looper = an_it.current()) != 0)
     {
         UIType *hunter = looper->GetType(name);
-        if(hunter)
+        if (hunter)
         {
             UIManagedTreeListType *hunted;
-            if( (hunted = dynamic_cast<UIManagedTreeListType*>(hunter)) )
+            if ( (hunted = dynamic_cast<UIManagedTreeListType*>(hunter)) )
             {
                 return hunted;
             }
@@ -1841,10 +1924,10 @@ UITextType* MythThemedDialog::getUITextType(const QString &name)
     while( (looper = an_it.current()) != 0)
     {
         UIType *hunter = looper->GetType(name);
-        if(hunter)
+        if (hunter)
         {
             UITextType *hunted;
-            if( (hunted = dynamic_cast<UITextType*>(hunter)) )
+            if ( (hunted = dynamic_cast<UITextType*>(hunter)) )
             {
                 return hunted;
             }
@@ -1862,10 +1945,10 @@ UIMultiTextType* MythThemedDialog::getUIMultiTextType(const QString &name)
     while( (looper = an_it.current()) != 0)
     {
         UIType *hunter = looper->GetType(name);
-        if(hunter)
+        if (hunter)
         {
             UIMultiTextType *hunted;
-            if( (hunted = dynamic_cast<UIMultiTextType*>(hunter)) )
+            if ( (hunted = dynamic_cast<UIMultiTextType*>(hunter)) )
             {
                 return hunted;
             }
@@ -1883,10 +1966,10 @@ UIPushButtonType* MythThemedDialog::getUIPushButtonType(const QString &name)
     while( (looper = an_it.current()) != 0)
     {
         UIType *hunter = looper->GetType(name);
-        if(hunter)
+        if (hunter)
         {
             UIPushButtonType *hunted;
-            if( (hunted = dynamic_cast<UIPushButtonType*>(hunter)) )
+            if ( (hunted = dynamic_cast<UIPushButtonType*>(hunter)) )
             {
                 return hunted;
             }
@@ -1904,10 +1987,10 @@ UITextButtonType* MythThemedDialog::getUITextButtonType(const QString &name)
     while( (looper = an_it.current()) != 0)
     {
         UIType *hunter = looper->GetType(name);
-        if(hunter)
+        if (hunter)
         {
             UITextButtonType *hunted;
-            if( (hunted = dynamic_cast<UITextButtonType*>(hunter)) )
+            if ( (hunted = dynamic_cast<UITextButtonType*>(hunter)) )
             {
                 return hunted;
             }
@@ -1925,10 +2008,10 @@ UIRepeatedImageType* MythThemedDialog::getUIRepeatedImageType(const QString &nam
     while( (looper = an_it.current()) != 0)
     {
         UIType *hunter = looper->GetType(name);
-        if(hunter)
+        if (hunter)
         {
             UIRepeatedImageType *hunted;
-            if( (hunted = dynamic_cast<UIRepeatedImageType*>(hunter)) )
+            if ( (hunted = dynamic_cast<UIRepeatedImageType*>(hunter)) )
             {
                 return hunted;
             }
@@ -1946,10 +2029,10 @@ UICheckBoxType* MythThemedDialog::getUICheckBoxType(const QString &name)
     while( (looper = an_it.current()) != 0)
     {
         UIType *hunter = looper->GetType(name);
-        if(hunter)
+        if (hunter)
         {
             UICheckBoxType *hunted;
-            if( (hunted = dynamic_cast<UICheckBoxType*>(hunter)) )
+            if ( (hunted = dynamic_cast<UICheckBoxType*>(hunter)) )
             {
                 return hunted;
             }
@@ -1967,10 +2050,10 @@ UISelectorType* MythThemedDialog::getUISelectorType(const QString &name)
     while( (looper = an_it.current()) != 0)
     {
         UIType *hunter = looper->GetType(name);
-        if(hunter)
+        if (hunter)
         {
             UISelectorType *hunted;
-            if( (hunted = dynamic_cast<UISelectorType*>(hunter)) )
+            if ( (hunted = dynamic_cast<UISelectorType*>(hunter)) )
             {
                 return hunted;
             }
@@ -1988,10 +2071,10 @@ UIBlackHoleType* MythThemedDialog::getUIBlackHoleType(const QString &name)
     while( (looper = an_it.current()) != 0)
     {
         UIType *hunter = looper->GetType(name);
-        if(hunter)
+        if (hunter)
         {
             UIBlackHoleType *hunted;
-            if( (hunted = dynamic_cast<UIBlackHoleType*>(hunter)) )
+            if ( (hunted = dynamic_cast<UIBlackHoleType*>(hunter)) )
             {
                 return hunted;
             }
@@ -2009,10 +2092,10 @@ UIImageType* MythThemedDialog::getUIImageType(const QString &name)
     while( (looper = an_it.current()) != 0)
     {
         UIType *hunter = looper->GetType(name);
-        if(hunter)
+        if (hunter)
         {
             UIImageType *hunted;
-            if( (hunted = dynamic_cast<UIImageType*>(hunter)) )
+            if ( (hunted = dynamic_cast<UIImageType*>(hunter)) )
             {
                 return hunted;
             }
@@ -2030,10 +2113,10 @@ UIStatusBarType* MythThemedDialog::getUIStatusBarType(const QString &name)
     while( (looper = an_it.current()) != 0)
     {
         UIType *hunter = looper->GetType(name);
-        if(hunter)
+        if (hunter)
         {
             UIStatusBarType *hunted;
-            if( (hunted = dynamic_cast<UIStatusBarType*>(hunter)) )
+            if ( (hunted = dynamic_cast<UIStatusBarType*>(hunter)) )
             {
                 return hunted;
             }
@@ -2085,6 +2168,22 @@ UIListTreeType* MythThemedDialog::getUIListTreeType(const QString &name)
     return NULL;
 }
 
+LayerSet* MythThemedDialog::getContainer(const QString& name)
+{
+    QPtrListIterator<LayerSet> an_it(my_containers);
+    LayerSet *looper;
+
+    while( (looper = an_it.current()) != 0)
+    {
+        if (looper->GetName() == name)
+            return looper;
+        
+        ++an_it;
+    }
+    
+    return NULL;
+}
+
 /*
 ---------------------------------------------------------------------
 */
@@ -2195,7 +2294,7 @@ MythImageFileDialog::MythImageFileDialog(QString *result,
     
 
     UIImageType *file_browser_background = getUIImageType("file_browser_background");
-    if(file_browser_background)
+    if (file_browser_background)
     {
         QPixmap background = file_browser_background->GetImage();
         
@@ -2206,7 +2305,7 @@ MythImageFileDialog::MythImageFileDialog(QString *result,
     else
     {
         cerr << "myhdialogs.o: Could not find a UIImageType called file_browser_background in your theme." << endl;
-        exit(0);
+        exit(-34);
     }
 
     //
@@ -2223,7 +2322,7 @@ MythImageFileDialog::MythImageFileDialog(QString *result,
     //
 
     file_browser = getUIManagedTreeListType("file_browser");
-    if(file_browser)
+    if (file_browser)
     {
         file_browser->calculateScreenArea();
         file_browser->showWholeTree(true);
@@ -2235,7 +2334,7 @@ MythImageFileDialog::MythImageFileDialog(QString *result,
     else
     {
         cerr << "mythdialogs.o: Could not find a UIManagedTreeListType called file_browser in your theme." << endl;
-        exit(0);
+        exit(-35);
     }    
     
     //
@@ -2244,7 +2343,7 @@ MythImageFileDialog::MythImageFileDialog(QString *result,
     //
     
     image_box = getUIImageType("image_box");
-    if(image_box)
+    if (image_box)
     {
         image_box->calculateScreenArea();
     }
@@ -2254,7 +2353,7 @@ MythImageFileDialog::MythImageFileDialog(QString *result,
     buildTree(top_directory);
 
     file_browser->assignTreeData(root_parent);
-    if(initial_node)
+    if (initial_node)
     {
         file_browser->setCurrentNode(initial_node);
     }
@@ -2311,13 +2410,13 @@ void MythImageFileDialog::buildTree(QString starting_where)
     {
         bool make_active = false;
         QString file_string = *(image_files.at(i));
-        if(file_string == *selected_file)
+        if (file_string == *selected_file)
         {
             make_active = true;
         }
         QString prefix = initialDir;
 
-        if(prefix.length() < 1)
+        if (prefix.length() < 1)
         {
             cerr << "mythdialogs.o: Seems unlikely that this is going to work" << endl;
         }
@@ -2329,11 +2428,11 @@ void MythImageFileDialog::buildTree(QString starting_where)
         QStringList::Iterator an_it = list.begin();
         for( ; an_it != list.end(); ++an_it)
         {
-            if(a_counter + 1 >= (int) list.count())
+            if (a_counter + 1 >= (int) list.count())
             {
                 QString title = (*an_it);
                 GenericTree *added_node = where_to_add->addNode(title.section(".",0,0), i, true);
-                if(make_active)
+                if (make_active)
                 {
                     initial_node = added_node;
                 }
@@ -2343,7 +2442,7 @@ void MythImageFileDialog::buildTree(QString starting_where)
                 QString dirname = *an_it + "/";
                 GenericTree *sub_node;
                 sub_node = where_to_add->getChildByName(dirname);
-                if(!sub_node)
+                if (!sub_node)
                 {
                     sub_node = where_to_add->addNode(dirname, -1, false);
                 }
@@ -2352,7 +2451,7 @@ void MythImageFileDialog::buildTree(QString starting_where)
             ++a_counter;
         }
     }
-    if(file_root->childCount() < 1)
+    if (file_root->childCount() < 1)
     {
         //
         //  Nothing survived the requirements
@@ -2393,7 +2492,7 @@ void MythImageFileDialog::buildFileList(QString directory)
             continue;
         }
             
-        if(fi->isDir())
+        if (fi->isDir())
         {
             buildFileList(fi->absFilePath());
         }
@@ -2402,7 +2501,7 @@ void MythImageFileDialog::buildFileList(QString directory)
             r.setPattern("^" + fi->extension() + "$");
             r.setCaseSensitive(false);
             QStringList result = imageExtensions.grep(r);
-            if(!result.isEmpty())
+            if (!result.isEmpty())
             {
                 image_files.append(fi->absFilePath());
             }
@@ -2411,7 +2510,7 @@ void MythImageFileDialog::buildFileList(QString directory)
                 r.setPattern("^" + fi->extension());
                 r.setCaseSensitive(false);
                 QStringList other_result = imageExtensions.grep(r);
-                if(!result.isEmpty())
+                if (!result.isEmpty())
                 {
                     image_files.append(fi->absFilePath());
                 }
@@ -2422,9 +2521,9 @@ void MythImageFileDialog::buildFileList(QString directory)
 
 void MythImageFileDialog::handleTreeListEntered(int type, IntVector*)
 {
-    if(image_box)
+    if (image_box)
     {
-        if(type > -1)
+        if (type > -1)
         {
             image_box->SetImage(image_files[type]);
         }
@@ -2438,7 +2537,7 @@ void MythImageFileDialog::handleTreeListEntered(int type, IntVector*)
 
 void MythImageFileDialog::handleTreeListSelection(int type, IntVector*)
 {
-    if(type > -1)
+    if (type > -1)
     {
         *selected_file = image_files[type];
         done(0);
@@ -2447,7 +2546,7 @@ void MythImageFileDialog::handleTreeListSelection(int type, IntVector*)
 
 MythImageFileDialog::~MythImageFileDialog()
 {
-    if(root_parent)
+    if (root_parent)
     {
         root_parent->deleteAllChildren();
         delete root_parent;
@@ -2465,7 +2564,7 @@ MythScrollDialog::MythScrollDialog(MythMainWindow *parent,
     if (!parent) {
         std::cerr << "MythScrollDialog: Trying to create a dialog without a parent"
                   << std::endl;
-        exit(-1);
+        exit(-36);
     }
 
     m_parent     = parent;
@@ -2556,7 +2655,7 @@ void MythScrollDialog::setAreaMultiplied(int areaWTimes, int areaHTimes)
     if (areaWTimes < 1 || areaHTimes < 1) {
         std::cerr << "MythScrollDialog: invalid areaWTimes or areaHTimes"
                   << std::endl;
-        exit(-1);
+        exit(-37);
     }
 
     resizeContents(m_screenWidth*areaWTimes,
