@@ -43,7 +43,7 @@ bool MythMediaDevice::openDevice()
     if (isDeviceOpen())
         return true;
  
-    m_DeviceHandle = open(m_DevicePath, O_RDONLY | O_NONBLOCK | O_EXCL);
+    m_DeviceHandle = open(m_DevicePath, O_RDONLY | O_NONBLOCK);
     
     return isDeviceOpen();
 }
@@ -85,7 +85,7 @@ bool MythMediaDevice::performMountCmd(bool DoMount)
         VERBOSE(VB_ALL,  QString("Executing '%1'").arg(MountCommand));
         if (0 == system(MountCommand)) 
         {
-            if(DoMount)
+            if (DoMount)
             {
                 m_Status = MEDIASTAT_MOUNTED;
                 onDeviceMounted();
@@ -127,14 +127,11 @@ MediaError MythMediaDevice::lock()
     return MEDIAERR_FAILED;
 }
 
-MediaError MythMediaDevice::unlock() 
+MediaError MythMediaDevice::unlock()
 { 
     m_Locked = false;
     
-    if (closeDevice())
-        return MEDIAERR_OK;
-    
-    return MEDIAERR_FAILED;
+    return MEDIAERR_OK;
 }
 
 bool MythMediaDevice::isMounted(bool Verify)
@@ -195,6 +192,7 @@ MediaStatus MythMediaDevice::setStatus( MediaStatus NewStatus, bool CloseIt )
         {
             // the disk is not / should not be mounted.
             case MEDIASTAT_ERROR:
+            case MEDIASTAT_OPEN:
                 if (MEDIASTAT_MOUNTED == OldStatus)
                     unmount();
                 break;
@@ -207,7 +205,6 @@ MediaStatus MythMediaDevice::setStatus( MediaStatus NewStatus, bool CloseIt )
                 mount();
                 break;
             case MEDIASTAT_UNKNOWN:
-            case MEDIASTAT_OPEN:
             case MEDIASTAT_USEABLE:
             case MEDIASTAT_MOUNTED:
                 // get rid of the compiler warning...
