@@ -35,12 +35,14 @@ class MythComboBox: public QComboBox
     Q_OBJECT
   public:
     MythComboBox(bool rw, QWidget* parent=0, const char* name=0):
-        QComboBox(rw, parent, name) {};
+        QComboBox(rw, parent, name) { AcceptOnSelect = false; };
 
     void setHelpText(QString help) { helptext = help; }
+    void setAcceptOnSelect(bool Accept) { AcceptOnSelect = Accept; }
 
   signals:
     void changeHelpText(QString);
+    void accepted(int);
     void gotFocus();
 
   public slots:
@@ -55,6 +57,7 @@ class MythComboBox: public QComboBox
 
   private:
     QString helptext;
+    bool AcceptOnSelect;
 };
 
 class MythSpinBox: public QSpinBox 
@@ -227,13 +230,13 @@ class MythPushButton : public QPushButton
 {
     Q_OBJECT
   public:
-    MythPushButton(QWidget *parent, const char *name = 0)
+    MythPushButton(QWidget *parent, const char *name = 0, bool aa = false)
                  : QPushButton(parent, name)
-                  { setBackgroundOrigin(WindowOrigin); }
+                  { setBackgroundOrigin(WindowOrigin); arrowAccel = aa;}
 
-    MythPushButton(const QString &text, QWidget *parent)
+    MythPushButton(const QString &text, QWidget *parent, bool aa = false)
                  : QPushButton(text, parent)
-                  { setBackgroundOrigin(WindowOrigin); }
+                  { setBackgroundOrigin(WindowOrigin); arrowAccel = aa;}
 
     void setHelpText(const QString &help) { helptext = help; }
 
@@ -250,6 +253,7 @@ class MythPushButton : public QPushButton
   private:
     QColor origcolor;
     QString helptext;
+    bool arrowAccel;
 };
 
 class MythCheckBox: public QCheckBox
