@@ -58,7 +58,7 @@ struct ThemedButton
 
     QString text;
     QString altText;
-    QString action;
+    QStringList action;
 
     int row;
     int col;
@@ -110,12 +110,12 @@ class ThemedMenuPrivate
     void setDefaults(void);
 
     void addButton(const QString &type, const QString &text,
-                   const QString &alttext, const QString &action);
+                   const QString &alttext, const QStringList &action);
     void layoutButtons(void);
     void positionButtons(bool resetpos);
     bool makeRowVisible(int newrow, int oldrow, bool forcedraw = true);
 
-    void handleAction(const QString &action);
+    bool handleAction(const QString &action);
     bool findDepends(const QString &file);
     QString findMenuFile(const QString &menuname);
 
@@ -141,6 +141,8 @@ class ThemedMenuPrivate
                       const QString &password_setting,
                       const QString &text);
 
+    void updateLCD(void);
+
     ThemedMenu *parent;
 
     float wmult;
@@ -973,6 +975,14 @@ void ThemedMenuPrivate::parseSettings(const QString &dir,
         cerr << "at line: " << errorLine << "  column: " << errorColumn << endl;
         cerr << errorMsg << endl;
         f.close();
+        MythPopupBox::showOkPopup(gContext->GetMainWindow(), 
+                                  QObject::tr("Bad Menu File"),
+                                  QObject::tr(QString("The menu file %1 is "
+                                              "incomplete.\n\nWe will now "
+                                              "return to the main menu.")
+                                              .arg(menuname)));
+        menulevel = 0;
+        parseMenu("mainmenu.xml");
         return;
     }
 
@@ -1048,7 +1058,7 @@ void ThemedMenuPrivate::parseThemeButton(QDomElement &element)
 {
     QString type = "";
     QString text = "";
-    QString action = "";
+    QStringList action;
     QString alttext = "";
 
     bool addit = true;
@@ -1091,7 +1101,7 @@ void ThemedMenuPrivate::parseThemeButton(QDomElement &element)
             }
             else if (info.tagName() == "action")
             {
-                action = getFirstText(info);
+                action += getFirstText(info);
             }
             else if (info.tagName() == "depends")
             {
@@ -1119,7 +1129,7 @@ void ThemedMenuPrivate::parseThemeButton(QDomElement &element)
         return;
     }
    
-    if (action == "")
+    if (action.empty())
     {
         cerr << "Missing 'action' in button\n";
         return;
@@ -1138,23 +1148,20 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
 
     if (!f.open(IO_ReadOnly))
     {
-        
         cerr << "Couldn't read menu file " << menuname << endl;
-        if(menuname == "mainmenu.xml" )
+        if (menuname == "mainmenu.xml" )
         {
-            exit(0);
+            exit(-38);
         }
         else
         {
             MythPopupBox::showOkPopup(gContext->GetMainWindow(), QObject::tr("No Menu File"),
                                       QObject::tr(QString("Myth could not locate the menu file %1.\n\n"
                                       "We will now return to the main menu.").arg(menuname)));
-
+            menulevel = 0;
             parseMenu("mainmenu.xml");
             return;
         }
-        
-        
     }
 
     QString errorMsg;
@@ -1167,7 +1174,21 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
         cerr << "at line: " << errorLine << "  column: " << errorColumn << endl;
         cerr << errorMsg << endl;
         f.close();
-        exit(1);
+
+        if (menuname == "mainmenu.xml" )
+        {
+            exit(-39);
+        }
+
+        MythPopupBox::showOkPopup(gContext->GetMainWindow(), 
+                                  QObject::tr("Bad Menu File"),
+                                  QObject::tr(QString("The menu file %1 is "
+                                              "incomplete.\n\nWe will now "
+                                              "return to the main menu.")
+                                              .arg(menuname)));
+        menulevel = 0;
+        parseMenu("mainmenu.xml");
+        return;
     }
 
     f.close();
@@ -1192,7 +1213,7 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
             else
             {
                 cerr << "Unknown element " << e.tagName() << endl;
-                exit(1);
+                exit(-40);
             }
         }
         n = n.nextSibling();
@@ -1201,7 +1222,7 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
     if (buttonList.size() == 0)
     {
         cerr << "No buttons for menu " << menuname << endl;
-        exit(1);
+        exit(-41);
     }
 
     layoutButtons();
@@ -1256,30 +1277,39 @@ void ThemedMenuPrivate::parseMenu(const QString &menuname, int row, int col)
     {
         titleText = "MYTH-";
         titleText += menumode;
-        QPtrList<LCDMenuItem> menuItems;
-        menuItems.setAutoDelete(true);
-        bool selected;
-
-        for (int r = 0; r < (int)buttonRows.size(); r++)
-        {
-            if (r == currentrow)
-                selected = true;
-            else
-                selected = false;
-
-            if (currentcolumn < buttonRows[r].numitems)
-                menuItems.append(new LCDMenuItem(selected, NOTCHECKABLE,
-                                 buttonRows[r].buttons[currentcolumn]->text));
-        }
-
-        if (!menuItems.isEmpty())
-            lcddev->switchToMenu(&menuItems, titleText);
+        updateLCD();
     }
 
     selection = "";
     parent->update(menuRect());
 }
 
+void ThemedMenuPrivate::updateLCD()
+{
+    if (!lcddev)
+        return;
+
+    // Build a list of the menu items
+    QPtrList<LCDMenuItem> menuItems;
+    menuItems.setAutoDelete(true);
+    bool selected;
+
+    for (int r = 0; r < (int)buttonRows.size(); r++)
+    {
+        if (r == currentrow)
+            selected = true;
+        else
+            selected = false;
+
+        if (currentcolumn < buttonRows[r].numitems)
+            menuItems.append(new LCDMenuItem(selected, NOTCHECKABLE,
+                             buttonRows[r].buttons[currentcolumn]->text));
+    }
+
+    if (!menuItems.isEmpty())
+        lcddev->switchToMenu(&menuItems, titleText);
+}
+
 QPoint ThemedMenuPrivate::parsePoint(const QString &text)
 {
     int x, y;
@@ -1300,7 +1330,8 @@ QRect ThemedMenuPrivate::parseRect(const QString &text)
 }
 
 void ThemedMenuPrivate::addButton(const QString &type, const QString &text, 
-                                  const QString &alttext, const QString &action)
+                                  const QString &alttext, 
+                                  const QStringList &action)
 {
     ThemedButton newbutton;
 
@@ -1327,13 +1358,13 @@ void ThemedMenuPrivate::layoutButtons(void)
     if (maxrows < 2)
     {
         cerr << "Must have room for at least 2 rows of buttons\n";
-        exit(1);
+        exit(-43);
     }
     
     if (columns < 1)
     {
         cerr << "Must have room for at least 1 column of buttons\n";
-        exit(1);
+        exit(-44);
     }
 
     // keep the rows balanced
@@ -1920,7 +1951,14 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
             lastbutton = activebutton;
             activebutton = NULL;
             parent->repaint(lastbutton->posRect);
-            handleAction(lastbutton->action);
+
+            QStringList::Iterator it = lastbutton->action.begin();
+            for (; it != lastbutton->action.end(); it++)
+            {
+                if (handleAction(*it))
+                    break;
+            }
+
             lastbutton = NULL;
         }
         else if (action == "ESCAPE")
@@ -1946,27 +1984,7 @@ bool ThemedMenuPrivate::keyPressHandler(QKeyEvent *e)
     }
 
     activebutton = buttonRows[currentrow].buttons[currentcolumn];
-    if (lcddev)
-    {
-        // Build a list of the menu items
-        QPtrList<LCDMenuItem> menuItems;
-        menuItems.setAutoDelete(true);
-        bool selected;
-        for (int r = 0; r < (int)buttonRows.size(); r++)
-        {
-            if (r == currentrow)
-                selected = true;
-            else
-                selected = false;
-
-            if (currentcolumn < buttonRows[r].numitems)
-                menuItems.append(new LCDMenuItem(selected, NOTCHECKABLE,
-                                 buttonRows[r].buttons[currentcolumn]->text));
-        }
-
-        if (!menuItems.isEmpty())
-            lcddev->switchToMenu(&menuItems, titleText);
-    }
+    updateLCD();
 
     parent->update(watermarkRect);
     if (lastbutton)
@@ -2007,12 +2025,14 @@ QString ThemedMenuPrivate::findMenuFile(const QString &menuname)
     return "";
 }
 
-void ThemedMenuPrivate::handleAction(const QString &action)
+bool ThemedMenuPrivate::handleAction(const QString &action)
 {
     if (action.left(5) == "EXEC ")
     {
         QString rest = action.right(action.length() - 5);
         myth_system(rest);
+
+        return false;
     }
     else if (action.left(7) == "EXECTV ")
     {
@@ -2059,7 +2079,7 @@ void ThemedMenuPrivate::handleAction(const QString &action)
             gContext->GetNumSetting("SetupPinCodeRequired", 0) &&
             !checkPinCode("SetupPinCodeTime", "SetupPinCode", "Setup Pin:"))
         {
-            return;
+            return true;
         }
 
         parseMenu(rest);
@@ -2107,22 +2127,29 @@ void ThemedMenuPrivate::handleAction(const QString &action)
         {
             callback(callbackdata, selection);
         }
-    }   
+    }
+
+    return true;
 }
 
-bool ThemedMenuPrivate::findDepends(const QString &file)
+bool ThemedMenuPrivate::findDepends(const QString &fileList)
 {
-    QString filename = findMenuFile(file);
-    if (filename != "")
-        return true;
-
-    QString newname = QString(PREFIX) + "/lib/mythtv/plugins/lib" + file +
-                      ".so";
-
-    QFile checkFile(newname);
-    if (checkFile.exists())
-        return true;
-
+    QStringList files = QStringList::split(" ", fileList);
+    QString filename;
+    
+    for ( QStringList::Iterator it = files.begin(); it != files.end(); ++it ) {
+        filename = findMenuFile(*it);
+        if (filename != "")
+            return true;
+    
+        QString newname = QString(PREFIX) + "/lib/mythtv/plugins/lib" + *it +
+                          ".so";
+    
+        QFile checkFile(newname);
+        if (checkFile.exists())
+            return true;
+    }
+    
     return false;
 }
 
@@ -2277,6 +2304,8 @@ void ThemedMenu::paintEvent(QPaintEvent *e)
         if (r.intersects(d->buttonList[i].posRect))
             d->paintButton(i, &p, e->erased());
     }
+
+    d->updateLCD();
 }
 
 void ThemedMenu::ReloadTheme(void)
