@@ -8,7 +8,7 @@ using namespace std;
 #include "util.h"
 #include "mythcontext.h"
 
-#ifndef QWS
+#if defined(Q_WS_X11)
 extern "C" {
 #include <X11/Xlib.h>
 #include <X11/extensions/Xinerama.h>
@@ -275,10 +275,38 @@ int ReadBlock(QSocketDevice *socket, void *data, int maxlen)
     return maxlen;
 }
 
-
+// QSocket (backend version)
 bool WriteStringList(QSocket *socket, QStringList &list)
-{// QSocket (backend)
+{
+    if (list.size() <= 0)
+    {
+        VERBOSE(VB_ALL, "invalid stringlist in WriteStringList");
+        return false;
+    }
+
+    if (!socket || socket->state() != QSocket::Connected)
+    {
+        VERBOSE(VB_ALL, "writing to unconnected socket in WriteStringList");
+        return false;
+    }
+
+    QStringList::iterator iter = list.begin();
+    for (; iter != list.end(); iter++)
+    {
+        if ((*iter) == QString::null)
+        {
+            VERBOSE(VB_ALL, "NULL in string in list in WriteStringList");
+            return false;
+        }
+    }
+
     QString str = list.join("[]:[]");
+    if (str == QString::null)
+    {
+        VERBOSE(VB_ALL, "joined null string in WriteStringList");
+        return false;
+    }
+
     QCString utf8 = str.utf8();
 
     int size = utf8.length();
@@ -310,9 +338,22 @@ bool WriteStringList(QSocket *socket, QStringList &list)
 
     while (size > 0)
     {
+        if (socket->state() != QSocket::Connected)
+        {
+            VERBOSE(VB_ALL, "writing to unconnected socket #2");
+            return false;
+        }
+
         qApp->lock();
         int temp = socket->writeBlock(payload.data() + written, size);
         qApp->unlock();
+
+        if (temp < 0)
+        {
+            VERBOSE(VB_ALL, "writeBlock failed!");
+            return false;
+        }
+
         // cerr << "  written: " << temp << endl; //DEBUG
         written += temp;
         size -= temp;
@@ -500,7 +541,7 @@ long long decodeLongLong(QStringList &list, QStringList::iterator &it)
     return retval;
 } 
 
-#ifndef QWS
+#if defined(Q_WS_X11)
 void GetMythTVGeometry(Display *dpy, int screen_num, int *x, int *y, 
                        int *w, int *h) 
 {
@@ -519,7 +560,7 @@ void GetMythTVGeometry(Display *dpy, int screen_num, int *x, int *y,
 
         printf("Found %d Xinerama Screens.\n", nr_xinerama_screens);
 
-        if(screen_nr > 0 && screen_nr < nr_xinerama_screens)
+        if (screen_nr > 0 && screen_nr < nr_xinerama_screens)
         {
             screen = &xinerama_screens[screen_nr];
             printf("Using screen %d, %dx%d+%d+%d\n",
@@ -607,3 +648,20 @@ QString cutDownString(QString text, QFont *testFont, int maxwidth)
     return text;
 }
 
+long long stringToLongLong(const QString &str)
+{
+    long long retval = 0;
+    if (str != QString::null)
+    {
+        retval = strtoll(str.ascii(), NULL, 0);
+    }
+    return retval;
+}
+
+QString longLongToString(long long ll)
+{
+    char str[21];
+    snprintf(str, 20, "%lld", ll);
+    str[20] = '\0';
+    return str;
+}
