@@ -1,6 +1,8 @@
 #ifndef VOLUMECONTROL_H_
 #define VOLUMECONTROL_H_
 
+typedef enum { MUTE_OFF=0, MUTE_LEFT, MUTE_RIGHT, MUTE_BOTH } kMuteState;
+
 class VolumeControl
 {
   public:
@@ -14,6 +16,7 @@ class VolumeControl
     void SetMute(bool on);
     void ToggleMute(void);
     bool GetMute(void) { return mute; }
+    kMuteState IterateMutedChannels(void);
 
   private:
     int mixerfd;
@@ -22,6 +25,7 @@ class VolumeControl
     int control;
 
     bool mute;
+    kMuteState current_mute_state;
 };
 
 #endif
