@@ -10,7 +10,7 @@ using namespace std;
 XMLParse::XMLParse(void)
 {
     allTypes = new vector<LayerSet *>;
-} 
+}
 
 XMLParse::~XMLParse()
 {
@@ -29,7 +29,7 @@ bool XMLParse::LoadTheme(QDomElement &ele, QString winName, QString specialfile)
     usetrans = gContext->GetNumSetting("PlayBoxTransparency", 1);
 
     fontSizeType = gContext->GetSetting("ThemeFontSizeType", "default");
-    
+
     QString themepath = gContext->GetThemeDir();
     QString themefile = themepath + specialfile + "ui.xml";
 
@@ -81,7 +81,7 @@ bool XMLParse::LoadTheme(QDomElement &ele, QString winName, QString specialfile)
 
                 if (name == winName)
                 {
-		    ele = e;
+            ele = e;
                     return true;
                 }
             }
@@ -188,15 +188,15 @@ void XMLParse::parseFont(QDomElement &element)
 
     if (fontSizeType == "small")
     {
-        if(sizeSmall > 0)
+        if (sizeSmall > 0)
             size = sizeSmall;
     }
     else if (fontSizeType == "big")
     {
-        if(sizeBig > 0)
+        if (sizeBig > 0)
             size = sizeBig;
     }
-    
+
     if (size < 0)
     {
         cerr << "Error: font size must be > 0\n";
@@ -305,6 +305,105 @@ void XMLParse::parseImage(LayerSet *container, QDomElement &element)
     container->bumpUpLayers(order.toInt());
 }
 
+void XMLParse::parseAnimatedImage(LayerSet *container, QDomElement &element)
+{
+    int context = -1;
+    QString name = element.attribute("name", "");
+    if (name.isNull() || name.isEmpty())
+    {
+        cerr << "Animated Image needs a name\n";
+        exit(-46);
+    }
+
+    QString order = element.attribute("draworder", "");
+    if (order.isNull() || order.isEmpty())
+    {
+        cerr << "Animated Image needs an order\n";
+        exit(-47);
+    }
+
+    QString filename = "";
+    QPoint pos = QPoint(0, 0);
+
+    QPoint scale = QPoint(-1, -1);
+    QPoint skipin = QPoint(0, 0);
+    QString interval, startinterval, imagecount;
+
+    for (QDomNode child = element.firstChild(); !child.isNull();
+         child = child.nextSibling())
+    {
+        QDomElement info = child.toElement();
+        if (!info.isNull())
+        {
+            if (info.tagName() == "context")
+            {
+                context = getFirstText(info).toInt();
+            }
+            else if (info.tagName() == "filename")
+            {
+                filename = getFirstText(info);
+            }
+            else if (info.tagName() == "position")
+            {
+                pos = parsePoint(getFirstText(info));
+                pos.setX((int)(pos.x() * wmult));
+                pos.setY((int)(pos.y() * hmult));
+            }
+            else if (info.tagName() == "staticsize")
+            {
+                scale = parsePoint(getFirstText(info));
+            }
+            else if (info.tagName() == "skipin")
+            {
+                skipin = parsePoint(getFirstText(info));
+                skipin.setX((int)(skipin.x() * wmult));
+                skipin.setY((int)(skipin.y() * hmult));
+            }
+            else if (info.tagName() == "interval")
+            {
+                interval = getFirstText(info);
+            }
+            else if (info.tagName() == "startinterval")
+            {
+                startinterval = getFirstText(info);
+            }
+            else if (info.tagName() == "imagecount")
+            {
+                imagecount = getFirstText(info);
+            }
+            else
+            {
+                cerr << "Unknown: " << info.tagName() << " in image\n";
+                exit(-49);
+            }
+        }
+    }
+
+    UIAnimatedImageType *image = new UIAnimatedImageType(name, filename, imagecount.toInt(),
+        interval.toInt(), startinterval.toInt(), order.toInt(), pos);
+    image->SetScreen(wmult, hmult);
+    if (scale.x() != -1 || scale.y() != -1)
+        image->SetSize(scale.x(), scale.y());
+    image->SetSkip(skipin.x(), skipin.y());
+    QString flex = element.attribute("fleximage", "");
+    if (!flex.isNull() && !flex.isEmpty())
+    {
+        if (flex.lower() == "yes")
+            image->SetFlex(true);
+        else
+            image->SetFlex(false);
+    }
+
+    //image->LoadImage();
+    if (context != -1)
+    {
+        image->SetContext(context);
+    }
+    image->SetParent(container);
+    container->AddType(image);
+    container->bumpUpLayers(order.toInt());
+}
+
 void XMLParse::parseRepeatedImage(LayerSet *container, QDomElement &element)
 {
     int orientation = 0;
@@ -362,19 +461,19 @@ void XMLParse::parseRepeatedImage(LayerSet *container, QDomElement &element)
             else if (info.tagName() == "orientation")
             {
                 QString orient_string = getFirstText(info).lower();
-                if(orient_string == "lefttoright")
+                if (orient_string == "lefttoright")
                 {
                     orientation = 0;
                 }
-                if(orient_string == "righttoleft")
+                if (orient_string == "righttoleft")
                 {
                     orientation = 1;
                 }
-                if(orient_string == "bottomtotop")
+                if (orient_string == "bottomtotop")
                 {
                     orientation = 2;
                 }
-                if(orient_string == "toptobottom")
+                if (orient_string == "toptobottom")
                 {
                     orientation = 3;
                 }
@@ -520,9 +619,9 @@ void XMLParse::parseGuideGrid(LayerSet *container, QDomElement &element)
                     inttype = 2;
                 else if (typ == "ChannelRecord")
                     inttype = 3;
-                else if (typ == "AllRecord") 
+                else if (typ == "AllRecord")
                     inttype = 4;
-                else if (typ == "WeekslotRecord") 
+                else if (typ == "WeekslotRecord")
                     inttype = 5;
                 else if (typ == "FindOneRecord")
                     inttype = 6;
@@ -540,7 +639,7 @@ void XMLParse::parseGuideGrid(LayerSet *container, QDomElement &element)
 
                 if (dir == "left")
                     arrows[0] = imag;
-                else 
+                else
                     arrows[1] = imag;
             }
             else if (info.tagName() == "catcolor")
@@ -605,7 +704,7 @@ void XMLParse::parseGuideGrid(LayerSet *container, QDomElement &element)
         else if (align.lower() == "hcenter")
             guide->SetJustification(Qt::AlignHCenter | jst);
     }
-    else 
+    else
         guide->SetJustification(jst);
 
     align = "";
@@ -712,7 +811,7 @@ void XMLParse::parseBar(LayerSet *container, QDomElement &element)
     bar->SetTextOffset(textoff);
     bar->SetIconOffset(iconoff);
     bar->SetIconSize(iconsize);
-    if (orientation == "horizontal") 
+    if (orientation == "horizontal")
        bar->SetOrientation(1);
     else if (orientation == "vertical")
        bar->SetOrientation(2);
@@ -729,7 +828,7 @@ void XMLParse::parseBar(LayerSet *container, QDomElement &element)
             bar->SetJustification(Qt::AlignVCenter);
         else if (align.lower() == "hcenter")
             bar->SetJustification(Qt::AlignHCenter);
-    
+
     }
     align = "";
 
@@ -824,6 +923,10 @@ void XMLParse::parseContainer(QDomElement &element, QString &newname, int &conte
             {
                 parseImage(container, info);
             }
+            else if (info.tagName() == "animatedimage")
+            {
+                parseAnimatedImage(container, info);
+            }
             else if (info.tagName() == "repeatedimage")
             {
                 parseRepeatedImage(container, info);
@@ -962,12 +1065,12 @@ void XMLParse::parseTextArea(LayerSet *container, QDomElement &element)
             }
             else if (info.tagName() == "value")
             {
-                if ((value.isNull() || value.isEmpty()) && 
+                if ((value.isNull() || value.isEmpty()) &&
                     info.attribute("lang","") == "")
                 {
                     value = qApp->translate("ThemeUI", getFirstText(info));
                 }
-                else if (info.attribute("lang","").lower() == 
+                else if (info.attribute("lang","").lower() ==
                          gContext->GetLanguage())
                 {
                     value = getFirstText(info);
@@ -1084,23 +1187,23 @@ void XMLParse::parseMultiTextArea(LayerSet *container, QDomElement &element)
             {
                 context = getFirstText(info).toInt();
             }
-            else if(info.tagName() == "padding")
+            else if (info.tagName() == "padding")
             {
                 padding = getFirstText(info).toInt();
             }
-            else if(info.tagName() == "dropdelay")
+            else if (info.tagName() == "dropdelay")
             {
                 drop_delay = getFirstText(info).toInt();
             }
-            else if(info.tagName() == "droppause")
+            else if (info.tagName() == "droppause")
             {
                 drop_pause = getFirstText(info).toInt();
             }
-            else if(info.tagName() == "scrolldelay")
+            else if (info.tagName() == "scrolldelay")
             {
                 scroll_delay = getFirstText(info).toInt();
             }
-            else if(info.tagName() == "scrollpause")
+            else if (info.tagName() == "scrollpause")
             {
                 scroll_pause = getFirstText(info).toInt();
             }
@@ -1130,8 +1233,8 @@ void XMLParse::parseMultiTextArea(LayerSet *container, QDomElement &element)
             }
             else
             {
-                cerr << "Unknown tag in multitext area: " 
-                     << info.tagName() 
+                cerr << "Unknown tag in multitext area: "
+                     << info.tagName()
                      << endl;
                 return;
             }
@@ -1145,7 +1248,7 @@ void XMLParse::parseMultiTextArea(LayerSet *container, QDomElement &element)
         return;
     }
 
-    UIMultiTextType *multitext = new UIMultiTextType(name, testfont, draworder, 
+    UIMultiTextType *multitext = new UIMultiTextType(name, testfont, draworder,
                                       area, altArea);
     multitext->SetScreen(wmult, hmult);
     if (context != -1)
@@ -1153,27 +1256,27 @@ void XMLParse::parseMultiTextArea(LayerSet *container, QDomElement &element)
         multitext->SetContext(context);
     }
 
-    if(padding > -1)
+    if (padding > -1)
     {
         multitext->setMessageSpacePadding(padding);
     }
-    if(drop_delay > -1)
+    if (drop_delay > -1)
     {
         multitext->setDropTimingLength(drop_delay);
     }
-    if(drop_pause > -1)
+    if (drop_pause > -1)
     {
         multitext->setDropTimingPause(drop_pause);
     }
-    if(scroll_delay > -1)
+    if (scroll_delay > -1)
     {
         multitext->setScrollTimingLength(scroll_delay);
     }
-    if(scroll_pause > -1)
+    if (scroll_pause > -1)
     {
         multitext->setScrollTimingPause(scroll_pause);
     }
-    
+
     multitext->SetParent(container);
     multitext->calculateScreenArea();
     container->AddType(multitext);
@@ -1202,9 +1305,14 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
     QPoint uparrow_loc;
     QPoint dnarrow_loc;
     QPoint select_loc;
+    QPoint rightarrow_loc;
+    QPoint leftarrow_loc;
+
     QPixmap *uparrow_img = NULL;
     QPixmap *dnarrow_img = NULL;
     QPixmap *select_img = NULL;
+    QPixmap *right_img = NULL;
+    QPixmap *left_img = NULL;
 
     QString name = element.attribute("name", "");
     if (name.isNull() || name.isEmpty())
@@ -1261,13 +1369,13 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
                 fontfcn = info.attribute("function", "");
 
                 if (fontname.isNull() || fontname.isEmpty())
-                {   
+                {
                     cerr << "FcnFont needs a name\n";
                     return;
                 }
 
                 if (fontfcn.isNull() || fontfcn.isEmpty())
-                {   
+                {
                     cerr << "FcnFont needs a function\n";
                     return;
                 }
@@ -1279,9 +1387,9 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
                 QString fillcolor = "";
                 QString fillarea = "";
                 QString filltype = "";
- 
+
                 fillfcn = info.attribute("function", "");
-                fillcolor = info.attribute("color", "#ffffff"); 
+                fillcolor = info.attribute("color", "#ffffff");
                 fillarea = info.attribute("area", "");
                 filltype = info.attribute("type", "");
 
@@ -1302,7 +1410,7 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
 
                 if (fillarea.isNull() || fillarea.isEmpty())
                 {
-                    fill_select_area = area; 
+                    fill_select_area = area;
                 }
                 else
                 {
@@ -1310,7 +1418,7 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
                     normalizeRect(fill_select_area);
                 }
                 fill_select_color = QColor(fillcolor);
- 
+
             }
             else if (info.tagName() == "image")
             {
@@ -1332,6 +1440,8 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
                     return;
                 }
 
+
+
                 imgpoint = info.attribute("location", "");
                 if (imgpoint.isNull() && imgpoint.isEmpty())
                 {
@@ -1360,6 +1470,22 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
                     dnarrow_loc.setX((int)(dnarrow_loc.x() * wmult));
                     dnarrow_loc.setY((int)(dnarrow_loc.y() * hmult));
                 }
+                if (imgname.lower() == "leftarrow")
+                {
+                    left_img = gContext->LoadScalePixmap(imgfile);
+                    leftarrow_loc = parsePoint(imgpoint);
+                    leftarrow_loc.setX((int)(leftarrow_loc.x() * wmult));
+                    leftarrow_loc.setY((int)(leftarrow_loc.y() * hmult));
+
+                }
+                else if (imgname.lower() == "rightarrow")
+                {
+                    right_img = gContext->LoadScalePixmap(imgfile);
+                    rightarrow_loc = parsePoint(imgpoint);
+                    rightarrow_loc.setX((int)(rightarrow_loc.x() * wmult));
+                    rightarrow_loc.setY((int)(rightarrow_loc.y() * hmult));
+                }
+
             }
             else if (info.tagName() == "column")
             {
@@ -1409,21 +1535,35 @@ void XMLParse::parseListArea(LayerSet *container, QDomElement &element)
         list->SetImageSelection(*select_img, select_loc);
         delete select_img;
     }
+
     if (uparrow_img)
     {
         list->SetImageUpArrow(*uparrow_img, uparrow_loc);
         delete uparrow_img;
     }
+
     if (dnarrow_img)
     {
         list->SetImageDownArrow(*dnarrow_img, dnarrow_loc);
         delete dnarrow_img;
     }
 
+    if (right_img)
+    {
+        list->SetImageRightArrow(*right_img, rightarrow_loc);
+        delete right_img;
+    }
+
+    if (left_img)
+    {
+        list->SetImageLeftArrow(*left_img, leftarrow_loc);
+        delete left_img;
+    }
+
     typedef QMap<QString,QString> fontdata;
     fontdata::Iterator it;
     for ( it = fontFunctions.begin(); it != fontFunctions.end(); ++it )
-    {   
+    {
         fontProp *testFont = GetFont(it.data());
         if (testFont)
             theFonts[it.data()] = *testFont;
@@ -1470,7 +1610,7 @@ void XMLParse::parseStatusBar(LayerSet *container, QDomElement &element)
     int imgFillSpace = 0;
     QPixmap *imgFiller = NULL;
     QPixmap *imgContainer = NULL;
- 
+
     QString name = element.attribute("name", "");
     if (name.isNull() || name.isEmpty())
     {
@@ -1553,19 +1693,19 @@ void XMLParse::parseStatusBar(LayerSet *container, QDomElement &element)
             else if (info.tagName() == "orientation")
             {
                 QString orient_string = getFirstText(info).lower();
-                if(orient_string == "lefttoright")
+                if (orient_string == "lefttoright")
                 {
                     orientation = 0;
                 }
-                if(orient_string == "righttoleft")
+                if (orient_string == "righttoleft")
                 {
                     orientation = 1;
                 }
-                if(orient_string == "bottomtotop")
+                if (orient_string == "bottomtotop")
                 {
                     orientation = 2;
                 }
-                if(orient_string == "toptobottom")
+                if (orient_string == "toptobottom")
                 {
                     orientation = 3;
                 }
@@ -1584,7 +1724,7 @@ void XMLParse::parseStatusBar(LayerSet *container, QDomElement &element)
     if (imgContainer)
     {
         sb->SetContainerImage(*imgContainer);
-        delete imgContainer; 
+        delete imgContainer;
     }
     if (imgFiller)
     {
@@ -1623,11 +1763,11 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
     typedef QMap<int, QRect> CornerMap;
     CornerMap bin_corners;
     bin_corners.clear();
-    
+
     //
     //  Some maps to store fonts as we parse
     //
-    
+
     QMap<QString, QString> fontFunctions;
     QMap<QString, fontProp> theFonts;
 
@@ -1662,7 +1802,7 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                 area = parseRect(getFirstText(info));
                 normalizeRect(area);
             }
-            else if(info.tagName() == "image")
+            else if (info.tagName() == "image")
             {
                 QString imgname = "";
                 QString file = "";
@@ -1680,8 +1820,8 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                     cerr << "Image needs a filename\n";
                     return;
                 }
-    
-                if(info.tagName() == "context")
+
+                if (info.tagName() == "context")
                 {
                     context = getFirstText(info).toInt();
                 }
@@ -1689,7 +1829,7 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                 if (imgname.lower() == "selectionbar")
                 {
                     select_img = gContext->LoadScalePixmap(file);
-                    if(!select_img)
+                    if (!select_img)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -1697,7 +1837,7 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                 else if (imgname.lower() == "uparrow")
                 {
                     uparrow_img = gContext->LoadScalePixmap(file);
-                    if(!uparrow_img)
+                    if (!uparrow_img)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -1705,7 +1845,7 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                 else if (imgname.lower() == "downarrow")
                 {
                     downarrow_img = gContext->LoadScalePixmap(file);
-                    if(!downarrow_img)
+                    if (!downarrow_img)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -1713,7 +1853,7 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                 else if (imgname.lower() == "leftarrow")
                 {
                     leftarrow_img = gContext->LoadScalePixmap(file);
-                    if(!leftarrow_img)
+                    if (!leftarrow_img)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -1721,7 +1861,7 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                 else if (imgname.lower() == "rightarrow")
                 {
                     rightarrow_img = gContext->LoadScalePixmap(file);
-                    if(!rightarrow_img)
+                    if (!rightarrow_img)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -1731,16 +1871,16 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                     cerr << "xmlparse.o: I don't know what to do with an image tag who's function is " << imgname << endl;
                 }
             }
-            else if(info.tagName() == "bin")
+            else if (info.tagName() == "bin")
             {
                 QString whichbin_string = info.attribute("number", "");
                 int whichbin = whichbin_string.toInt();
-                if(whichbin < 1)
+                if (whichbin < 1)
                 {
                     cerr << "xmlparse.o: Bad setting for bin number in bin tag" << endl;
                     return;
                 }
-                if(whichbin > bins + 1)
+                if (whichbin > bins + 1)
                 {
                     cerr << "xmlparse.o: Attempt to set bin with a reference larger than number of bins" << endl;
                     return;
@@ -1766,13 +1906,13 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
                             fontfcn = info.attribute("function", "");
 
                             if (fontname.isNull() || fontname.isEmpty())
-                            {   
+                            {
                                 cerr << "FcnFont needs a name\n";
                                 return;
                             }
 
                             if (fontfcn.isNull() || fontfcn.isEmpty())
-                            {   
+                            {
                                 cerr << "FcnFont needs a function\n";
                                 return;
                             }
@@ -1812,7 +1952,7 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
     typedef QMap<QString,QString> fontdata;
     fontdata::Iterator it;
     for ( it = fontFunctions.begin(); it != fontFunctions.end(); ++it )
-    {   
+    {
         fontProp *testFont = GetFont(it.data());
         if (testFont)
             theFonts[it.data()] = *testFont;
@@ -1836,7 +1976,7 @@ void XMLParse::parseManagedTreeList(LayerSet *container, QDomElement &element)
     if (!rightarrow_img)
         rightarrow_img = new QPixmap();
 
-    mtl->setArrowImages(*uparrow_img, *downarrow_img, *leftarrow_img, 
+    mtl->setArrowImages(*uparrow_img, *downarrow_img, *leftarrow_img,
                         *rightarrow_img);
 
     delete uparrow_img;
@@ -1887,7 +2027,7 @@ void XMLParse::parsePushButton(LayerSet *container, QDomElement &element)
                 pos.setX((int)(pos.x() * wmult));
                 pos.setY((int)(pos.y() * hmult));
             }
-            else if(info.tagName() == "image")
+            else if (info.tagName() == "image")
             {
                 QString imgname = "";
                 QString file = "";
@@ -1917,7 +2057,7 @@ void XMLParse::parsePushButton(LayerSet *container, QDomElement &element)
                 if (imgname.lower() == "off")
                 {
                     image_off = gContext->LoadScalePixmap(file);
-                    if(!image_off)
+                    if (!image_off)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -1925,7 +2065,7 @@ void XMLParse::parsePushButton(LayerSet *container, QDomElement &element)
                 if (imgname.lower() == "pushed")
                 {
                     image_pushed = gContext->LoadScalePixmap(file);
-                    if(!image_pushed)
+                    if (!image_pushed)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -1946,7 +2086,7 @@ void XMLParse::parsePushButton(LayerSet *container, QDomElement &element)
     if (!image_pushed)
         image_pushed = new QPixmap();
 
-    UIPushButtonType *pbt = new UIPushButtonType(name, *image_on, *image_off, 
+    UIPushButtonType *pbt = new UIPushButtonType(name, *image_on, *image_off,
                                                  *image_pushed);
 
     delete image_on;
@@ -2008,7 +2148,7 @@ void XMLParse::parseTextButton(LayerSet *container, QDomElement &element)
             {
                 font = getFirstText(info);
             }
-            else if(info.tagName() == "image")
+            else if (info.tagName() == "image")
             {
                 QString imgname = "";
                 QString file = "";
@@ -2030,7 +2170,7 @@ void XMLParse::parseTextButton(LayerSet *container, QDomElement &element)
                 if (imgname.lower() == "on")
                 {
                     image_on = gContext->LoadScalePixmap(file);
-                    if(!image_on)
+                    if (!image_on)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -2038,7 +2178,7 @@ void XMLParse::parseTextButton(LayerSet *container, QDomElement &element)
                 if (imgname.lower() == "off")
                 {
                     image_off = gContext->LoadScalePixmap(file);
-                    if(!image_off)
+                    if (!image_off)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -2047,7 +2187,7 @@ void XMLParse::parseTextButton(LayerSet *container, QDomElement &element)
                 {
                     image_pushed = gContext->LoadScalePixmap(file);
 
-                    if(!image_pushed)
+                    if (!image_pushed)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -2075,7 +2215,7 @@ void XMLParse::parseTextButton(LayerSet *container, QDomElement &element)
     if (!image_pushed)
         image_pushed = new QPixmap();
 
-    UITextButtonType *tbt = new UITextButtonType(name, *image_on, *image_off, 
+    UITextButtonType *tbt = new UITextButtonType(name, *image_on, *image_off,
                                                  *image_pushed);
 
     delete image_on;
@@ -2134,7 +2274,7 @@ void XMLParse::parseCheckBox(LayerSet *container, QDomElement &element)
                 pos.setX((int)(pos.x() * wmult));
                 pos.setY((int)(pos.y() * hmult));
             }
-            else if(info.tagName() == "image")
+            else if (info.tagName() == "image")
             {
                 QString imgname = "";
                 QString file = "";
@@ -2156,7 +2296,7 @@ void XMLParse::parseCheckBox(LayerSet *container, QDomElement &element)
                 if (imgname.lower() == "checked")
                 {
                     image_checked = gContext->LoadScalePixmap(file);
-                    if(!image_checked)
+                    if (!image_checked)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -2164,7 +2304,7 @@ void XMLParse::parseCheckBox(LayerSet *container, QDomElement &element)
                 if (imgname.lower() == "unchecked")
                 {
                     image_unchecked = gContext->LoadScalePixmap(file);
-                    if(!image_unchecked)
+                    if (!image_unchecked)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -2172,7 +2312,7 @@ void XMLParse::parseCheckBox(LayerSet *container, QDomElement &element)
                 if (imgname.lower() == "checked_high")
                 {
                     image_checked_high = gContext->LoadScalePixmap(file);
-                    if(!image_checked_high)
+                    if (!image_checked_high)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -2180,7 +2320,7 @@ void XMLParse::parseCheckBox(LayerSet *container, QDomElement &element)
                 if (imgname.lower() == "unchecked_high")
                 {
                     image_unchecked_high = gContext->LoadScalePixmap(file);
-                    if(!image_unchecked_high)
+                    if (!image_unchecked_high)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -2203,8 +2343,8 @@ void XMLParse::parseCheckBox(LayerSet *container, QDomElement &element)
     if (!image_unchecked_high)
         image_unchecked_high = new QPixmap();
 
-    UICheckBoxType *cbt = new UICheckBoxType(name, 
-                                             *image_checked, *image_unchecked, 
+    UICheckBoxType *cbt = new UICheckBoxType(name,
+                                             *image_checked, *image_unchecked,
                                              *image_checked_high, *image_unchecked_high);
 
     delete image_checked;
@@ -2266,7 +2406,7 @@ void XMLParse::parseSelector(LayerSet *container, QDomElement &element)
             {
                 font = getFirstText(info);
             }
-            else if(info.tagName() == "image")
+            else if (info.tagName() == "image")
             {
                 QString imgname = "";
                 QString file = "";
@@ -2288,7 +2428,7 @@ void XMLParse::parseSelector(LayerSet *container, QDomElement &element)
                 if (imgname.lower() == "on")
                 {
                     image_on = gContext->LoadScalePixmap(file);
-                    if(!image_on)
+                    if (!image_on)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -2296,7 +2436,7 @@ void XMLParse::parseSelector(LayerSet *container, QDomElement &element)
                 if (imgname.lower() == "off")
                 {
                     image_off = gContext->LoadScalePixmap(file);
-                    if(!image_off)
+                    if (!image_off)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -2305,7 +2445,7 @@ void XMLParse::parseSelector(LayerSet *container, QDomElement &element)
                 {
                     image_pushed = gContext->LoadScalePixmap(file);
 
-                    if(!image_pushed)
+                    if (!image_pushed)
                     {
                         cerr << "xmparse.o: I can't find a file called " << file << endl ;
                     }
@@ -2333,7 +2473,7 @@ void XMLParse::parseSelector(LayerSet *container, QDomElement &element)
     if (!image_pushed)
         image_pushed = new QPixmap();
 
-    UISelectorType *st = new UISelectorType(name, *image_on, *image_off, 
+    UISelectorType *st = new UISelectorType(name, *image_on, *image_off,
                                                  *image_pushed, area);
 
     delete image_on;
@@ -2422,7 +2562,7 @@ void XMLParse::parseListBtnArea(LayerSet *container, QDomElement &element)
         cerr << "ListBtn area needs a draworder\n";
         return;
     }
-    
+
     draworder = layerNum.toInt();
     for (QDomNode child = element.firstChild(); !child.isNull();
          child = child.nextSibling()) {
@@ -2445,7 +2585,7 @@ void XMLParse::parseListBtnArea(LayerSet *container, QDomElement &element)
                     fontInactive = fontFcn;
                 else {
                     std::cerr << "Unknown font function for listbtn area: "
-                              << fontFcn 
+                              << fontFcn
                               << std::endl;
                     return;
                 }
@@ -2501,7 +2641,7 @@ void XMLParse::parseListBtnArea(LayerSet *container, QDomElement &element)
                           << info.tagName() << endl;
                 return;
             }
-            
+
         }
     }
 
@@ -2512,7 +2652,7 @@ void XMLParse::parseListBtnArea(LayerSet *container, QDomElement &element)
              << " in listbtn area: " << name << endl;
         return;
     }
-    
+
     fontProp *fpInactive = GetFont(fontInactive);
     if (!fpInactive)
     {
@@ -2520,7 +2660,7 @@ void XMLParse::parseListBtnArea(LayerSet *container, QDomElement &element)
              << " in listbtn area: " << name << endl;
         return;
     }
-    
+
 
     UIListBtnType *l = new UIListBtnType(name, area, draworder, showArrow,
                                          showScrollArrows);
@@ -2689,7 +2829,7 @@ void XMLParse::parseListTreeArea(LayerSet *container, QDomElement &element)
     l->SetMargin((int)(margin*wmult));
     l->SetParent(container);
     l->calculateScreenArea();
-    
+
     container->AddType(l);
     container->bumpUpLayers(0);
 }
