@@ -29,7 +29,7 @@ Channel::Channel(TVRec *parent, const QString &videodevice, bool strength)
     usingv4l2 = false;
     videomode = VIDEO_MODE_NTSC;
     currentFormat = "";
-    usingstrength = strength;
+    usingATSCstrength = strength;
 }
 
 Channel::~Channel(void)
@@ -197,6 +197,7 @@ void Channel::SetFormat(const QString &format)
 
     struct video_channel vc;
     memset(&vc, 0, sizeof(vc));
+    vc.channel = currentcapchannel;
     ioctl(videofd, VIDIOCGCHAN, &vc);
     vc.norm = mode;
     ioctl(videofd, VIDIOCSCHAN, &vc);
@@ -352,7 +353,7 @@ bool Channel::SetChannelByString(const QString &chan)
 
     // If CheckChannel filled in the inputName then we need to change inputs
     // and return, since the act of changing inputs will change the channel as well.
-    if(!inputName.isEmpty())
+    if (!inputName.isEmpty())
     {
         ChannelBase::SwitchToInput(inputName, chan);
         return true;
@@ -403,6 +404,16 @@ bool Channel::SetChannelByString(const QString &chan)
 
     SetFormat(tvformat);
 
+    curchannelname = chan;
+
+    pParent->SetVideoFiltersForChannel(this, chan);
+    SetContrast();
+    SetColour();
+    SetBrightness();
+    SetHue();
+
+    inputChannel[currentcapchannel] = curchannelname;
+
     // Tune
     if (externalChanger[currentcapchannel].isEmpty())
     {
@@ -421,16 +432,6 @@ bool Channel::SetChannelByString(const QString &chan)
     else if (!ChangeExternalChannel(freqid))
         return false;
 
-    curchannelname = chan;
-
-    pParent->SetVideoFiltersForChannel(this, chan);
-    SetContrast();
-    SetColour();
-    SetBrightness();
-    SetHue();
-
-    inputChannel[currentcapchannel] = curchannelname;
-
     return true;
 }
 
@@ -466,7 +467,7 @@ int signalStrengthATSC_v4l2(int device, int input)
     int ioctlval = ioctl(device,VIDIOC_G_TUNER, &vsig);
     if (ioctlval == -1)
     {
-        perror("VIDIOC_G_TUNER problem in channel.h's signalStrengthATSC()");
+	perror("VIDIOC_G_TUNER");
         return 0;
     }
     VERBOSE(VB_CHANNEL, QString("signal strength: %1").arg(vsig.signal));
@@ -480,7 +481,7 @@ bool Channel::CheckSignal(int msecTotal, int reqSignal, int input)
 
     msecTotal = max(msecSleep, msecTotal);
 
-    if (usingstrength) 
+    if (usingATSCstrength) 
     {
         VERBOSE(VB_CHANNEL, QString("CheckSignal(%1, %2, %3) usingv4l2(%4)")
                 .arg(msecTotal).arg(reqSignal).arg(input).arg(usingv4l2));
@@ -528,7 +529,7 @@ bool Channel::CheckSignalFull(void)
 {
     int signalThresholdWait = 5000;
     int signalThreshold = 65;
-    if (usingstrength)
+    if (usingATSCstrength)
     {
         signalThresholdWait = gContext->GetNumSetting("ATSCCheckSignalWait",
                                                       5000);
@@ -545,7 +546,7 @@ bool Channel::TuneToFrequency(int frequency)
 
     int signalThresholdWait = 5000;
     int signalThreshold = 65;
-    if (usingstrength) 
+    if (usingATSCstrength) 
     {
         signalThresholdWait = gContext->GetNumSetting("ATSCCheckSignalWait", 
                                                       5000);
@@ -565,6 +566,8 @@ bool Channel::TuneToFrequency(int frequency)
             perror("VIDIOC_S_FREQUENCY");
             return false;
         }
+        
+
         return CheckSignal(signalThresholdWait, signalThreshold,
                            currentcapchannel);
     }
