@@ -105,12 +105,18 @@ void ChannelListSetting::fillSelections(void)
     QString currentValue = getValue();
     clearSelections();
     addSelection("(New Channel)");
-    QString querystr = QString("SELECT name, channum, chanid FROM channel");
-    if (currentSourceID != "")
-        querystr += QString(" WHERE sourceid='%1'").arg(currentSourceID);
+    QString querystr = "SELECT channel.name,channum,chanid ";
+    if ((currentSourceID == "") ||
+        (currentSourceID == "Unassigned"))
+        querystr += ",videosource.name FROM channel "
+                    "LEFT JOIN videosource ON "
+                    "(channel.sourceid = videosource.sourceid) ";
+    else
+        querystr += QString("FROM channel WHERE sourceid='%1' ")
+                           .arg(currentSourceID);
         
     if (currentSortMode == QObject::tr("Channel Name"))
-        querystr += " ORDER BY name";
+        querystr += " ORDER BY channel.name";
     else if (currentSortMode == QObject::tr("Channel Number"))
         querystr += " ORDER BY channum + 0";
 
@@ -120,7 +126,15 @@ void ChannelListSetting::fillSelections(void)
             QString name = QString::fromUtf8(query.value(0).toString());
             QString channum = query.value(1).toString();
             QString chanid = query.value(2).toString();
-             
+            QString sourceid = "Unassigned";
+			
+            if (!query.value(3).toString().isNull())
+            {
+                sourceid = query.value(3).toString();
+                if (currentSourceID == "Unassigned")
+                    continue;
+            }
+ 
             if (channum == "" && currentHideMode) continue;
 
             if (name == "") name = "(Unnamed : " + chanid + ")";
@@ -134,6 +148,10 @@ void ChannelListSetting::fillSelections(void)
                     name = "???. " + name;
             }
 
+            if ((currentSourceID == "") &&
+                (currentSourceID != "Unassigned"))
+                name += " (" + sourceid  + ")";
+
             addSelection(name, chanid, (chanid == currentValue) ? true : false);
         }
 }
@@ -154,6 +172,7 @@ public:
             while(query.next())
                 addSelection(query.value(0).toString(),
                              query.value(1).toString());
+        addSelection(QObject::tr("(Unassigned)"),"Unassigned");
     };
 };
 
