@@ -445,7 +445,8 @@ void DataDirectProcessor::grabData(bool plineupsOnly, QDateTime pstartDate,
     if (mkstemp(ctempfilename) == -1) 
     {
          perror("mkstemp");
-         exit(1);
+         VERBOSE(VB_IMPORTANT, "DDP: error creating temp files");
+         exit(-12);
     }
 
     QString tmpfilename = QString(ctempfilename);
@@ -475,7 +476,7 @@ void DataDirectProcessor::grabData(bool plineupsOnly, QDateTime pstartDate,
 
     QString command = QString("wget --http-user='%1' --http-passwd='%2' "
                               "--post-file='%3' --header='Accept-Encoding:gzip'"
-                              " %4 --output-document=- | gzip -d")
+                              " %4 --output-document=- | gzip -df")
                              .arg(getUserID())
                              .arg(getPassword())
                              .arg(tmpfilename)
