@@ -8,7 +8,7 @@ using namespace std;
 
 #include "mythcontext.h"
 
-const QString currentDatabaseVersion = "1049";
+const QString currentDatabaseVersion = "1055";
 
 void UpdateDBVersionNumber(const QString &newnumber)
 {
@@ -41,7 +41,7 @@ void performActualUpdate(const QString updates[], QString version,
                  << thequery << endl
                  << MythContext::DBErrorMessage(db_conn->lastError()) << endl;
 
-            exit(-1);
+            exit(-13);
         }
 
         counter++;
@@ -917,6 +917,84 @@ QString("ALTER TABLE videosource ADD COLUMN freqtable VARCHAR(16) NOT NULL DEFAU
 };
         performActualUpdate(updates, "1049", dbver);
     }
+
+    if (dbver == "1049")
+    {
+        const QString updates[] = {
+"ALTER TABLE keyword ADD COLUMN searchtype INT UNSIGNED NOT NULL DEFAULT 3;",
+"ALTER TABLE keyword DROP INDEX phrase;",
+"ALTER TABLE keyword DROP PRIMARY KEY;",
+"ALTER TABLE keyword ADD UNIQUE(phrase,searchtype);",
+""
+};
+        performActualUpdate(updates, "1050", dbver);
+    }
+    
+
+    if (dbver == "1050")
+    {
+        const QString updates[] = {
+"ALTER TABLE recorded CHANGE starttime starttime DATETIME NOT NULL;",
+"ALTER TABLE recorded CHANGE endtime endtime DATETIME NOT NULL;",
+"ALTER TABLE recorded ADD COLUMN lastmodified TIMESTAMP NOT NULL;",
+"ALTER TABLE recorded ADD COLUMN filesize BIGINT(20) DEFAULT 0 NOT NULL;",
+"ALTER TABLE credits CHANGE starttime starttime DATETIME NOT NULL;",
+"ALTER TABLE oldprogram CHANGE airdate airdate DATETIME NOT NULL;",
+"ALTER TABLE oldrecorded CHANGE starttime starttime DATETIME NOT NULL;",
+"ALTER TABLE oldrecorded CHANGE endtime endtime DATETIME NOT NULL;",
+"ALTER TABLE program CHANGE starttime starttime DATETIME NOT NULL;",
+"ALTER TABLE program CHANGE endtime endtime DATETIME NOT NULL;",
+"ALTER TABLE programgenres CHANGE starttime starttime DATETIME NOT NULL;",
+"ALTER TABLE programrating CHANGE starttime starttime DATETIME NOT NULL;",
+"ALTER TABLE recordedmarkup CHANGE starttime starttime DATETIME NOT NULL;",
+"ALTER TABLE recordoverride CHANGE starttime starttime DATETIME NOT NULL;",
+"ALTER TABLE recordoverride CHANGE endtime endtime DATETIME NOT NULL;",
+"ALTER TABLE transcoding CHANGE starttime starttime DATETIME NOT NULL;",
+""
+};
+        performActualUpdate(updates, "1051", dbver);
+    }
+
+    if (dbver == "1051")
+    {
+        const QString updates[] = {
+"update record set dupmethod = 6 where dupmethod = 0;",
+"update record set dupin = 15 where dupin = 0;",
+""
+};
+        performActualUpdate(updates, "1052", dbver);
+    }
+
+    if (dbver == "1052")
+    {
+        const QString updates[] = {
+"ALTER TABLE recorded ADD COLUMN stars FLOAT UNSIGNED NOT NULL DEFAULT 0;",
+"ALTER TABLE recorded ADD COLUMN previouslyshown TINYINT(1) DEFAULT 0;",
+"ALTER TABLE recorded ADD COLUMN originalairdate DATE;",
+"INSERT INTO settings VALUES ('HaveRepeats', '0', NULL);",
+""
+};
+        performActualUpdate(updates, "1053", dbver);
+    }
+
+    if (dbver == "1053")
+    {
+        const QString updates[] = {
+"ALTER TABLE channel CHANGE freqid freqid VARCHAR(10);",
+""
+};
+        performActualUpdate(updates, "1054", dbver);
+    }
+
+    if (dbver == "1054")
+    {
+        const QString updates[] = {
+"ALTER TABLE program ADD INDEX id_start_end (chanid,starttime,endtime);",
+"ALTER TABLE channel ADD INDEX channel_src (channum,sourceid);",
+""
+};
+        performActualUpdate(updates, "1055", dbver);
+    }
 }
 
 void InitializeDatabase(void)
@@ -929,7 +1007,7 @@ void InitializeDatabase(void)
              << "\r\nhas " << qQuery.numRowsAffected() << " tables.\r\n";
         cerr << "If you are sure this is a good mythtv database, verify\r\n"
              << "that the settings table has the DBSchemaVer variable.\r\n";
-        exit(1);
+        exit(-14);
     }
 
     VERBOSE(VB_ALL, "Inserting MythTV initial database information.");
