@@ -84,7 +84,11 @@ bool DVBCam::Open()
 
     GENERAL("CAM - Initialized successfully.");
 
-    pthread_create(&ciHandlerThread, NULL, CiHandlerThreadHelper, this);
+    pthread_attr_t attr;
+    pthread_attr_init(&attr);
+    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+
+    pthread_create(&ciHandlerThread, &attr, CiHandlerThreadHelper, this);
 
     return true;
 }
@@ -100,7 +104,7 @@ bool DVBCam::Close()
 
 void DVBCam::ChannelChanged(dvb_channel_t& chan)
 {
-    pthread_mutex_lock(&db_lock);
+    pthread_mutex_lock(db_lock);
 
     MythContext::KickDatabase(db);
     QSqlQuery query = db->exec(QString("SELECT pmtcache FROM dvb_channel"
@@ -131,7 +135,7 @@ void DVBCam::ChannelChanged(dvb_channel_t& chan)
         }
     }
 
-    pthread_mutex_unlock(&db_lock);
+    pthread_mutex_unlock(db_lock);
 }
 
 void DVBCam::ChannelChanged(dvb_channel_t& chan, uint8_t* pmt, int len)
@@ -166,7 +170,7 @@ void DVBCam::ChannelChanged(dvb_channel_t& chan, uint8_t* pmt, int len)
     ba.resize(len);
     memcpy(ba.data(), pmt, len);
 
-    pthread_mutex_lock(&db_lock);
+    pthread_mutex_lock(db_lock);
     QSqlQuery query(QString::null, db);
     query.prepare(QString("UPDATE dvb_channel SET pmtcache=?"
                   " WHERE serviceid=%1").arg(chan.serviceID));
@@ -174,16 +178,18 @@ void DVBCam::ChannelChanged(dvb_channel_t& chan, uint8_t* pmt, int len)
 
     MythContext::KickDatabase(db);
     query.exec();
-    pthread_mutex_unlock(&db_lock);
 
     if (!query.isActive())
     {
         MythContext::DBError("DVBCam PMT Update", query);
+        pthread_mutex_unlock(db_lock);
         return;
     }
 
     if (query.numRowsAffected() == 1)
         GENERAL("CAM - PMT cache updated.");
+
+    pthread_mutex_unlock(db_lock);
 }
 
 void *DVBCam::CiHandlerThreadHelper(void*self)
@@ -202,24 +208,81 @@ void DVBCam::CiHandlerLoop()
         if (!ciHandler->Process())
             continue;
 
+        if (ciHandler->NeedCaPmt())
+            first_send = true;
+
+        if (ciHandler->HasUserIO())
+        {
+            cCiEnquiry* enq = ciHandler->GetEnquiry();
+            if (enq != NULL)
+            {
+                if (enq->Text() != NULL)
+                    cerr << "Message from CAM: " << enq->Text() << endl;
+                delete enq;
+            }
+
+            cCiMenu* menu = ciHandler->GetMenu();
+            if (menu != NULL)
+            {
+                if (menu->TitleText() != NULL)
+                    cerr << "CAM Menu Title: " << menu->TitleText() << endl;
+                if (menu->SubTitleText() != NULL)
+                    cerr << "CAM Menu SubTitle: " << menu->SubTitleText() << endl;
+                if (menu->BottomText() != NULL)
+                    cerr << "CAM Menu BottomText: " << menu->BottomText() << endl;
+
+                for (int i=0; i<menu->NumEntries(); i++)
+                    if (menu->Entry(i) != NULL)
+                        cerr << "CAM Menu Entry(" << i << "): " << menu->Entry(i) << endl;
+
+                if (menu->Selectable())
+                {
+                    cerr << "Menu is selectable" << endl;
+                }
+
+                if (menu->NumEntries() > 0)
+                {
+                    cerr << "Selecting first entry" << endl;
+                    menu->Select(0);
+                }
+                else
+                {
+                    cerr << "Canceling menu" << endl;
+                    menu->Cancel();
+                }
+
+                delete menu;
+            }
+
+            first_send = true;
+        }
+
         for (int s=0; s<ciHandler->NumSlots(); s++)
         {
             const unsigned short *caids = ciHandler->GetCaSystemIds(s);
-            // TODO: Only set cam with correct id.
 
             pthread_mutex_lock(&pmt_lock);
             if (caids != NULL && pmtbuf != NULL)
             {
-                if (first_send)
+                if (!first_send)
                 {
+                    pthread_mutex_unlock(&pmt_lock);
+                    continue;
+                }
+
+                cCiCaPmt capmt(chan_opts.serviceID, CPLM_FIRST);
+
+                /* FIXME:
+                 * We have to make some kind of CAM association map like VDR has,
+                 * for now we just clear the CAM that has no provider support.
+                 */
+                if (FindCaDescriptors(capmt, caids, s))
+                {
+                    SetPids(capmt, chan_opts.pids);
                     GENERAL(QString("CAM - Sending PMT to slot %1.").arg(s));
                     first_send = false;
                 }
 
-                cCiCaPmt capmt(chan_opts.serviceID);
-                capmt.AddCaDescriptor(pmtlen, pmtbuf);
-                SetPids(capmt, chan_opts.pids);
-
                 ciHandler->SetCaPmt(capmt,s);
             }
             pthread_mutex_unlock(&pmt_lock);
@@ -250,3 +313,62 @@ void DVBCam::SetPids(cCiCaPmt& capmt, dvb_pids_t& pids)
         capmt.AddPid(pids.other[i]);
 }
 
+bool DVBCam::FindCaDescriptors(cCiCaPmt &capmt, const unsigned short *caids, int slot)
+{
+    bool found = false;
+    uint16_t prg_len = ((*(pmtbuf+2)&0x0f)<<8) | *(pmtbuf+3);
+    uint8_t *b = pmtbuf + 3;
+    uint8_t *e = pmtbuf + 4 + prg_len;
+
+    for ( ;; )
+    {
+        if (e+2 > pmtbuf+pmtlen)
+            break;
+
+        while ( prg_len > 0 && b+2 <= e )
+        {
+            uint8_t tag = *(b+1);
+            uint8_t len = *(b+2);
+
+            if (b+2+len > e)
+                break;
+
+            prg_len -= len - 2;
+
+            if (tag == 0x09)
+            {
+                uint16_t ca_system_id = ((*(b+3))<<8) | *(b+4);
+                uint16_t ca_pid = ((*(b+5))<<8) | *(b+6);
+                fprintf(stderr,"Found CA Descriptor (CASID=%0.4X,"
+                        " CAPID=%0.4X)\n", ca_system_id, ca_pid);
+
+                for (int i=0; caids[i] != 0; i++)
+                {
+                    if (ca_system_id == caids[i] && ca_pid != 0)
+                    {
+                        if (slot == 1 && ca_system_id == 0x0B00)
+                            continue;
+
+                        if (first_send)
+                            fprintf(stderr,"Adding CA Descriptor (CASID=%0.4X, "
+                                    "CAPID=%0.4X)\n", ca_system_id, ca_pid);
+
+                        capmt.AddCaDescriptor(len + 2, b+1);
+                        found = true;
+                    }
+                }
+            }
+            b += len + 2;
+        }
+
+        if (e+5 > pmtbuf+pmtlen)
+            break;
+
+        prg_len = ((*(e+3)&0x0f)<<8) | *(e+4);
+        b = e + 4;
+        e = b + 1 + prg_len;
+    }
+
+    return found;
+}
+
