@@ -58,9 +58,8 @@ using namespace std;
 DVBChannel::DVBChannel(int aCardNum, TVRec *parent)
            : ChannelBase(parent), cardnum(aCardNum)
 {
-    fd_frontend = 0;
+    fd_frontend = -1;
     force_channel_change = false;
-    first_tune = true;
     isOpen = false;
     diseqc = NULL;
     monitorRunning = false;
@@ -74,10 +73,6 @@ DVBChannel::DVBChannel(int aCardNum, TVRec *parent)
 
 DVBChannel::~DVBChannel()
 {
-    if (dvbsct)
-        delete dvbsct;
-    if (dvbcam)
-        delete dvbcam;
 
     CloseDVB();
 }
@@ -86,22 +81,31 @@ void DVBChannel::CloseDVB()
 {
     VERBOSE(VB_ALL, "Closing DVB channel");
 
-    if (fd_frontend > 0)
+    if (fd_frontend >= 0)
     {
         close(fd_frontend);
-        fd_frontend = 0;
+        fd_frontend = -1;
+        if (dvbcam)
+            delete dvbcam;
+        dvbcam = NULL;
+        if (dvbsct)
+            delete dvbsct;
+        dvbsct = NULL;
+        if (diseqc)
+            delete diseqc;
+        diseqc = NULL;
         isOpen = false;
     }
 }
 
 bool DVBChannel::Open()
 {
-    if (fd_frontend > 0)
+    if (fd_frontend >= 0)
         return true;
 
     fd_frontend = open(dvbdevice(DVB_DEV_FRONTEND, cardnum),
                        O_RDWR | O_NONBLOCK);
-    if(fd_frontend < 0)
+    if (fd_frontend < 0)
     {
         ERRNO("Opening DVB frontend device failed.");
         return false;
@@ -145,6 +149,7 @@ bool DVBChannel::Open()
     }
 
     force_channel_change = true;
+    first_tune = true;
 
     return isOpen = true;
 }
@@ -526,9 +531,9 @@ void DVBChannel::CheckOptions()
     unsigned int frequency;
     if (info.type == FE_QPSK)
         if (t.params.frequency > t.lnb_lof_switch)
-            frequency = t.params.frequency - t.lnb_lof_hi;
+            frequency = abs((int)t.params.frequency - (int)t.lnb_lof_hi);
         else
-            frequency = t.params.frequency - t.lnb_lof_lo;
+            frequency = abs((int)t.params.frequency - (int)t.lnb_lof_lo);
     else
         frequency = t.params.frequency;
 
@@ -632,7 +637,7 @@ bool DVBChannel::CheckModulation(fe_modulation_t& modulation)
 
 bool DVBChannel::FillFrontendStats(dvb_stats_t& stats)
 {
-    if (fd_frontend <= 0)
+    if (fd_frontend < 0)
         return false;
 
     ioctl(fd_frontend, FE_READ_SNR, &stats.snr);
@@ -756,7 +761,7 @@ bool DVBChannel::ParseQuery(QSqlQuery& query)
             }
             break;
         case FE_QAM:
-            if(!ParseQAM(query.value(4).toString(), query.value(5).toString(),
+            if (!ParseQAM(query.value(4).toString(), query.value(5).toString(),
                          query.value(6).toString(), query.value(7).toString(),
                          query.value(8).toString(), chan_opts.tuning))
             {
@@ -1156,12 +1161,14 @@ bool DVBChannel::TuneQPSK(dvb_tuning_t& tuning, bool reset, bool& havetuned)
     int frequency = tuning.params.frequency;
     if (tuning.params.frequency >= tuning.lnb_lof_switch)
     {
-        tuning.params.frequency = tuning.params.frequency - tuning.lnb_lof_hi;
+        tuning.params.frequency = abs((int)tuning.params.frequency - 
+                                      (int)tuning.lnb_lof_hi);
         tuning.tone = SEC_TONE_ON;
     }
     else
     {
-        tuning.params.frequency = tuning.params.frequency - tuning.lnb_lof_lo;
+        tuning.params.frequency = abs((int)tuning.params.frequency - 
+                                      (int)tuning.lnb_lof_lo);
         tuning.tone = SEC_TONE_OFF;
     }
 
