@@ -315,10 +315,9 @@ bool DVBDiSEqC::Diseqc1xSwitch(dvb_tuning_t& tuning, bool reset,
 
     GENERAL(QString("DiSEqC 1.0 Switch - Port %1").arg(tuning.diseqc_port));
 
-    if (tuning.diseqc_port != 0 &&
-        ((prev_tuning.diseqc_port != tuning.diseqc_port ||
-          prev_tuning.tone != tuning.tone ||
-          prev_tuning.voltage != tuning.voltage) || reset))
+    if ((prev_tuning.diseqc_port != tuning.diseqc_port ||
+        prev_tuning.tone != tuning.tone ||
+        prev_tuning.voltage != tuning.voltage) || reset)
     {
         if (tuning.diseqc_port > 3)
         {
@@ -332,9 +331,9 @@ bool DVBDiSEqC::Diseqc1xSwitch(dvb_tuning_t& tuning, bool reset,
         cmd.msg[DATA_1] = 0xF0 
                           | (((tuning.diseqc_port) * 4) & 0x0F) 
                           | ((tuning.voltage == SEC_VOLTAGE_18) ? 2 : 0) 
-                          | ((tuning.tone == SEC_TONE_ON) ? 0 : 1);
+                          | ((tuning.tone == SEC_TONE_ON) ? 1 : 0);
 
-        if(!SendDiSEqCMessage(tuning,cmd)) 
+        if (!SendDiSEqCMessage(tuning,cmd)) 
         {
             ERRNO("Setting DiSEqC failed.\n");
             return false;
