@@ -1,12 +1,15 @@
 #ifndef FILTERMANAGER
 #define FILTERMANAGER
 
+extern "C" {
 #include "filter.h"
 #include "frame.h"
+}
+
 #include <qdict.h>
 #include <qptrlist.h>
 #include <qstring.h>
-#include <stdlib.h>
+#include <cstdlib>
 #include <dlfcn.h>
 
 using namespace std;
