@@ -27,11 +27,7 @@ NuppelDecoder::NuppelDecoder(NuppelVideoPlayer *parent, MythSqlDatabase *db,
     usingextradata = false;
     memset(&extradata, 0, sizeof(extendeddata));
 
-    hasFullPositionMap = false;
-    positionMap = new QMap<long long, long long>;
-
-    hasKeyFrameAdjustMap = false;
-    keyFrameAdjustMap = new QMap<long long, int>;
+    positionMapType = MARK_KEYFRAME;
 
     totalLength = 0;
     totalFrames = 0;
@@ -83,10 +79,6 @@ NuppelDecoder::~NuppelDecoder()
         delete rtjd;
     if (ffmpeg_extradata)
         delete [] ffmpeg_extradata;
-    if (positionMap)
-        delete positionMap;
-    if (keyFrameAdjustMap)
-        delete keyFrameAdjustMap;
     if (buf)
         delete [] buf;
     if (buf2)
@@ -167,7 +159,8 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
     }
 
     m_parent->SetVideoParams(fileheader.width, fileheader.height,
-                             fileheader.fps, fileheader.keyframedist);
+                             fileheader.fps, fileheader.keyframedist,
+                             4.0 / 3, kScan_Detect);
 
     video_width = fileheader.width;
     video_height = fileheader.height;
@@ -270,13 +263,19 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                 struct seektable_entry ste;
                 int offset = 0;
 
+                m_positionMap.clear();
+                m_positionMap.reserve(numentries);
+
                 for (int z = 0; z < numentries; z++)
                 {
                     memcpy(&ste, seekbuf + offset,
                            sizeof(struct seektable_entry));
                     offset += sizeof(struct seektable_entry);
 
-                    (*positionMap)[ste.keyframe_number] = ste.file_offset;
+                    PosMapEntry e = {ste.keyframe_number,
+                                     ste.keyframe_number * keyframedist,
+                                     ste.file_offset};
+                    m_positionMap.push_back(e);
                 }
                 hasFullPositionMap = true;
                 totalLength = (int)((ste.keyframe_number * keyframedist * 1.0) /
@@ -324,6 +323,7 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                 struct kfatable_entry kfate;
                 int offset = 0;
                 int adjust = 0;
+                QMap<long long, int> keyFrameAdjustMap;
 
                 for (int z = 0; z < numentries; z++)
                 {
@@ -331,15 +331,24 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
                            sizeof(struct kfatable_entry));
                     offset += sizeof(struct kfatable_entry);
 
-                    (*keyFrameAdjustMap)[kfate.keyframe_number] = kfate.adjust;
+                    keyFrameAdjustMap[kfate.keyframe_number] = kfate.adjust;
                     adjust += kfate.adjust;
                 }
-                hasKeyFrameAdjustMap = true;
+                hasKeyFrameAdjustTable = true;
 
                 totalLength -= (int)(adjust / video_frame_rate);
                 totalFrames -= adjust;
                 m_parent->SetFileLength(totalLength, totalFrames);
 
+                adjust = 0;
+                for (unsigned int i=0; i < m_positionMap.size(); i++) 
+                {
+                    if (keyFrameAdjustMap.contains(m_positionMap[i].adjFrame))
+                        adjust += keyFrameAdjustMap[m_positionMap[i].adjFrame];
+
+                    m_positionMap[i].adjFrame -= adjust;
+                }
+
                 delete [] kfa_buf;
             }
             else
@@ -466,46 +475,12 @@ int NuppelDecoder::OpenFile(RingBuffer *rbuffer, bool novideo,
     if (hasFullPositionMap)
         return 1;
 
-    if (m_playbackinfo && m_db && positionMap)
-    {
-        m_db->lock();
-        m_playbackinfo->GetPositionMap(*positionMap, MARK_KEYFRAME, m_db->db());
-        m_db->unlock();
-
-        if (positionMap->size() && !livetv && !watchingrecording)
-        {
-            QMap<long long, long long>::Iterator it;
-            long long max_keyframe = -1;
-            for (it = positionMap->begin(); it != positionMap->end(); ++it)
-                if (it.key() > max_keyframe)
-                    max_keyframe = it.key();
-
-            if (max_keyframe >= 0)
-            {
-                totalLength = (int)((max_keyframe * keyframedist * 1.0) /
-                                     video_frame_rate);
-                totalFrames = (long long)max_keyframe * keyframedist;
-                m_parent->SetFileLength(totalLength, totalFrames);
-            }
-            hasFullPositionMap = true;
-            return 1;
-        }
-    }
+    if (SyncPositionMap())
+        return 1;
 
     return 0;
 }
 
-void NuppelDecoder::Reset(void)
-{
-    delete positionMap;
-    positionMap = new QMap<long long, long long>;
-
-    if (mpa_codec)
-        avcodec_flush_buffers(mpa_ctx);
-
-    framesPlayed = 0;
-}
-
 int get_nuppel_buffer(struct AVCodecContext *c, AVFrame *pic)
 {
     NuppelDecoder *nd = (NuppelDecoder *)(c->opaque);
@@ -846,6 +821,7 @@ void NuppelDecoder::GetFrame(int avignore)
         if ((ringBuffer->Read(&frameheader, FRAMEHEADERSIZE) != FRAMEHEADERSIZE)
             || (frameheader.frametype == 'Q') || (frameheader.frametype == 'K'))
         {
+            ateof = true;
             m_parent->SetEof();
             return;
         }
@@ -864,6 +840,7 @@ void NuppelDecoder::GetFrame(int avignore)
             if (ringBuffer->Read(&frameheader, FRAMEHEADERSIZE)
                 != FRAMEHEADERSIZE)
             {
+                ateof = true;
                 m_parent->SetEof();
                 return;
             }
@@ -878,6 +855,7 @@ void NuppelDecoder::GetFrame(int avignore)
             if (ringBuffer->Read(dummy, sizetoskip) != sizetoskip)
             {
                 delete [] dummy;
+                ateof = true;
                 m_parent->SetEof();
                 return;
             }
@@ -897,7 +875,8 @@ void NuppelDecoder::GetFrame(int avignore)
         {
             if (frameheader.comptype == 'A')
             {
-                if (frameheader.timecode > 0 && frameheader.timecode < 5500000)
+                if (frameheader.timecode > 2000000 && 
+                    frameheader.timecode < 5500000)
                 {
                     effdsp = frameheader.timecode;
                     m_parent->SetEffDsp(effdsp);
@@ -907,8 +886,25 @@ void NuppelDecoder::GetFrame(int avignore)
             {
                 lastKey = frameheader.timecode;
                 framesPlayed = frameheader.timecode - 1;
+
                 if (!hasFullPositionMap)
-                    (*positionMap)[lastKey / keyframedist] = currentposition;
+                {
+                    long long last_index = 0;
+                    long long this_index = lastKey / keyframedist;
+                    if (!m_positionMap.empty())
+                        last_index =
+                            m_positionMap[m_positionMap.size() - 1].index;
+
+                    if (this_index > last_index)
+                    {
+                        // Grow positionMap vector several entries at a time
+                        if (m_positionMap.capacity() == m_positionMap.size())
+                            m_positionMap.reserve(m_positionMap.size() + 60);
+                        PosMapEntry entry = {this_index, lastKey,
+                                             currentposition};
+                        m_positionMap.push_back(entry);
+                    }
+                }
             }
             if (getrawframes)
                 StoreRawData(NULL);
@@ -920,12 +916,14 @@ void NuppelDecoder::GetFrame(int avignore)
             {
                 cerr << "Broken packet: " << frameheader.frametype
                      << " " << frameheader.packetlength << endl;
+                ateof = true;
                 m_parent->SetEof();
                 return;
             }
             if (ringBuffer->Read(strm, frameheader.packetlength) !=
                 frameheader.packetlength)
             {
+                ateof = true;
                 m_parent->SetEof();
                 return;
             }
@@ -1000,9 +998,8 @@ void NuppelDecoder::GetFrame(int avignore)
                     }
                     else if (lameret < 0)
                     {
-                        cerr << "lame decode error: " << lameret 
-                             << ", exiting\n";
-                        exit(-1);
+                        VERBOSE(VB_IMPORTANT, QString("lame decode error: %1, exiting").arg(lameret));
+                        exit(-15);
                     }
                     packetlen = 0;
                 } while (lameret > 0);
@@ -1025,310 +1022,21 @@ void NuppelDecoder::GetFrame(int avignore)
         }
     }
 
-    m_parent->SetFramesPlayed(framesPlayed);
+    UpdateFramesPlayed();
 }
 
-int NuppelDecoder::GetKeyIndex(int keyFrame)
+void NuppelDecoder::SeekReset(long long newKey, int skipFrames,
+                              bool needFlush)
 {
-    if (hasKeyFrameAdjustMap)
-    {
-        int accum = 0;
-        QMap <long long, int>::Iterator ki;
-        for (ki = (*keyFrameAdjustMap).begin();
-             ki != (*keyFrameAdjustMap).end(); ki++)
-        {
-            if (keyFrame < (ki.key() * keyframedist - ki.data() - accum))
-                break;
-            accum += ki.data();
-        }
-        return ((keyFrame + accum) / keyframedist);
-    }
-    else
-       return (keyFrame / keyframedist);
-}
-
-bool NuppelDecoder::DoRewind(long long desiredFrame)
-{
-    long long storelastKey = lastKey;
-    int keyIndex;
-
-    if (hasKeyFrameAdjustMap)
-    {
-        keyIndex = GetKeyIndex(lastKey);
-        QMap <long long, int>::Iterator keyiter = NULL;
-        keyiter = (*keyFrameAdjustMap).find(lastKey);
-        if (keyiter != (*keyFrameAdjustMap).end())
-            lastKey += keyiter.data();
-
-        while (lastKey > desiredFrame)
-        {
-            lastKey -= keyframedist;
-            keyIndex--;
-            keyiter = (*keyFrameAdjustMap).find(lastKey);
-            if (keyiter != (*keyFrameAdjustMap).end())
-                lastKey += keyiter.data();
-        }
-    }
-    else
-    {
-        while (lastKey > desiredFrame)
-        {
-            lastKey -= keyframedist;
-        }
-        keyIndex = lastKey / keyframedist;
-    }
-
-    if (lastKey < 1)
-        lastKey = 1;
-
-    int normalframes = desiredFrame - lastKey;
-    long long keyPos = (*positionMap)[keyIndex];
-    long long curPosition = ringBuffer->GetTotalReadPosition();
-    long long diff = keyPos - curPosition;
-
-    while (ringBuffer->GetFreeSpaceWithReadChange(diff) < 0)
-    {
-        lastKey += keyframedist;
-        keyIndex++;
-        keyPos = (*positionMap)[keyIndex];
-        if (keyPos == 0)
-            continue;
-        diff = keyPos - curPosition;
-        normalframes = 0;
-        if (lastKey > storelastKey)
-        {
-            lastKey = storelastKey;
-            diff = 0;
-            normalframes = 0;
-            return false;
-        }
-    }
-
-    int iter = 0;
-
-    while (keyPos == 0 && lastKey > 1 && iter < 4)
-    {
-        VERBOSE(VB_PLAYBACK, QString("No keyframe in position map for %1")
-                                     .arg((int)lastKey));
-
-        lastKey -= keyframedist;
-        keyIndex--;
-        keyPos = (*positionMap)[keyIndex];
-        diff = keyPos - curPosition;
-        normalframes += keyframedist;
-
-        if (keyPos != 0)
-           VERBOSE(VB_PLAYBACK, QString("Using seek position: %1")
-                                       .arg((int)lastKey));
-
-        iter++;
-    }
-
-    if (keyPos == 0)
-    {
-        VERBOSE(VB_GENERAL, QString("Unknown seek position: %1")
-                                    .arg((int)lastKey));
-
-        lastKey = storelastKey;
-        diff = 0;
-        normalframes = 0;
-
-        return false;
-    }
-
-    ringBuffer->Seek(diff, SEEK_CUR);
-    framesPlayed = lastKey - 1;
-
-    int fileend = 0;
-
-    normalframes = desiredFrame - framesPlayed;
-
-    if (!exactseeks)
-        normalframes = 0;
-
-    if (mpa_codec)
+    if (mpa_codec && needFlush)
         avcodec_flush_buffers(mpa_ctx);
 
-    // disbale frame storage
-    int oldrawstate = getrawframes;
-    getrawframes = 0;
-
-    while (normalframes > 0 && !fileend)
+    while (skipFrames > 0)
     {
-        if (normalframes != 1)
-            GetFrame(1);
-        else
-        {
-            getrawframes = oldrawstate;
-            GetFrame(0);
-        }
-
-        fileend = m_parent->GetEof();
-        normalframes--;
-    }
-
-    getrawframes = oldrawstate;
-
-    return true;
-}
-
-bool NuppelDecoder::DoFastForward(long long desiredFrame)
-{
-    // disable frame storage
-    int oldrawstate = getrawframes;
-    getrawframes = 0;
-
-    long long number = desiredFrame - framesPlayed;
-    long long desiredKey = lastKey;
-    int lastKeyIndex = GetKeyIndex(lastKey);
-    int desiredIndex = lastKeyIndex;
-
-    if (hasKeyFrameAdjustMap)
-    {
-        QMap <long long, int>::Iterator keyiter = NULL;
-        keyiter = (*keyFrameAdjustMap).begin();
-        while (keyiter.key() < lastKey && keyiter != (*keyFrameAdjustMap).end())
-            keyiter++;
-        long long prevKey = desiredKey;
-        while (desiredKey <= desiredFrame)
-        {
-            prevKey = desiredKey;
-            desiredKey += keyframedist;
-            desiredIndex++;
-            if (desiredKey == keyiter.key())
-            {
-                desiredKey -= keyiter.data();
-                keyiter++;
-            }
-        }
-        desiredKey = prevKey;
-        desiredIndex--;
-    }
-    else
-    {
-        while (desiredKey <= desiredFrame)
-            desiredKey += keyframedist;
-        desiredKey -= keyframedist;
-        desiredIndex = desiredKey / keyframedist;
-    }
-
-    int normalframes = desiredFrame - desiredKey;
-    int fileend = 0;
-
-    if (desiredKey == lastKey)
-        normalframes = number;
-
-    long long keyPos = -1;
-
-    long long tmpKey = desiredKey;
-    int tmpIndex = desiredIndex;
-
-    while (keyPos == -1 && tmpKey > lastKey)
-    {
-        if (positionMap->find(tmpIndex) != positionMap->end())
-        {
-            keyPos = (*positionMap)[tmpIndex];
-        }
-        else if (livetv || (watchingrecording && nvr_enc &&
-                            nvr_enc->IsValidRecorder()))
-        {
-            for (int i = lastKeyIndex; i <= tmpIndex; i++)
-            {
-                if (positionMap->find(i) == positionMap->end())
-                    nvr_enc->FillPositionMap(i, tmpIndex, *positionMap);
-            }
-            keyPos = (*positionMap)[tmpIndex];
-        }
-
-        if (keyPos == -1)
-        {
-            VERBOSE(VB_PLAYBACK, QString("No keyframe in position map for %1")
-                    .arg((int)tmpKey));
-
-            tmpKey -= keyframedist;
-            tmpIndex--;
-        }
-        else
-        {
-            desiredKey = tmpKey;
-            desiredIndex = tmpIndex;
-        }
-    }
-
-    bool needflush = false;
-
-    if (keyPos != -1)
-    {
-        lastKey = desiredKey;
-        long long diff = keyPos - ringBuffer->GetTotalReadPosition();
-
-        ringBuffer->Seek(diff, SEEK_CUR);
-        framesPlayed = lastKey - 1;
-        needflush = true;
-    }
-    else if (!livetv && !watchingrecording)
-    {
-        if (lastKey < desiredKey)
-        {
-            VERBOSE(VB_IMPORTANT, "Did not find keyframe position.");
-
-            VERBOSE(VB_PLAYBACK, QString("lastKey: %1 desiredKey: %2")
-                    .arg((int)lastKey).arg((int)desiredKey));
-        }
-
-        while (lastKey < desiredKey && !fileend)
-        {
-            GetFrame(-1);
-            needflush = true;
-            fileend = m_parent->GetEof();
-        }
-
-        if (needflush)
-        {
-            lastKey = desiredKey;
-            keyPos = (*positionMap)[desiredKey / keyframedist];
-            long long diff = keyPos - ringBuffer->GetTotalReadPosition();
-
-            ringBuffer->Seek(diff, SEEK_CUR);
-
-            framesPlayed = lastKey - 1;
-        }
-    }
-
-    normalframes = desiredFrame - framesPlayed;
-
-    if (!exactseeks)
-        normalframes = 0;
-
-    if (mpa_codec && needflush)
-        avcodec_flush_buffers(mpa_ctx);
-
-    while (normalframes > 0 && !fileend)
-    {
-        if (normalframes != 1)
-            GetFrame(1);
-        else
-        {
-            getrawframes = oldrawstate;
-            GetFrame(false);
-        }
-        fileend = m_parent->GetEof();
-        normalframes--;
-    }
-
-    getrawframes = oldrawstate;
-
-    return true;
-}
-
-void NuppelDecoder::SetPositionMap(void)
-{
-    if (m_playbackinfo && m_db && positionMap)
-    {
-        m_db->lock();
-        m_playbackinfo->SetPositionMap(*positionMap, MARK_KEYFRAME, 
-                                       m_db->db());
-        m_db->unlock();
+        GetFrame(0);
+        if (ateof)
+            break;
+        skipFrames--;
     }
 }
 
