@@ -1,6 +1,7 @@
 #include <qstring.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <unistd.h>
 #include <string.h>
 #include <qimage.h>
 #include <qpixmap.h>
@@ -8,6 +9,7 @@
 #include <qdir.h>
 #include <qfile.h>
 #include <qcolor.h>
+#include <qregexp.h>
 
 #include <iostream>
 #include <algorithm>
@@ -329,6 +331,7 @@ void OSD::parseFont(QDomElement &element)
     bool outline = false;
     QPoint shadowOffset = QPoint(0, 0);
     int color = 255;
+    QColor color_normal, color_outline, color_shadow;
 
     name = element.attribute("name", "");
     if (name.isNull() || name.isEmpty())
@@ -358,6 +361,15 @@ void OSD::parseFont(QDomElement &element)
             else if (info.tagName() == "color")
             {
                 color = getFirstText(info).toInt();
+
+                if (info.hasAttribute("normal"))
+                    color_normal = parseColor(info.attribute("normal"));
+
+                if (info.hasAttribute("outline"))
+                    color_outline = parseColor(info.attribute("outline"));
+
+                if (info.hasAttribute("shadow"))
+                    color_shadow = parseColor(info.attribute("shadow"));
             }
             else if (info.tagName() == "outline")
             {
@@ -391,12 +403,12 @@ void OSD::parseFont(QDomElement &element)
 
     if (fontSizeType == "small")
     {
-        if(sizeSmall > 0)
+        if (sizeSmall > 0)
             size = sizeSmall;
     }
     else if (fontSizeType == "big")
     {
-        if(sizeBig > 0)
+        if (sizeBig > 0)
             size = sizeBig;
     }
     
@@ -413,6 +425,15 @@ void OSD::parseFont(QDomElement &element)
     font->setOutline(outline);
     font->setShadow(shadowOffset.x(), shadowOffset.y());
 
+    if (color_normal.isValid())
+        font->setColor(color_normal, kTTF_Normal);
+
+    if (color_outline.isValid())
+        font->setColor(color_outline, kTTF_Outline);
+
+    if (color_shadow.isValid())
+        font->setColor(color_shadow, kTTF_Shadow);
+
     fontMap[name] = font;
 }
 
@@ -1172,6 +1193,25 @@ QPoint OSD::parsePoint(QString text)
     return retval;
 }
 
+QColor OSD::parseColor(QString text)
+{
+    QColor retval;
+    QRegExp regexp("#([0-9a-fA-F]){6}");
+    if (regexp.exactMatch(text))
+    {
+        int val;
+        if (sscanf(text.data(), "#%x", &val) == 1)
+            retval = QColor((val >> 16) & 0xff, (val >> 8) & 0xff, val & 0xff);
+    }
+    else
+    {
+        int r, g, b;
+        if (sscanf(text.data(), "%d,%d,%d", &r, &g, &b) == 3)
+            retval = QColor(r, g, b);
+    }
+    return retval;
+}
+
 QRect OSD::parseRect(QString text)
 {
     int x, y, w, h;
@@ -1558,6 +1598,10 @@ void OSD::NewDialogBox(const QString &name, const QString &message,
     changed = true;
 
     osdlock.unlock();
+
+    int count = 0;
+    while (!container->HasDisplayed() && count++ < 10)
+        usleep(1000);
 }
 
 void OSD::HighlightDialogSelection(OSDSet *container, int num)
@@ -1650,7 +1694,11 @@ bool OSD::DialogShowing(const QString &name)
     if (name == "")
         return false;
 
-    return (GetSet(name) != NULL);
+    osdlock.lock();
+    bool ret = (GetSet(name) != NULL);
+    osdlock.unlock();
+
+    return ret;
 }
 
 void OSD::DialogAbort(const QString &name)
@@ -1733,6 +1781,8 @@ void OSD::HideAll(void)
         if (*i)
             (*i)->Hide();
 
+    runningTreeMenu = NULL;
+
     changed = true;
     osdlock.unlock();
 }
