@@ -62,7 +62,10 @@ OSDListTreeType::OSDListTreeType(const QString &name, const QRect &area,
     m_levelsize = levelsize;
     m_levelspacing = levelspacing;
 
-    m_arrowAccel = true;
+    if (gContext->GetNumSetting("UseArrowAccels", 1))
+        m_arrowAccel = true;
+    else
+        m_arrowAccel = false;
     
     levels = 0;
     curlevel = -1;
@@ -264,7 +267,7 @@ bool OSDListTreeType::HandleKeypress(QKeyEvent *e)
                     emit itemSelected(this, currentpos);
                 }
             }
-            else if (action == "ESCAPE")
+            else if (action == "ESCAPE" || action == "MENU")
                 m_visible = false;
             else if (action == "SELECT")
                 emit itemSelected(this, currentpos);
@@ -426,6 +429,8 @@ void OSDListBtnType::SetActive(bool active)
 
 void OSDListBtnType::Reset()
 {
+    QMutexLocker lock(&m_update);
+
     m_clearing = true;
 
     OSDListBtnTypeItem* item = 0;
@@ -487,6 +492,8 @@ void OSDListBtnType::RemoveItem(OSDListBtnTypeItem *item)
 
 void OSDListBtnType::SetItemCurrent(OSDListBtnTypeItem* item)
 {
+    bool locked = m_update.tryLock();
+
     if (m_itemList.find(item) == -1)
         return;
 
@@ -499,10 +506,15 @@ void OSDListBtnType::SetItemCurrent(OSDListBtnTypeItem* item)
         m_showDnArrow = false;
 
     emit itemSelected(m_selItem);
+
+    if (locked)
+        m_update.unlock();
 }
 
 void OSDListBtnType::SetItemCurrent(int current)
 {
+    QMutexLocker lock(&m_update);
+
     OSDListBtnTypeItem* item = m_itemList.at(current);
     if (!item)
         item = m_itemList.first();
@@ -522,6 +534,8 @@ OSDListBtnTypeItem* OSDListBtnType::GetItemFirst()
 
 OSDListBtnTypeItem* OSDListBtnType::GetItemNext(OSDListBtnTypeItem *item)
 {
+    QMutexLocker lock(&m_update);
+
     if (m_itemList.find(item) == -1)
         return 0;
 
@@ -540,17 +554,30 @@ OSDListBtnTypeItem* OSDListBtnType::GetItemAt(int pos)
 
 int OSDListBtnType::GetItemPos(OSDListBtnTypeItem* item)
 {
+    QMutexLocker lock(&m_update);
+
     return m_itemList.find(item);    
 }
 
 void OSDListBtnType::MoveUp()
 {
+    QMutexLocker lock(&m_update);
+
     if (m_itemList.find(m_selItem) == -1)
         return;
 
     OSDListBtnTypeItem *item = m_itemList.prev();
-    if (!item) 
-        return;
+    if (!item)
+    {
+        item = m_itemList.last();
+        if (!item)
+            return;
+
+        if (m_itemList.count() > m_itemsVisible)
+            m_topItem = m_itemList.at(m_itemList.count() - m_itemsVisible);
+        else
+            m_topItem = m_itemList.first();
+    }
 
     m_selItem = item;
 
@@ -572,11 +599,20 @@ void OSDListBtnType::MoveUp()
 
 void OSDListBtnType::MoveDown()
 {
+    QMutexLocker lock(&m_update);
+
     if (m_itemList.find(m_selItem) == -1)
         return;
+
     OSDListBtnTypeItem *item = m_itemList.next();
-    if (!item) 
-        return;
+    if (!item)
+    {
+        item = m_itemList.first();
+        if (!item)
+            return;
+
+        m_topItem = item;
+    }
 
     m_selItem = item;
 
@@ -608,6 +644,8 @@ void OSDListBtnType::Draw(OSDSurface *surface, int fade, int maxfade, int xoff,
     if (!m_visible)
         return;
 
+    QMutexLocker lock(&m_update);
+
     if (!m_initialized)
         Init();
 
@@ -821,8 +859,8 @@ OSDListBtnTypeItem::OSDListBtnTypeItem(OSDListBtnType* lbtype,
                                        bool showArrow, CheckState state)
 {
     if (!lbtype) {
-        cerr << "OSDListBtnTypeItem: trying to creating item without parent\n";
-        exit(-1);
+        VERBOSE(VB_IMPORTANT, "OSDListBtnTypeItem: trying to creating item without parent");
+        exit(-16);
     }
     
     m_parent    = lbtype;
