@@ -18,7 +18,6 @@ using namespace std;
 
 #include "proglist.h"
 #include "scheduledrecording.h"
-#include "infodialog.h"
 #include "dialogbox.h"
 #include "mythcontext.h"
 #include "remoteutil.h"
@@ -31,11 +30,23 @@ ProgLister::ProgLister(ProgListType pltype, const QString &view,
     type = pltype;
     db = ldb;
     startTime = QDateTime::currentDateTime();
-    timeFormat = gContext->GetSetting("ShortDateFormat") +
-        " " + gContext->GetSetting("TimeFormat");
+    searchTime = startTime;
+
+    dayFormat = gContext->GetSetting("DateFormat");
+    hourFormat = gContext->GetSetting("TimeFormat");
+    timeFormat = gContext->GetSetting("ShortDateFormat") + " " + hourFormat;
+    fullDateFormat = dayFormat + " " + hourFormat;
     channelOrdering = gContext->GetSetting("ChannelOrdering", "channum + 0");
     channelFormat = gContext->GetSetting("ChannelFormat", "<num> <sign>");
 
+    switch (pltype)
+    {
+        case plTitleSearch:   searchtype = kTitleSearch;   break;
+        case plKeywordSearch: searchtype = kKeywordSearch; break;
+        case plPeopleSearch:  searchtype = kPeopleSearch;  break;
+        default:              searchtype = kNoSearch;      break;
+    }
+
     allowEvents = true;
     allowUpdates = true;
     updateAll = false;
@@ -75,11 +86,13 @@ ProgLister::ProgLister(ProgListType pltype, const QString &view,
     chooseLineEdit = NULL;
     chooseOkButton = NULL;
     chooseDeleteButton = NULL;
+    chooseRecordButton = NULL;
+    chooseDay = NULL;
+    chooseHour = NULL;
 
     curView = -1;
     fillViewList(view);
 
-    itemList.setAutoDelete(true);
     curItem = -1;
     fillItemList();
 
@@ -135,6 +148,10 @@ void ProgLister::keyPressEvent(QKeyEvent *e)
             accept();
         else if (action == "INFO")
             edit();
+        else if (action == "UPCOMING")
+            upcoming();
+        else if (action == "DETAILS")
+            details();
         else if (action == "TOGGLERECORD")
             quickRecord();
         else
@@ -185,8 +202,8 @@ void ProgLister::LoadWindow(QDomElement &element)
             }
             else
             {
-                cerr << "Unknown element: " << e.tagName() << endl;
-                exit(0);
+                VERBOSE(VB_IMPORTANT, QString("Unknown element: %1, exiting").arg(e.tagName()));
+                exit(-17);
             }
         }
     }
@@ -211,10 +228,12 @@ void ProgLister::updateBackground(void)
                 case plTitle: value = tr("Program Listings"); break;
                 case plNewListings: value = tr("New Title Search"); break;
                 case plTitleSearch: value = tr("Title Search"); break;
-                case plDescSearch: value = tr("Description Search"); break;
+                case plKeywordSearch: value = tr("Keyword Search"); break;
+                case plPeopleSearch: value = tr("People Search"); break;
                 case plChannel: value = tr("Channel Search"); break;
                 case plCategory: value = tr("Category Search"); break;
                 case plMovies: value = tr("Movie Search"); break;
+                case plTime: value = tr("Time Search"); break;
                 default: value = tr("Unknown Search"); break;
             }
             ltype->SetText(value);
@@ -250,11 +269,11 @@ void ProgLister::paintEvent(QPaintEvent *e)
 
 void ProgLister::cursorDown(bool page)
 {
-    if (curItem < itemCount - 1)
+    if (curItem < (int)itemList.count() - 1)
     {
         curItem += (page ? listsize : 1);
-        if (curItem > itemCount - 1)
-            curItem = itemCount - 1;
+        if (curItem > (int)itemList.count() - 1)
+            curItem = itemList.count() - 1;
         update(fullRect);
     }
 }
@@ -272,12 +291,12 @@ void ProgLister::cursorUp(bool page)
 
 void ProgLister::prevView(void)
 {
-    if (viewCount < 2)
+    if (viewList.count() < 2)
         return;
 
     curView--;
     if (curView < 0)
-        curView = viewCount - 1;
+        curView = viewList.count() - 1;
 
     curItem = -1;
     refillAll = true;
@@ -285,11 +304,11 @@ void ProgLister::prevView(void)
 
 void ProgLister::nextView(void)
 {
-    if (viewCount < 2)
+    if (viewList.count() < 2)
         return;
 
     curView++;
-    if (curView >= viewCount)
+    if (curView >= (int)viewList.count())
         curView = 0;
 
     curItem = -1;
@@ -303,7 +322,8 @@ void ProgLister::setViewFromList(void)
 
     int view = chooseListBox->currentItem();
 
-    if (type == plTitleSearch || type == plDescSearch)
+    if (type == plTitleSearch || type == plKeywordSearch || 
+        type == plPeopleSearch)
     {
         view--;
         if (view < 0)
@@ -327,11 +347,13 @@ void ProgLister::setViewFromList(void)
 
 void ProgLister::chooseEditChanged(void)
 {
-    if (!chooseOkButton || !chooseLineEdit)
+    if (!chooseOkButton || !chooseRecordButton || !chooseLineEdit)
         return;
 
     chooseOkButton->setEnabled(chooseLineEdit->text().
                                stripWhiteSpace().length() > 0);
+    chooseRecordButton->setEnabled(chooseLineEdit->text().
+                                   stripWhiteSpace().length() > 0);
 }
 
 void ProgLister::chooseListBoxChanged(void)
@@ -362,20 +384,30 @@ void ProgLister::setViewFromEdit(void)
     int oldview = chooseListBox->currentItem() - 1;
     int newview = viewList.findIndex(text);
 
+    QString querystr = NULL;
+    QString qphrase = NULL;
+
     if (newview < 0 || newview != oldview)
     {
         if (oldview >= 0)
         {
-            QString querystr = QString("DELETE FROM keyword "
-                                       "WHERE phrase = '%1';")
-                                       .arg(viewList[oldview].utf8());
+            qphrase = viewList[oldview].utf8();
+            qphrase.replace("\'", "\\\'");
+
+            querystr = QString("DELETE FROM keyword "
+                               "WHERE phrase = '%1' AND searchtype = '%2';")
+                               .arg(qphrase).arg(searchtype);
             QSqlQuery query;
             query.exec(querystr);
         }
         if (newview < 0)
         {
-            QString querystr = QString("REPLACE INTO keyword "
-                                       "VALUES('%1');").arg(text.utf8());
+            qphrase = text.utf8();
+            qphrase.replace("\'", "\\\'");
+
+            querystr = QString("REPLACE INTO keyword (phrase, searchtype)"
+                               "VALUES('%1','%2');")
+                               .arg(qphrase).arg(searchtype);
             QSqlQuery query;
             query.exec(querystr);
         }
@@ -389,6 +421,29 @@ void ProgLister::setViewFromEdit(void)
     refillAll = true;
 }
 
+void ProgLister::addSearchRecord(void)
+{
+    if (!choosePopup || !chooseListBox || !chooseLineEdit)
+        return;
+
+    QString text = chooseLineEdit->text();
+
+    if (text.stripWhiteSpace().length() == 0)
+        return;
+
+    if (searchtype == kNoSearch)
+    {
+        VERBOSE(VB_IMPORTANT, "Unknown search in ProgLister");
+        return;
+    }
+
+    ScheduledRecording record;
+    record.loadBySearch(db, searchtype, text);
+    record.exec(db);
+
+    setViewFromEdit();
+}
+
 void ProgLister::deleteKeyword(void)
 {
     if (!chooseDeleteButton || !chooseListBox)
@@ -400,16 +455,18 @@ void ProgLister::deleteKeyword(void)
         return;
 
     QString text = viewList[view];
+    QString qphrase = text.utf8();
+    qphrase.replace("\'", "\\\'");
 
-    QString querystr = QString("DELETE FROM keyword WHERE phrase = '%1';")
-                               .arg(text.utf8());
+    QString querystr = QString("DELETE FROM keyword "
+                               "WHERE phrase = '%1' AND searchtype = '%2';")
+                               .arg(qphrase).arg(searchtype);
     QSqlQuery query;
     query.exec(querystr);
 
     chooseListBox->removeItem(view + 1);
     viewList.remove(text);
     viewTextList.remove(text);
-    viewCount--;
 
     if (view < curView)
         curView--;
@@ -420,17 +477,39 @@ void ProgLister::deleteKeyword(void)
         view = chooseListBox->count() - 2;
 
     chooseListBox->setSelected(view + 1, true);
-    if (viewCount < 1)
+    if (viewList.count() < 1)
         chooseLineEdit->setFocus();
     else
         chooseListBox->setFocus();
 }
 
+void ProgLister::setViewFromTime(void)
+{
+    if (!choosePopup || !chooseDay || !chooseHour)
+        return;
+
+    int dayOffset = chooseDay->currentItem() - 1;
+    searchTime.setDate(startTime.addDays(dayOffset).date());
+
+    QTime m_hr;
+    m_hr.setHMS(chooseHour->currentItem(), 0, 0);
+    searchTime.setTime(m_hr);
+
+    curView = 0;
+    viewList[curView] = searchTime.toString(fullDateFormat);
+    viewTextList[curView] = viewList[curView];
+
+    choosePopup->done(0);
+
+    curItem = -1;
+    refillAll = true;
+}
+
 void ProgLister::chooseView(void)
 {
     if (type == plChannel || type == plCategory)
     {
-        if (viewCount < 2)
+        if (viewList.count() < 2)
             return;
 
         choosePopup = new MythPopupBox(gContext->GetMainWindow(), "");
@@ -459,7 +538,8 @@ void ProgLister::chooseView(void)
         delete choosePopup;
         choosePopup = NULL;
     }
-    else if (type == plTitleSearch || type == plDescSearch)
+    else if (type == plTitleSearch || type == plKeywordSearch ||
+             type == plPeopleSearch)
     {
         int oldView = curView;
 
@@ -492,9 +572,15 @@ void ProgLister::chooseView(void)
         chooseDeleteButton->setText(tr("Delete"));
         choosePopup->addWidget(chooseDeleteButton);
 
+        chooseRecordButton = new MythPushButton(choosePopup);
+        chooseRecordButton->setText(tr("Record"));
+        choosePopup->addWidget(chooseRecordButton);
+
         chooseOkButton->setEnabled(chooseLineEdit->text()
                                    .stripWhiteSpace().length() > 0);
         chooseDeleteButton->setEnabled(curView >= 0);
+        chooseRecordButton->setEnabled(chooseLineEdit->text()
+                                       .stripWhiteSpace().length() > 0);
 
         connect(chooseListBox, SIGNAL(accepted(int)), this, SLOT(setViewFromList()));
         connect(chooseListBox, SIGNAL(menuButtonPressed(int)), chooseLineEdit, SLOT(setFocus()));
@@ -502,8 +588,9 @@ void ProgLister::chooseView(void)
         connect(chooseLineEdit, SIGNAL(textChanged()), this, SLOT(chooseEditChanged()));
         connect(chooseOkButton, SIGNAL(clicked()), this, SLOT(setViewFromEdit()));
         connect(chooseDeleteButton, SIGNAL(clicked()), this, SLOT(deleteKeyword()));
+        connect(chooseRecordButton, SIGNAL(clicked()), this, SLOT(addSearchRecord()));
 
-        if (viewCount < 1)
+        if (viewList.count() < 1)
             chooseLineEdit->setFocus();
         else
             chooseListBox->setFocus();
@@ -515,12 +602,14 @@ void ProgLister::chooseView(void)
         chooseOkButton = NULL;
         delete chooseDeleteButton;
         chooseDeleteButton = NULL;
+        delete chooseRecordButton;
+        chooseRecordButton = NULL;
         delete chooseListBox;
         chooseListBox = NULL;
         delete choosePopup;
         choosePopup = NULL;
 
-        if (viewCount < 1 || (oldView < 0 && curView < 0))
+        if (viewList.count() < 1 || (oldView < 0 && curView < 0))
             reject();
         else if (curView < 0)
         {
@@ -529,6 +618,54 @@ void ProgLister::chooseView(void)
             refillAll = true;
         }
     }
+    else if (type == plTime)
+    {
+        choosePopup = new MythPopupBox(gContext->GetMainWindow(), "");
+        choosePopup->addLabel(tr("Select Time"));
+
+        chooseDay = new MythComboBox(false, choosePopup);
+
+        for(int m_index = -1; m_index <= 14; m_index++)
+        {
+            chooseDay->insertItem(startTime.addDays(m_index)
+                                  .toString(dayFormat));
+            if (startTime.addDays(m_index).toString("MMdd") ==
+                                searchTime.toString("MMdd"))
+                chooseDay->setCurrentItem(chooseDay->count() - 1);
+        }
+        choosePopup->addWidget(chooseDay);
+
+        chooseHour = new MythComboBox(false, choosePopup);
+
+        QTime m_hr;
+        for(int m_index = 0; m_index < 24; m_index++)
+        {
+            m_hr.setHMS(m_index, 0, 0);
+            chooseHour->insertItem(m_hr.toString(hourFormat));
+            if (m_hr.toString("hh") == searchTime.toString("hh"))
+                chooseHour->setCurrentItem(m_index);
+        }
+        choosePopup->addWidget(chooseHour);
+
+        chooseOkButton = new MythPushButton(choosePopup);
+        chooseOkButton->setText(tr("OK"));
+        choosePopup->addWidget(chooseOkButton);
+
+        connect(chooseOkButton,
+                SIGNAL(clicked()), this, SLOT(setViewFromTime()));
+
+        chooseOkButton->setFocus();
+        choosePopup->ExecPopup();
+
+        delete chooseDay;
+        chooseDay = NULL;
+        delete chooseHour;
+        chooseHour = NULL;
+        delete chooseOkButton;
+        chooseOkButton = NULL;
+        delete choosePopup;
+        choosePopup = NULL;
+    }
 }
 
 void ProgLister::quickRecord()
@@ -561,11 +698,32 @@ void ProgLister::edit()
     pi->EditScheduled(db);
 }
 
+void ProgLister::upcoming()
+{
+    ProgramInfo *pi = itemList.at(curItem);
+
+    if (!pi || type == plTitle)
+        return;
+
+    ProgLister *pl = new ProgLister(plTitle, pi->title,
+                                   QSqlDatabase::database(),
+                                   gContext->GetMainWindow(), "proglist");
+    pl->exec();
+    delete pl;
+}
+
+void ProgLister::details()
+{
+    ProgramInfo *pi = itemList.at(curItem);
+
+    if (pi)
+        pi->showDetails(QSqlDatabase::database());
+}
+
 void ProgLister::fillViewList(const QString &view)
 {
     viewList.clear();
     viewTextList.clear();
-    viewCount = 0;
 
     if (type == plChannel) // list by channel
     {
@@ -582,8 +740,8 @@ void ProgLister::fillViewList(const QString &view)
             {
                 QString chanid = query.value(0).toString();
                 QString channum = query.value(1).toString();
-                QString chansign = query.value(2).toString();
-                QString channame = query.value(3).toString();
+                QString chansign = QString::fromUtf8(query.value(2).toString());
+                QString channame = QString::fromUtf8(query.value(3).toString());
 
                 QString chantext = channelFormat;
                 chantext.replace("<num>", channum)
@@ -592,7 +750,6 @@ void ProgLister::fillViewList(const QString &view)
 
                 viewList << chanid;
                 viewTextList << chantext;
-                viewCount++;
             }
         }
         if (view != "")
@@ -613,15 +770,17 @@ void ProgLister::fillViewList(const QString &view)
                 category = QString::fromUtf8(query.value(0).toString());
                 viewList << category;
                 viewTextList << category;
-                viewCount++;
             }
         }
         if (view != "")
             curView = viewList.findIndex(view);
     }
-    else if (type == plTitleSearch || type == plDescSearch)
+    else if (type == plTitleSearch || type == plKeywordSearch ||
+             type == plPeopleSearch)
     {
-        QString querystr = "SELECT phrase FROM keyword;";
+        QString querystr = QString("SELECT phrase FROM keyword "
+                                   "WHERE searchtype = '%1';")
+                                   .arg(searchtype);
         QSqlQuery query;
         query.exec(querystr);
         if (query.isActive() && query.numRowsAffected())
@@ -634,7 +793,6 @@ void ProgLister::fillViewList(const QString &view)
                 phrase = QString::fromUtf8(query.value(0).toString());
                 viewList << phrase;
                 viewTextList << phrase;
-                viewCount++;
             }
         }
         if (view != "")
@@ -644,11 +802,12 @@ void ProgLister::fillViewList(const QString &view)
     }
     else if (type == plTitle)
     {
-        viewList << view;
-        viewTextList << view;
-        viewCount++;
         if (view != "")
+        {
+            viewList << view;
+            viewTextList << view;
             curView = 0;
+        }
         else
             curView = -1;
     }
@@ -656,137 +815,111 @@ void ProgLister::fillViewList(const QString &view)
     {
         viewList << "";
         viewTextList << "";
-        viewCount++;
         curView = 0;
     }
+    else if (type == plTime)
+    {
+        curView = 0;
+        viewList[curView] = searchTime.toString(fullDateFormat);
+        viewTextList[curView] = viewList[curView];
+    }
 
-    if (curView >= viewCount)
-        curView = viewCount - 1;
+    if (curView >= (int)viewList.count())
+        curView = viewList.count() - 1;
 }
 
 void ProgLister::fillItemList(void)
 {
-    itemList.clear();
-    itemCount = 0;
-
     if (curView < 0)
         return;
 
     QString where;
+    QString startstr = startTime.toString("yyyyMMddhhmm50");
+    QString qphrase = viewList[curView].utf8();
+    qphrase.replace("\'", "\\\'");
 
     if (type == plTitle) // per title listings
     {
         where = QString("WHERE channel.visible = 1 "
-                        "AND program.title = \"%1\" "
-                        "AND program.endtime > %2 "
-                        "AND program.chanid = channel.chanid "
-                        "GROUP BY starttime,endtime,channum,callsign "
-                        "ORDER BY program.starttime,%3;")
-                        .arg(viewList[curView].utf8())
-                        .arg(startTime.toString("yyyyMMddhhmm50"))
-                        .arg(channelOrdering);
+                        "  AND program.endtime > %1 "
+                        "  AND program.title = '%2' ")
+                        .arg(startstr).arg(qphrase);
     }
     else if (type == plNewListings) // what's new list
     {
-        where = QString("LEFT JOIN oldprogram ON title=oldtitle "
+        where = QString("LEFT JOIN oldprogram ON "
+                        "  oldprogram.oldtitle = program.title "
                         "WHERE channel.visible = 1 "
-                        "AND oldtitle IS NULL AND program.endtime > %1 "
-                        "AND program.chanid = channel.chanid "
-                        "AND (program.category_type <> 'movie' OR "
-                        "(program.category_type = 'movie' AND "
-                        "program.airdate >= YEAR(NOW() - INTERVAL 2 YEAR))) "
-                        "GROUP BY title ORDER BY starttime LIMIT 500;")
-                        .arg(startTime.toString("yyyyMMddhhmm50"));
+                        "  AND program.endtime > %1 "
+                        "  AND oldprogram.oldtitle IS NULL "
+                        "  AND (program.category_type <> 'movie' "
+                        "    OR program.airdate >= "
+                        "      YEAR(NOW() - INTERVAL 2 YEAR)) "
+                        "GROUP BY title ")
+                        .arg(startstr);
     }
     else if (type == plTitleSearch) // keyword search
     {
         where = QString("WHERE channel.visible = 1 "
-                        "AND program.title LIKE \"\%%1\%\" "
-                        "AND program.endtime > %2 "
-                        "AND program.chanid = channel.chanid "
-                        "GROUP BY starttime,endtime,channum,callsign "
-                        "ORDER BY program.starttime,%3 "
-                        "LIMIT 500;")
-                        .arg(viewList[curView].utf8())
-                        .arg(startTime.toString("yyyyMMddhhmm50"))
-                        .arg(channelOrdering);
+                        "  AND program.endtime > %1 "
+                        "  AND program.title LIKE '\%%2\%' ")
+                        .arg(startstr).arg(qphrase);
     }
-    else if (type == plDescSearch) // description search
+    else if (type == plKeywordSearch) // keyword search
     {
         where = QString("WHERE channel.visible = 1 "
-                        "AND (program.title LIKE \"\%%1\%\" "
-                        "OR program.subtitle LIKE \"\%%2\%\" "
-                        "OR program.description LIKE \"\%%3\%\") "
-                        "AND program.endtime > %4 "
-                        "AND program.chanid = channel.chanid "
-                        "GROUP BY starttime,endtime,channum,callsign "
-                        "ORDER BY program.starttime,%5 "
-                        "LIMIT 500;")
-                        .arg(viewList[curView].utf8())
-                        .arg(viewList[curView].utf8())
-                        .arg(viewList[curView].utf8())
-                        .arg(startTime.toString("yyyyMMddhhmm50"))
-                        .arg(channelOrdering);
+                        "  AND program.endtime > %1 "
+                        "  AND (program.title LIKE '\%%2\%' "
+                        "    OR program.subtitle LIKE '\%%3\%' "
+                        "    OR program.description LIKE '\%%4\%') ")
+                        .arg(startstr).arg(qphrase).arg(qphrase).arg(qphrase);
+
+    }
+    else if (type == plPeopleSearch) // people search
+    {
+        where = QString("LEFT JOIN credits ON credits.chanid = program.chanid "
+                        "  AND credits.starttime = program.starttime "
+                        "LEFT JOIN people ON people.person = credits.person "
+                        "WHERE channel.visible = 1 "
+                        "  AND program.endtime > %1 "
+                        "  AND people.name LIKE '\%%2\%' ")
+                        .arg(startstr).arg(qphrase);
     }
     else if (type == plChannel) // list by channel
     {
         where = QString("WHERE channel.visible = 1 "
-                        "AND channel.chanid = \"%1\" "
-                        "AND program.endtime > %2 "
-                        "AND program.chanid = channel.chanid "
-                        "ORDER BY program.starttime;")
-                        .arg(viewList[curView])
-                        .arg(startTime.toString("yyyyMMddhhmm50"));
+                        "  AND program.endtime > %1 "
+                        "  AND channel.chanid = '%2' ")
+                        .arg(startstr).arg(qphrase);
     }
     else if (type == plCategory) // list by category
     {
         where = QString("WHERE channel.visible = 1 "
-                        "AND program.category = \"\%1\" "
-                        "AND program.endtime > %2 "
-                        "AND program.chanid = channel.chanid "
-                        "GROUP BY starttime,endtime,channum,callsign "
-                        "ORDER BY program.starttime,%3 "
-                        "LIMIT 500;")
-                        .arg(viewList[curView].utf8())
-                        .arg(startTime.toString("yyyyMMddhhmm50"))
-                        .arg(channelOrdering);
+                        "  AND program.endtime > %1 "
+                        "  AND program.category = '\%2' ")
+                        .arg(startstr).arg(qphrase);
     }
     else if (type == plMovies) // list movies
     {
         where = QString("WHERE channel.visible = 1 "
-                        "AND (program.category_type = 'movie' "
-                        "OR programid LIKE \"MV\%\") "
-                        "AND program.endtime > %1 "
-                        "AND program.chanid = channel.chanid "
-                        "GROUP BY starttime,endtime,channum,callsign "
-                        "ORDER BY program.starttime,%2 "
-                        "LIMIT 500;")
-                        .arg(startTime.toString("yyyyMMddhhmm50"))
-                        .arg(channelOrdering);
+                        "  AND program.endtime > %1 "
+                        "  AND program.category_type = 'movie' ")
+                        .arg(startstr);
     }
-
-    ProgramInfo::GetProgramListByQuery(db, &itemList, where);
-    itemCount = itemList.count();
-
-    if (curItem < 0 && itemCount > 0)
-        curItem = 0;
-    else if (curItem >= itemCount)
-        curItem = itemCount - 1;
-
-    vector<ProgramInfo *> recList;
-    RemoteGetAllPendingRecordings(recList);
-
-    ProgramInfo *pi;
-
-    for (pi = itemList.first(); pi; pi = itemList.next())
-        pi->FillInRecordInfo(recList);
-
-    while (!recList.empty())
+    else if (type == plTime) // list by time
     {
-        pi = recList.back();
-        delete pi;
-        recList.pop_back();
+        where = QString("WHERE channel.visible = 1 "
+                        "  AND program.starttime >= %1 ")
+                        .arg(searchTime.toString("yyyyMMddhh0000"));
     }
+
+    schedList.FromScheduler();
+    itemList.FromProgram(db, where, schedList);
+
+    if (curItem < 0 && itemList.count() > 0)
+        curItem = 0;
+    else if (curItem >= (int)itemList.count())
+        curItem = itemList.count() - 1;
 }
 
 void ProgLister::updateView(QPainter *p)
@@ -835,19 +968,19 @@ void ProgLister::updateList(QPainter *p)
             ltype->SetActive(true);
 
             int skip;
-            if (itemCount <= listsize || curItem <= listsize / 2)
+            if ((int)itemList.count() <= listsize || curItem <= listsize/2)
                 skip = 0;
-            else if (curItem >= itemCount - listsize + listsize / 2)
-                skip = itemCount - listsize;
+            else if (curItem >= (int)itemList.count() - listsize + listsize/2)
+                skip = itemList.count() - listsize;
             else
                 skip = curItem - listsize / 2;
             ltype->SetUpArrow(skip > 0);
-            ltype->SetDownArrow(skip + listsize < itemCount);
+            ltype->SetDownArrow(skip + listsize < (int)itemList.count());
 
             int i;
             for (i = 0; i < listsize; i++)
             {
-                if (i + skip >= itemCount)
+                if (i + skip >= (int)itemList.count())
                     break;
 
                 ProgramInfo *pi = itemList.at(i+skip);
@@ -880,7 +1013,7 @@ void ProgLister::updateList(QPainter *p)
         }
     }
 
-    if (itemCount == 0)
+    if (itemList.count() == 0)
         container = theme->GetSet("noprograms_list");
 
     if (container)
