@@ -13,10 +13,12 @@ enum ProgListType {
     plTitle = 1,
     plNewListings = 2,
     plTitleSearch = 3,
-    plDescSearch = 4,
+    plKeywordSearch = 4,
     plChannel = 5,
     plCategory = 6,
-    plMovies = 7
+    plMovies = 7,
+    plPeopleSearch = 8,
+    plTime = 9
 };
 
 class QSqlDatabase;
@@ -39,9 +41,13 @@ class ProgLister : public MythDialog
     void chooseEditChanged(void);
     void chooseListBoxChanged(void);
     void setViewFromEdit(void);
+    void addSearchRecord(void);
     void deleteKeyword(void);
+    void setViewFromTime(void);
     void select(void);
     void edit(void);
+    void upcoming(void);
+    void details(void);
     void chooseView(void);
 
 
@@ -55,18 +61,23 @@ class ProgLister : public MythDialog
     ProgListType type;
     QSqlDatabase *db;
     QDateTime startTime;
+    QDateTime searchTime;
+    QString dayFormat;
+    QString hourFormat;
     QString timeFormat;
+    QString fullDateFormat;
     QString channelOrdering;
     QString channelFormat;
 
+    RecSearchType searchtype;
+
     int curView;
-    int viewCount;
     QStringList viewList;
     QStringList viewTextList;
 
     int curItem;
-    int itemCount;
-    QPtrList<ProgramInfo> itemList;
+    ProgramList itemList;
+    ProgramList schedList;
 
     XMLParse *theme;
     QDomElement xmldata;
@@ -98,6 +109,9 @@ class ProgLister : public MythDialog
     MythRemoteLineEdit *chooseLineEdit;
     MythPushButton *chooseOkButton;
     MythPushButton *chooseDeleteButton;
+    MythPushButton *chooseRecordButton;
+    MythComboBox *chooseDay;
+    MythComboBox *chooseHour;
 };
 
 #endif
