@@ -1,18 +1,22 @@
 #ifndef PROGRAMINFO_H_
 #define PROGRAMINFO_H_
 
+#include "recordingtypes.h"
+
 #include <qsqldatabase.h>
 #include <qstring.h>
 #include <qdatetime.h>
 #include <qmap.h>
-#include "scheduledrecording.h"
+#include <vector>
 
-#define NUMPROGRAMLINES 35
+using namespace std;
 
-enum MarkTypes {
+#define NUMPROGRAMLINES 38
+
+typedef enum {
+    MARK_UNSET = -10,
     MARK_UPDATED_CUT = -3,
     MARK_EDIT_MODE = -2,
-    MARK_PROCESSING = -1,
     MARK_CUT_END = 0,
     MARK_CUT_START = 1,
     MARK_BOOKMARK = 2,
@@ -23,6 +27,13 @@ enum MarkTypes {
     MARK_KEYFRAME = 7,
     MARK_SCENE_CHANGE = 8,
     MARK_GOP_BYFRAME = 9
+} MarkTypes;
+
+enum CommFlagStatuses {
+    COMM_FLAG_NOT_FLAGGED = 0,
+    COMM_FLAG_DONE = 1,
+    COMM_FLAG_PROCESSING = 2,
+    COMM_FLAG_COMMFREE = 3
 };
 
 enum TranscoderStatus {
@@ -60,23 +71,27 @@ enum RecStatusType {
     rsCancelled = 6,
     rsConflict = 7,
     rsLaterShowing = 8,
-    //rsUnused = 9,
+    rsRepeat = 9,
     //rsUnused = 10,
     rsLowDiskSpace = 11,
     rsTunerBusy = 12
 };
 
 class QSqlDatabase;
+class ScheduledRecording;
+class QGridLayout;
 
 class ProgramInfo
 {
   public:
     ProgramInfo();
     ProgramInfo(const ProgramInfo &other);
+    
     ~ProgramInfo();
 
     ProgramInfo& operator=(const ProgramInfo &other);
-
+    ProgramInfo& clone(const ProgramInfo &other);
+    
     // returns 0 for one-time, 1 for weekdaily, 2 for weekly
     int IsProgramRecurring(void);
 
@@ -104,23 +119,17 @@ class ProgramInfo
                                const QString &newrecgroup);
     void ToggleRecord(QSqlDatabase *dB);
 
-    ScheduledRecording* GetScheduledRecording(QSqlDatabase *db) 
-    {
-        GetProgramRecordingStatus(db);
-        return record;
-    };
+    ScheduledRecording* GetScheduledRecording(QSqlDatabase *db);
 
-    int getRecordID(QSqlDatabase *db)
-    {
-        GetProgramRecordingStatus(db);
-        recordid = record->getRecordID();
-        return recordid;
-    }
+    int getRecordID(QSqlDatabase *db);
 
     void StartedRecording(QSqlDatabase *db);
     void FinishedRecording(QSqlDatabase* db, bool prematurestop);
 
-    QGridLayout* DisplayWidget(QWidget *parent = NULL);
+    QGridLayout* DisplayWidget(QWidget *parent = NULL,
+                               QString searchtitle = "");
+
+    void showDetails(QSqlDatabase *db);
 
     QString GetRecordBasename(void);
     QString GetRecordFilename(const QString &prefix);
@@ -130,10 +139,12 @@ class ProgramInfo
     int CalculateLength(void);
 
     void ToStringList(QStringList &list);
-    void FromStringList(QStringList &list, int offset);
-    void FromStringList(QStringList &list, QStringList::iterator &it);
+    bool FromStringList(QStringList &list, int offset);
+    bool FromStringList(QStringList &list, QStringList::iterator &it);
     void ToMap(QSqlDatabase *db, QMap<QString, QString> &progMap);
 
+    void SetFilesize(long long fsize, QSqlDatabase *db);
+    long long GetFilesize(QSqlDatabase *db);
     void SetBookmark(long long pos, QSqlDatabase *db);
     long long GetBookmark(QSqlDatabase *db);
     bool IsEditing(QSqlDatabase *db);
@@ -166,9 +177,12 @@ class ProgramInfo
     void SetMarkupFlag(int type, bool processing, QSqlDatabase *db);
     void GetPositionMap(QMap<long long, long long> &posMap, int type,
                         QSqlDatabase *db);
+    void ClearPositionMap(int type, QSqlDatabase *db);
     void SetPositionMap(QMap<long long, long long> &posMap, int type,
                         QSqlDatabase *db,
                         long long min_frame = -1, long long max_frame = -1);
+    void SetPositionMapDelta(QMap<long long, long long> &posMap, int type,
+                             QSqlDatabase *db);
 
     void DeleteHistory(QSqlDatabase *db);
     QString RecTypeChar(void);
@@ -183,17 +197,6 @@ class ProgramInfo
 
     int getProgramFlags(QSqlDatabase *db);
 
-    static void GetProgramListByQuery(QSqlDatabase *db,
-                                        QPtrList<ProgramInfo> *proglist,
-                                        const QString &where);
-    static void GetProgramRangeDateTime(QSqlDatabase *db,
-                                        QPtrList<ProgramInfo> *proglist, 
-                                        const QString &channel, 
-                                        const QString &ltime, 
-                                        const QString &rtime);
-    static ProgramInfo *GetProgramAtDateTime(QSqlDatabase *db, 
-                                             const QString &channel, 
-                                             const QString &time);
     static ProgramInfo *GetProgramAtDateTime(QSqlDatabase *db,
                                              const QString &channel, 
                                              QDateTime &dtime);
@@ -229,6 +232,15 @@ class ProgramInfo
     QDateTime recstartts;
     QDateTime recendts;
 
+    
+    bool isVideo;
+    int lenMins;
+    
+    float stars;
+    QDate originalAirDate;
+    QDateTime lastmodified;
+    
+    bool hasAirDate;
     bool repeat;
 
     int spread;
@@ -269,33 +281,52 @@ private:
     class ScheduledRecording* record;
 };
 
-class PGInfoCon
-{
-  public:
-    PGInfoCon();
-    ~PGInfoCon();
-    
-    typedef QPtrListIterator<ProgramInfo> PGInfoConIt;
-    
-    void updateAll(const vector<ProgramInfo *> &pginfoList);
-    bool update(const ProgramInfo &pginfo);
-    int count();
-    void clear();
-   
-    int selectedIndex();
-    void setSelected(int delta = 1);
-    bool getSelected(ProgramInfo &pginfo);
-    
-    PGInfoConIt iterator();
-  private:
-    void init();
-    void checkSetIndex();
-    ProgramInfo *find(const ProgramInfo &pginfo);
-    ProgramInfo *findFromKey(const QString &key);
-    ProgramInfo *findNearest(const ProgramInfo &pginfo);
-     
-    QPtrList<ProgramInfo> allData;
-    int selectedPos;
+class ProgramList: public QPtrList<ProgramInfo> {
+ public:
+    ProgramList(bool autoDelete = true) {
+        setAutoDelete(autoDelete);
+        compareFunc = NULL;
+    };
+    ~ProgramList(void) { };
+
+    ProgramInfo * operator[](uint index) {
+        return at(index);
+    };
+
+    bool FromScheduler(bool &hasConflicts);
+    bool FromScheduler(void) {
+        bool dummyConflicts;
+        return FromScheduler(dummyConflicts);
+    };
+
+    bool FromProgram(QSqlDatabase *db, const QString sql,
+                     ProgramList &schedList);
+    bool FromProgram(QSqlDatabase *db, const QString sql) {
+        ProgramList dummySched;
+        return FromProgram(db, sql, dummySched);
+    }
+
+    bool FromRecorded(QSqlDatabase *db, const QString sql,
+                      ProgramList &schedList);
+    bool FromRecorded(QSqlDatabase *db, const QString sql) {
+        ProgramList dummySched;
+        return FromRecorded(db, sql, dummySched);
+    }
+
+    bool FromRecord(QSqlDatabase *db, const QString sql);
+
+    typedef int (*CompareFunc)(ProgramInfo *p1, ProgramInfo *p2);
+    void Sort(CompareFunc func) {
+        compareFunc = func;
+        sort();
+    };
+
+ protected:
+    virtual int compareItems(ProgramInfo *p1, ProgramInfo *p2);
+
+ private:
+    CompareFunc compareFunc;
 };
 
 #endif
+
