@@ -6,6 +6,8 @@
 #include <qlayout.h>
 #include <iostream>
 
+#include "managedlist.h"
+
 #define TRANSCODER_GROUP 6
 
 QString RecordingProfileParam::whereClause(void) {
@@ -232,9 +234,9 @@ public:
 
     void selectCodecs(QString groupType)
     {
-        if(!groupType.isNull())
+        if (!groupType.isNull())
         {
-            if(groupType == "MPEG")
+            if (groupType == "MPEG")
                codecName->addSelection("MPEG-2 Hardware Encoder");
             else
             {
@@ -337,7 +339,7 @@ class MPEG4MaxQuality: public CodecParam, public SliderSetting {
 public:
     MPEG4MaxQuality(const RecordingProfile& parent):
         CodecParam(parent, "mpeg4maxquality"),
-        SliderSetting(1,31,1) {
+        SliderSetting(0,31,1) {
 
         setLabel(QObject::tr("Maximum quality"));
         setValue(2);
@@ -586,11 +588,11 @@ public:
 
     void selectCodecs(QString groupType)
     {
-        if(!groupType.isNull())
+        if (!groupType.isNull())
         {
-            if(groupType == "MPEG")
+            if (groupType == "MPEG")
                codecName->addSelection("MPEG-2 Hardware Encoder");
-            else if(groupType == "MJPEG")
+            else if (groupType == "MJPEG")
                 codecName->addSelection("Hardware MJPEG");
             else
             {
@@ -718,12 +720,12 @@ bool RecordingProfile::loadByCard(QSqlDatabase* db, QString name, int cardid) {
     if (result.isActive() && result.numRowsAffected() > 0) {
         while (result.next())
         {
-            if(result.value(1).toString() == hostname)
+            if (result.value(1).toString() == hostname)
             {
                 id = result.value(0).toInt();
                 break;
             }
-            else if(result.value(2).toInt() == 1)
+            else if (result.value(2).toInt() == 1)
                 id = result.value(0).toInt();
         }
     }
@@ -780,7 +782,7 @@ RecordingProfileEditor::RecordingProfileEditor(QSqlDatabase* _db,
     labelName = profName;
     db = _db;
     group = id;
-    if(! labelName.isNull())
+    if (! labelName.isNull())
         this->setLabel(labelName);
 }
 
@@ -797,8 +799,8 @@ int RecordingProfileEditor::exec(QSqlDatabase* db) {
     return QDialog::Rejected;
 }
 
-void RecordingProfile::fillSelections(QSqlDatabase* db, SelectSetting* setting,
-                                      int group) {
+void RecordingProfile::fillSelections(QSqlDatabase* db, SelectSetting* setting, int group)
+{
     if (group == 0)
     {
        for(int i = 0; availProfiles[i] != ""; i++)
@@ -815,7 +817,7 @@ void RecordingProfile::fillSelections(QSqlDatabase* db, SelectSetting* setting,
         {
             while (result.next())
             {
-                if(group == TRANSCODER_GROUP)
+                if (group == TRANSCODER_GROUP)
                 {
                     setting->addSelection(QString( "From %1").arg(result.value(0).toString()),
                                           result.value(1).toString());
@@ -830,6 +832,42 @@ void RecordingProfile::fillSelections(QSqlDatabase* db, SelectSetting* setting,
     }
 }
 
+void RecordingProfile::fillSelections(QSqlDatabase* db, SelectManagedListItem* setting, int group)
+{
+    if (group == 0)
+    {
+       for(int i = 0; availProfiles[i] != ""; i++)
+       {
+          QString tempLabel(QObject::tr("Record using the \"%1\" profile"));
+          setting->addSelection(QString(tempLabel).arg(availProfiles[i]), availProfiles[i], false);
+       }
+    }
+    else
+    {
+        QString query = QString("SELECT name, id FROM recordingprofiles "
+                                "WHERE profilegroup = %1 ORDER BY id;")
+                                .arg(group);
+
+        QSqlQuery result = db->exec(query);
+        if (result.isActive() && result.numRowsAffected() > 0)
+        {
+            while (result.next())
+            {
+                if (group == TRANSCODER_GROUP)
+                {
+                    setting->addSelection(QString( "From %1").arg(result.value(0).toString()),
+                                          result.value(1).toString());
+                }
+                else
+                {
+                    setting->addSelection(result.value(0).toString() ,result.value(1).toString(), false);
+
+                }
+            }
+        }
+    }
+}
+
 QString RecordingProfile::groupType(QSqlDatabase *db)
 {
     if (db != NULL)
