@@ -14,7 +14,6 @@ void RemoteQueueTranscode(ProgramInfo *pginfo, int state);
 void RemoteStopRecording(ProgramInfo *pginfo);
 void RemoteDeleteRecording(ProgramInfo *pginfo, bool forgetHistory);
 bool RemoteReactivateRecording(ProgramInfo *pginfo);
-bool RemoteGetAllPendingRecordings(vector<ProgramInfo *> &recordinglist);
 void RemoteGetAllScheduledRecordings(vector<ProgramInfo *> &scheduledlist);
 vector<ProgramInfo *> *RemoteGetConflictList(ProgramInfo *pginfo);
 void RemoteSendMessage(const QString &message);
@@ -26,6 +25,7 @@ void RemoteGeneratePreviewPixmap(ProgramInfo *pginfo);
 void RemoteFillProginfo(ProgramInfo *pginfo, const QString &playbackhostname);
 int RemoteIsRecording(void);
 int RemoteGetRecordingMask(void);
+int RemoteGetFreeRecorderCount(void);
 
 int RemoteCheckForRecording(ProgramInfo *pginfo);
 int RemoteGetRecordingStatus(ProgramInfo *pginfo, int overrecsecs, 
