@@ -24,6 +24,7 @@ using namespace std;
 #include "NuppelVideoPlayer.h"
 #include "channel.h"
 #include "mythdbcon.h"
+#include "scheduledrecording.h"
 
 #ifdef USING_IVTV
 #include "mpegrecorder.h"
@@ -58,7 +59,6 @@ TVRec::TVRec(int capturecardnum)
     autoTranscode = 0;
 
     pthread_mutex_init(&db_lock, NULL);
-    pthread_mutex_init(&commLock, NULL);
 
     ConnectDB(capturecardnum);
 
@@ -90,9 +90,13 @@ TVRec::TVRec(int capturecardnum)
                               "but no DVB support compiled in!");
         VERBOSE(VB_IMPORTANT, "Remove the card from configuration, "
                               "or recompile MythTV.");
-        exit(-1);
+        exit(-20);
 #endif
     }
+    else if ((cardtype == "MPEG") && (videodev.lower().left(5) == "file:"))
+    {
+        channel = NULL;
+    }
     else // "V4L" or "MPEG", ie, analog TV, or "HDTV"
     {
         Channel *achannel = new Channel(this, videodev, (cardtype == "HDTV"));
@@ -347,6 +351,10 @@ void TVRec::StartedRecording(void)
     MythContext::KickDatabase(db_conn);
 
     curRecording->StartedRecording(db_conn);
+
+    if (curRecording->chancommfree != 0)
+        curRecording->SetCommFlagged(COMM_FLAG_COMMFREE, db_conn);
+
     pthread_mutex_unlock(&db_lock);
 
     MythEvent me("RECORDING_LIST_CHANGE");
@@ -384,8 +392,8 @@ void TVRec::HandleStateChange(void)
 
     if (nextState == kState_Error)
     {
-        printf("ERROR: Attempting to set to an error state.\n");
-        exit(-1);
+        VERBOSE(VB_IMPORTANT, "TVRec: Attempting to set to an error state, exiting");
+        exit(-21);
     }
 
     if (tmpInternalState == kState_None && nextState == kState_WatchingLiveTV)
@@ -398,7 +406,8 @@ void TVRec::HandleStateChange(void)
     else if (tmpInternalState == kState_WatchingLiveTV && 
              nextState == kState_None)
     {
-        channel->StoreInputChannels();
+        if (channel)
+            channel->StoreInputChannels();
 
         closeRecorder = true;
 
@@ -514,16 +523,18 @@ void TVRec::HandleStateChange(void)
             nvr->SetRecording(curRecording);
             nvr->SetDB(db_conn, &db_lock);
             if (channel != NULL)
+            {
                 nvr->ChannelNameChanged(channel->GetCurrentName());
 
-            SetVideoFiltersForChannel(channel, channel->GetCurrentName());
-            if (channel->Open())
-            {
-                channel->SetBrightness();
-                channel->SetContrast();
-                channel->SetColour();
-                channel->SetHue();
-                channel->Close();
+                SetVideoFiltersForChannel(channel, channel->GetCurrentName());
+                if (channel->Open())
+                {
+                    channel->SetBrightness();
+                    channel->SetContrast();
+                    channel->SetColour();
+                    channel->SetHue();
+                    channel->Close();
+                }
             }
             pthread_create(&encode, NULL, SpawnEncode, nvr);
 
@@ -534,9 +545,10 @@ void TVRec::HandleStateChange(void)
             VERBOSE(VB_IMPORTANT, "Tuning Error -- aborting recording");
 
         if (nvr->IsRecording())
-	{
+        {
             // evil.
-            channel->SetFd(nvr->GetVideoFd());
+            if (channel)
+                channel->SetFd(nvr->GetVideoFd());
             frameRate = nvr->GetFrameRate();
         }
         else
@@ -553,7 +565,8 @@ void TVRec::HandleStateChange(void)
     if (closeRecorder)
     {
         TeardownRecorder(killRecordingFile);
-        channel->SetFd(-1);
+        if (channel)
+            channel->SetFd(-1);
     }
 
     internalState = tmpInternalState;
@@ -584,6 +597,7 @@ void TVRec::SetupRecorder(RecordingProfile &profile)
     }
     else if (cardtype == "HDTV")
     {
+        rbuffer->SetWriteBufferSize(4*1024*1024);
         nvr = new HDTVRecorder();
         nvr->SetRingBuffer(rbuffer);
 
@@ -703,7 +717,20 @@ void TVRec::TeardownRecorder(bool killFile)
         {
             if ((gContext->GetNumSetting("AutoCommercialFlag", 1)) &&
                 (prevRecording->chancommfree == 0))
-                FlagCommercials();
+            {
+                QString detectionHost =
+                            gContext->GetSetting("CommercialSkipHost");
+
+                if (detectionHost == "Default")
+                    detectionHost = gContext->GetHostName();
+
+                QString message = QString("GLOBAL_COMMFLAG START %1 %2 %3")
+                           .arg(prevRecording->chanid)
+                           .arg(prevRecording->recstartts.toString(Qt::ISODate))
+                           .arg(detectionHost);
+                MythEvent me(message);
+                gContext->dispatch(me);
+            }
 
             if (autoTranscode)
             {
@@ -753,7 +780,7 @@ char *TVRec::GetScreenGrab(ProgramInfo *pginfo, const QString &filename,
 
 void TVRec::SetChannel(bool needopen)
 {
-    if (needopen)
+    if (needopen && channel)
         channel->Open();
 
     pthread_mutex_lock(&db_lock);
@@ -786,9 +813,10 @@ void TVRec::SetChannel(bool needopen)
 
     pthread_mutex_unlock(&db_lock);
 
-    channel->SwitchToInput(inputname, chanstr);
+    if (channel)
+        channel->SwitchToInput(inputname, chanstr);
 
-    if (needopen)
+    if (needopen && channel)
         channel->Close();
 }
 
@@ -881,7 +909,9 @@ void TVRec::GetChannelInfo(ChannelBase *chan, QString &title, QString &subtitle,
                            QString &starttime, QString &endtime, 
                            QString &callsign, QString &iconpath, 
                            QString &channelname, QString &chanid,
-                           QString &seriesid, QString &programid)
+                           QString &seriesid, QString &programid,
+                           QString &outputFilters, QString &repeat, QString &airdate,
+                           QString &stars)
 {
     title = "";
     subtitle = "";
@@ -895,7 +925,11 @@ void TVRec::GetChannelInfo(ChannelBase *chan, QString &title, QString &subtitle,
     chanid = "";
     seriesid = "";
     programid = "";
-
+    outputFilters = "";
+    repeat = "";
+    airdate = "";
+    stars = "";
+    
     if (!db_conn)
         return;
     pthread_mutex_lock(&db_lock);
@@ -906,6 +940,9 @@ void TVRec::GetChannelInfo(ChannelBase *chan, QString &title, QString &subtitle,
     time_t curtime;
     struct tm *loctime;
 
+    if (!chan)
+        return;
+
     curtime = time(NULL);
     loctime = localtime(&curtime);
 
@@ -916,7 +953,8 @@ void TVRec::GetChannelInfo(ChannelBase *chan, QString &title, QString &subtitle,
  
     QString thequery = QString("SELECT starttime,endtime,title,subtitle,"
                                "description,category,callsign,icon,"
-                               "channel.chanid, seriesid, programid "
+                               "channel.chanid, seriesid, programid, "
+                               "channel.outputfilters, previouslyshown, originalairdate, stars "
                                "FROM program,channel,capturecard,cardinput "
                                "WHERE channel.channum = \"%1\" "
                                "AND starttime < %2 AND endtime > %3 AND "
@@ -940,17 +978,23 @@ void TVRec::GetChannelInfo(ChannelBase *chan, QString &title, QString &subtitle,
         subtitle = QString::fromUtf8(query.value(3).toString());
         desc = QString::fromUtf8(query.value(4).toString());
         category = QString::fromUtf8(query.value(5).toString());
-        callsign = query.value(6).toString();
+        callsign = QString::fromUtf8(query.value(6).toString());
         iconpath = query.value(7).toString();
         chanid = query.value(8).toString();
         seriesid = query.value(9).toString();
         programid = query.value(10).toString();
+        outputFilters = query.value(11).toString();
+        
+        repeat = query.value(12).toString();
+        airdate = query.value(13).toString();
+        stars = query.value(14).toString();
     }
     else
     {
         // couldn't find a matching program for the current channel.
         // get the information about the channel anyway
-        QString thequery = QString("SELECT callsign,icon,channel.chanid "
+        QString thequery = QString("SELECT callsign,icon, channel.chanid, "
+                                   "channel.outputfilters "
                                    "FROM channel,capturecard,cardinput "
                                    "WHERE channel.channum = \"%1\" AND "
                                    "channel.sourceid = cardinput.sourceid AND "
@@ -967,9 +1011,10 @@ void TVRec::GetChannelInfo(ChannelBase *chan, QString &title, QString &subtitle,
         {
             query.next();
 
-            callsign = query.value(0).toString();
+            callsign = QString::fromUtf8(query.value(0).toString());
             iconpath = query.value(1).toString();
             chanid = query.value(2).toString();
+            outputFilters = query.value(3).toString();
         }
      }
 
@@ -1101,6 +1146,9 @@ void TVRec::GetDevices(int cardnum, QString &video, QString &vbi,
 
 bool TVRec::CheckChannel(QString name)
 {
+    if (!channel)
+        return false;
+
     QSqlDatabase* dummy1;
     pthread_mutex_t* dummy2;
     QString dummyID;
@@ -1110,6 +1158,9 @@ bool TVRec::CheckChannel(QString name)
 bool TVRec::CheckChannel(ChannelBase *chan, const QString &channum, 
                          QSqlDatabase *&a_db_conn, pthread_mutex_t *&a_db_lock, QString& inputName)
 {
+    if (!chan)
+        return false;
+
     if (!db_conn)
         return true;
 
@@ -1208,6 +1259,9 @@ bool TVRec::CheckChannel(ChannelBase *chan, const QString &channum,
  */
 bool TVRec::CheckChannelPrefix(QString name, bool &unique)
 {
+    if (!channel)
+        return false;
+
     if (!db_conn)
         return true;
 
@@ -1263,6 +1317,8 @@ bool TVRec::CheckChannelPrefix(QString name, bool &unique)
 
 bool TVRec::SetVideoFiltersForChannel(ChannelBase *chan, const QString &channum)
 {
+    if (!chan)
+        return false;
 
     if (!db_conn)
         return true;
@@ -1321,6 +1377,9 @@ bool TVRec::SetVideoFiltersForChannel(ChannelBase *chan, const QString &channum)
 int TVRec::GetChannelValue(const QString &channel_field, ChannelBase *chan, 
                            const QString &channum)
 {
+    if (!chan)
+        return -1;
+
     int retval = -1;
 
     if (!db_conn)
@@ -1362,6 +1421,8 @@ int TVRec::GetChannelValue(const QString &channel_field, ChannelBase *chan,
 void TVRec::SetChannelValue(QString &field_name,int value, ChannelBase *chan,
                             const QString &channum)
 {
+    if (!chan)
+        return;
 
     if (!db_conn)
         return;
@@ -1414,6 +1475,9 @@ QString TVRec::GetNextChannel(ChannelBase *chan, int channeldirection)
 {
     QString ret = "";
 
+    if (!chan)
+        return ret;
+
     // Get info on the current channel we're on
     QString channum = chan->GetCurrentName();
     QString chanid = "";
@@ -1426,11 +1490,13 @@ QString TVRec::GetNextChannel(ChannelBase *chan, int channeldirection)
 
 QString TVRec::GetNextRelativeChanID(QString channum, int channeldirection)
 {
-
     // Get info on the current channel we're on
     QString channum_out = channum;
     QString chanid = "";
 
+    if (!channel)
+        return chanid;
+
     DoGetNextChannel(channum_out, channel->GetCurrentInput(), 
                      m_capturecardnum, channel->GetOrdering(),
                      channeldirection, chanid);
@@ -1534,7 +1600,8 @@ void TVRec::DoGetNextChannel(QString &channum, QString channelinput,
 
     QString wherepart = QString("cardinput.cardid = capturecard.cardid AND "
                                 "capturecard.cardid = \"%1\" AND "
-                                "capturecard.hostname = \"%2\" ")
+                                "capturecard.hostname = \"%2\" AND "
+                                "cardinput.sourceid = channel.sourceid ")
                                 .arg(cardid)
                                 .arg(gContext->GetHostName());
 
@@ -1671,6 +1738,7 @@ void TVRec::SetupRingBuffer(QString &path, long long &filesize,
     fillamount = fillamount * 1024 * 1024;
 
     rbuffer = new RingBuffer(path, filesize, fillamount);
+    rbuffer->SetWriteBufferMinWriteSize(1);
 }
 
 void TVRec::SpawnLiveTV(void)
@@ -1757,6 +1825,9 @@ void TVRec::ChangeChannel(int channeldirection)
 
 void TVRec::ToggleChannelFavorite(void)
 {
+    if (!channel)
+        return;
+
     // Get current channel id...
     QString channum = channel->GetCurrentName();
     QString channelinput = channel->GetCurrentInput();
@@ -1832,24 +1903,36 @@ void TVRec::ToggleChannelFavorite(void)
 
 int TVRec::ChangeContrast(bool direction)
 {
+    if (!channel)
+        return -1;
+
     int ret = channel->ChangeContrast(direction);
     return ret;
 }
 
 int TVRec::ChangeBrightness(bool direction)
 {
+    if (!channel)
+        return -1;
+
     int ret = channel->ChangeBrightness(direction);
     return ret;
 }
 
 int TVRec::ChangeColour(bool direction)
 {
+    if (!channel)
+        return -1;
+
     int ret = channel->ChangeColour(direction);
     return ret;
 }
 
 int TVRec::ChangeHue(bool direction)
 {
+    if (!channel)
+        return -1;
+
     int ret = channel->ChangeHue(direction);
     return ret;
 }
@@ -1986,15 +2069,23 @@ void TVRec::GetChannelInfo(QString &title, QString &subtitle, QString &desc,
                         QString &category, QString &starttime,
                         QString &endtime, QString &callsign, QString &iconpath,
                         QString &channelname, QString &chanid,
-                        QString &seriesid, QString &programid)
+                        QString &seriesid, QString &programid,
+                        QString &outputFilters, QString &repeat, QString &airdate,
+                        QString &stars)
 {
+    if (!channel)
+        return;
+
     GetChannelInfo(channel, title, subtitle, desc, category, starttime,
                    endtime, callsign, iconpath, channelname, chanid,
-                   seriesid, programid);
+                   seriesid, programid, outputFilters, repeat, airdate, stars);
 }
 
 void TVRec::GetInputName(QString &inputname)
 {
+    if (!channel)
+        return;
+
     inputname = channel->GetCurrentInput();
 }
 
@@ -2065,7 +2156,6 @@ void TVRec::SetReadThreadSock(QSocket *sock)
 
 int TVRec::RequestRingBufferBlock(int size)
 {
-    char buffer[256001];
     int tot = 0;
     int ret = 0;
 
@@ -2084,12 +2174,12 @@ int TVRec::RequestRingBufferBlock(int size)
         if (request > 256000)
             request = 256000;
  
-        ret = rbuffer->Read(buffer, request);
+        ret = rbuffer->Read(requestBuffer, request);
         
         if (rbuffer->GetStopReads() || ret <= 0)
             break;
         
-        if (!WriteBlock(readthreadSock->socketDevice(), buffer, ret))
+        if (!WriteBlock(readthreadSock->socketDevice(), requestBuffer, ret))
         {
             tot = -1;
             break;
@@ -2107,71 +2197,14 @@ int TVRec::RequestRingBufferBlock(int size)
     return tot;
 }
 
-void TVRec::DoFlagCommercialsThread(void)
-{
-    ProgramInfo *program_info = new ProgramInfo(*prevRecording);
-
-    flagthreadstarted = true;
-
-    QString name = QString("commercial%1%2").arg(getpid()).arg(rand());
-
-    MythSqlDatabase *commthread_db = new MythSqlDatabase(name);
-
-    if (!commthread_db || !commthread_db->isOpen())
-        return;
-
-    nice(19);
-
-    pthread_mutex_lock(&commLock);
-    commercialFlag.append(program_info);
-    pthread_mutex_unlock(&commLock);
-
-    QString filename = program_info->GetRecordFilename(
-                                gContext->GetSetting("RecordFilePrefix"));
-
-    RingBuffer *tmprbuf = new RingBuffer(filename, false);
-
-    NuppelVideoPlayer *nvp = new NuppelVideoPlayer(commthread_db, program_info);
-    nvp->SetRingBuffer(tmprbuf);
-
-    nvp->FlagCommercials();
-
-    pthread_mutex_lock(&commLock);
-    commercialFlag.removeRef(program_info);
-    pthread_mutex_unlock(&commLock);
-
-    delete nvp;
-    delete tmprbuf;
-    delete program_info;
-    delete commthread_db;
-}
-
-void *TVRec::FlagCommercialsThread(void *param)
-{
-    TVRec *thetv = (TVRec *)param;
-    thetv->DoFlagCommercialsThread();
-
-    return NULL;
-}
-
-void TVRec::FlagCommercials(void)
-{
-    flagthreadstarted = false;
-
-    pthread_attr_t attr;
-    pthread_attr_init(&attr);
-    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-    pthread_create(&commercials, &attr, FlagCommercialsThread, this);
-
-    while (!flagthreadstarted)
-        usleep(50);
-}
-
 void TVRec::RetrieveInputChannels(map<int, QString> &inputChannel,
                                   map<int, QString> &inputTuneTo,
                                   map<int, QString> &externalChanger,
                                   map<int, QString> &sourceid)
 {
+    if (!channel)
+        return;
+
     pthread_mutex_lock(&db_lock);
     MythContext::KickDatabase(db_conn);
 
@@ -2211,6 +2244,9 @@ void TVRec::RetrieveInputChannels(map<int, QString> &inputChannel,
 
 void TVRec::StoreInputChannels(map<int, QString> &inputChannel)
 {
+    if (!channel)
+        return;
+
     QString query, input;
 
     pthread_mutex_lock(&db_lock);
@@ -2235,19 +2271,3 @@ void TVRec::StoreInputChannels(map<int, QString> &inputChannel)
     pthread_mutex_unlock(&db_lock);
 }
 
-bool TVRec::isParsingCommercials(ProgramInfo *pginfo)
-{
-    ProgramInfo *pinfo;
-    pthread_mutex_lock(&commLock);
-    for (pinfo = commercialFlag.first(); pinfo; pinfo = commercialFlag.next())
-    {
-        if(pginfo->chanid == pinfo->chanid || pginfo->recstartts == pinfo->recstartts)
-        {
-            pthread_mutex_unlock(&commLock);
-            return true;
-        }
-    }
-    pthread_mutex_unlock(&commLock);
-    return false;
-}
-
