@@ -22,73 +22,73 @@ using namespace std;
 {\
     DFBResult err = dfbcommand;\
 \
-	if (err != DFB_OK)\
-	{\
-		fprintf( stderr, "%s <%d>:\n\t", __FILE__, __LINE__ );\
-		DirectFBError( #dfbcommand, err );\
-		return returnstmt;\
-	}\
+        if (err != DFB_OK)\
+        {\
+                fprintf( stderr, "%s <%d>:\n\t", __FILE__, __LINE__ );\
+                DirectFBError( #dfbcommand, err );\
+                return returnstmt;\
+        }\
 }
 
 #define DFBCHECK(x...)\
 {\
     DFBResult err = x;\
 \
-	if (err != DFB_OK)\
-	{\
-		fprintf( stderr, "%s <%d>:\n\t", __FILE__, __LINE__ );\
-		DirectFBError( #x, err );\
-	}\
+        if (err != DFB_OK)\
+        {\
+                fprintf( stderr, "%s <%d>:\n\t", __FILE__, __LINE__ );\
+                DirectFBError( #x, err );\
+        }\
 }
 #define DFB_KBID_OFFSET 62976
 
 #define INCREMENT(x, step, min, max)\
 {\
-	int __n;\
-	__n = x + step;\
-	x = ( __n < min )	? min : ( ( __n > max ) ? max : __n );\
+        int __n;\
+        __n = x + step;\
+        x = ( __n < min )       ? min : ( ( __n > max ) ? max : __n );\
 }
 
 static const unsigned int QT_KEYS[DIKI_NUMBER_OF_KEYS][2] =
     {
         {0xffff,0 }, // unknown key
-        {0x41,	0x61},	// a...z
-        {0x42,	0x62},
-        {0x43,	0x63},
-        {0x44,	0x64},
-        {0x45,	0x65},
-        {0x46,	0x66},
-        {0x47,	0x67},
-        {0x48,	0x68},
-        {0x49,	0x69},
-        {0x4a,	0x6a},
-        {0x4b,	0x8b},
-        {0x4c,	0x6c},
-        {0x4d,	0x6d},
-        {0x4e,	0x6e},
-        {0x4f,	0x6f},
-        {0x50,	0x70},
-        {0x51,	0x71},
-        {0x52,	0x72},
-        {0x53,	0x73},
-        {0x54,	0x74},
-        {0x55,	0x75},
-        {0x56,	0x76},
-        {0x57,	0x77},
-        {0x58,	0x78},
-        {0x59,	0x79},
-        {0x5a,	0x7a},
-        {0x30,	0x30},	// 0-9
-        {0x31,	0x31},
-        {0x32,	0x32},
-        {0x33,	0x33},
-        {0x34,	0x34},
-        {0x35,	0x35},
-        {0x36,	0x36},
-        {0x37,	0x37},
-        {0x38,	0x38},
-        {0x39,	0x39},
-        {0x1030,0x00},	// function keys F1 - F12
+        {0x41,  0x61},  // a...z
+        {0x42,  0x62},
+        {0x43,  0x63},
+        {0x44,  0x64},
+        {0x45,  0x65},
+        {0x46,  0x66},
+        {0x47,  0x67},
+        {0x48,  0x68},
+        {0x49,  0x69},
+        {0x4a,  0x6a},
+        {0x4b,  0x8b},
+        {0x4c,  0x6c},
+        {0x4d,  0x6d},
+        {0x4e,  0x6e},
+        {0x4f,  0x6f},
+        {0x50,  0x70},
+        {0x51,  0x71},
+        {0x52,  0x72},
+        {0x53,  0x73},
+        {0x54,  0x74},
+        {0x55,  0x75},
+        {0x56,  0x76},
+        {0x57,  0x77},
+        {0x58,  0x78},
+        {0x59,  0x79},
+        {0x5a,  0x7a},
+        {0x30,  0x30},  // 0-9
+        {0x31,  0x31},
+        {0x32,  0x32},
+        {0x33,  0x33},
+        {0x34,  0x34},
+        {0x35,  0x35},
+        {0x36,  0x36},
+        {0x37,  0x37},
+        {0x38,  0x38},
+        {0x39,  0x39},
+        {0x1030,0x00},  // function keys F1 - F12
         {0x1031,0x00},
         {0x1032,0x00},
         {0x1033,0x00},
@@ -100,51 +100,51 @@ static const unsigned int QT_KEYS[DIKI_NUMBER_OF_KEYS][2] =
         {0x1039,0x00},
         {0x103a,0x00},
         {0x103b,0x00},
-        {0x1020,0x00},	// Shift Left
-        {0x1020,0x00},	// Shift Right
+        {0x1020,0x00},  // Shift Left
+        {0x1020,0x00},  // Shift Right
         {0x1021,0x00}, // Control Left
-        {0x1021,0x00},	// Control Right
-        {0x1023,0x00},	// ALT Left
-        {0x1023,0x00},	// ALT Right
+        {0x1021,0x00},  // Control Right
+        {0x1023,0x00},  // ALT Left
+        {0x1023,0x00},  // ALT Right
         {0xffff,0x00}, // DIKS_ALTGR  not sure what QT Key is
-        {0x1022,0x00},	// META Left
-        {0x1022,0x00},	// META Right
+        {0x1022,0x00},  // META Left
+        {0x1022,0x00},  // META Right
         {0x1053,0x00}, // Super Left
-        {0x1054,0x00},	// Super Right
-        {0x1056,0x00},	// Hyper Left
-        {0x1057,0x00},	// Hyper Right
-        {0x1024,0x00},	// CAPS Lock
-        {0x1025,0x00},	// NUM Locka
-        {0x1026,0x00},	// Scroll Lock
-        {0x1000,0x1b},	// Escape
-        {0x1012,0x00},	// Left
-        {0x1014,0x00},	// Right
-        {0x1013,0x00},	// Up
-        {0x1015,0x00},	// Down
-        {0x1001,0x09},	// Tab
-        {0x1004,0x0d},	// Enter
-        {0x20,	0x20},	// 7 bit printable ASCII
-        {0x1003,0x00},	// Backspace
-        {0x1006,0x00},	// Insert
-        {0x1007,0x7f},	// Delete
-        {0x1010,0x00},	// Home
-        {0x1011,0x00},	// End
-        {0x1016,0x00},	// Page Up
-        {0x1017,0x00},	// Page Down
-        {0x1009,0x00},	// Print
-        {0x1008,0x00},	// Pause
-        {0x60,	0x27},	// Quote Left
-        {0x2d,	0x2d},	// Minus
-        {0x3d,	0x3d},	// Equals
-        {0x5b,	0x5b},	// Bracket Left
-        {0x5d,	0x5d},	// Bracket Right
-        {0x5c,	0x5c},	// Back Slash
-        {0x3b,	0x3b},	// Semicolon
+        {0x1054,0x00},  // Super Right
+        {0x1056,0x00},  // Hyper Left
+        {0x1057,0x00},  // Hyper Right
+        {0x1024,0x00},  // CAPS Lock
+        {0x1025,0x00},  // NUM Locka
+        {0x1026,0x00},  // Scroll Lock
+        {0x1000,0x1b},  // Escape
+        {0x1012,0x00},  // Left
+        {0x1014,0x00},  // Right
+        {0x1013,0x00},  // Up
+        {0x1015,0x00},  // Down
+        {0x1001,0x09},  // Tab
+        {0x1004,0x0d},  // Enter
+        {0x20,  0x20},  // 7 bit printable ASCII
+        {0x1003,0x00},  // Backspace
+        {0x1006,0x00},  // Insert
+        {0x1007,0x7f},  // Delete
+        {0x1010,0x00},  // Home
+        {0x1011,0x00},  // End
+        {0x1016,0x00},  // Page Up
+        {0x1017,0x00},  // Page Down
+        {0x1009,0x00},  // Print
+        {0x1008,0x00},  // Pause
+        {0x60,  0x27},  // Quote Left
+        {0x2d,  0x2d},  // Minus
+        {0x3d,  0x3d},  // Equals
+        {0x5b,  0x5b},  // Bracket Left
+        {0x5d,  0x5d},  // Bracket Right
+        {0x5c,  0x5c},  // Back Slash
+        {0x3b,  0x3b},  // Semicolon
         {0xffff,0x00},  // DIKS_QUOTE_RIGHT not sure what QT Key is...
-        {0x2c,	0x2c},	// Comma
-        {0x2e,	0x2e},	// Period
-        {0x2f,	0x2f},	// Slash
-        {0x3c,	0x3c},	// Less Than
+        {0x2c,  0x2c},  // Comma
+        {0x2e,  0x2e},  // Period
+        {0x2f,  0x2f},  // Slash
+        {0x3c,  0x3c},  // Less Than
 
         // keypad keys.
         // from what i can tell QT doiesnt have a seperate key code for them.
@@ -175,12 +175,17 @@ static const unsigned int QT_KEYS[DIKI_NUMBER_OF_KEYS][2] =
         {0xffff,0x00}
     };
 
+#ifndef DSCAPS_DOUBLE
+#define DSCAPS_DOUBLE DSCAPS_FLIPPING
+#endif
+
 //8 1 3 2 works, but not for everyone :-(
 const int kNumBuffers = 31;
 const int kNeedFreeFrames = 1;
 const int kPrebufferFrames = 12;
 const int kKeepPrebuffer = 2;
 typedef map<unsigned char *, IDirectFBSurface *> BufferMap;
+
 struct DirectfbData
 {
     //DirectFB hook
@@ -235,7 +240,7 @@ VideoOutputDirectfb::~VideoOutputDirectfb()
         tempYV12Surface->Release(tempYV12Surface);
     if (data->primarySurface)
         data->primarySurface->Release(data->primarySurface);
-    if(data->primaryLayer)
+    if (data->primaryLayer)
         data->primaryLayer->Release(data->primaryLayer);
     if (data->videoLayer)
         data->videoLayer->Release(data->videoLayer);
@@ -336,7 +341,7 @@ bool VideoOutputDirectfb::Init(int width, int height, float aspect, WId winid,
     //clear primary layer
     desc.flags = DSDESC_CAPS;
     desc.caps = DSCAPS_PRIMARY;
-    if(data->cardCapabilities.acceleration_mask & DFXL_BLIT)
+    if (data->cardCapabilities.acceleration_mask & DFXL_BLIT)
         desc.caps = (DFBSurfaceCapabilities)(desc.caps | DSCAPS_DOUBLE);
     DFBCHECKFAIL(data->dfb->CreateSurface(data->dfb, &desc, &(data->primarySurface)), false);
     DFBCHECKFAIL(data->primarySurface->Clear(data->primarySurface, 0, 0, 0, 0xff), false);
@@ -350,10 +355,10 @@ bool VideoOutputDirectfb::Init(int width, int height, float aspect, WId winid,
 
     DFBCHECK(data->dfb->EnumDisplayLayers(data->dfb, LayerCallback, data));
 
-    if(data->videoLayer == NULL) {
+    if (data->videoLayer == NULL) {
         data->videoLayerConfig.pixelformat = DSPF_YV12;
         DFBCHECK(data->dfb->EnumDisplayLayers(data->dfb, LayerCallback, data));
-        if(data->videoLayer == NULL) {
+        if (data->videoLayer == NULL) {
             ret = data->primaryLayer->TestConfiguration(data->primaryLayer, &(data->videoLayerConfig), NULL);
             if (DFB_OK == ret) {
                 data->primaryLayer->AddRef(data->primaryLayer);
@@ -428,7 +433,7 @@ bool VideoOutputDirectfb::Init(int width, int height, float aspect, WId winid,
            );
     //prepare to do a software conversion when the output format is Yvu
     //(DirectFB does not support software conversion from Yuv -> Yvu yet)
-    if(data->videoLayerConfig.pixelformat == DSPF_YV12)
+    if (data->videoLayerConfig.pixelformat == DSPF_YV12)
     {
         desc.flags = (DFBSurfaceDescriptionFlags)(DSDESC_HEIGHT | DSDESC_WIDTH | DSDESC_PIXELFORMAT | DSDESC_CAPS);
         desc.width = width;
@@ -469,10 +474,15 @@ bool VideoOutputDirectfb::Init(int width, int height, float aspect, WId winid,
     //display video output
     DFBCHECK(data->videoLayer->SetOpacity(data->videoLayer, 0xff));
 
-    w_mm = data->screen_width;
-    h_mm = data->screen_height;
-    //w_mm = DisplayWidthMM(data->XJ_disp, XJ_screen_num);
-    //h_mm = DisplayHeightMM(data->XJ_disp, XJ_screen_num);
+    if (myth_dsw != 0)
+        w_mm = myth_dsw;
+    else
+        w_mm = data->screen_width;
+
+    if (myth_dsh != 0)
+        h_mm = myth_dsh;
+    else
+        h_mm = data->screen_height;
 
     XJ_started = true;
     return true;
@@ -483,14 +493,14 @@ float VideoOutputDirectfb::GetDisplayAspect(void)
     return (float)data->screen_width / (float)data->screen_height;
 }
 
-void VideoOutputDirectfb::PrepareFrame(VideoFrame *buffer)
+void VideoOutputDirectfb::PrepareFrame(VideoFrame *buffer, FrameScanType t)
 {
     if (!buffer)
         buffer=scratchFrame;
 
     IDirectFBSurface *bufferSurface = data->buffers[buffer->buf];
 
-    if(data->videoLayerConfig.pixelformat == DSPF_YV12)
+    if (data->videoLayerConfig.pixelformat == DSPF_YV12)
     {
         //do a software conversion in a temporary memory buffer, since DirectFB does not handle this (yet ?)
         int pitch;
@@ -514,9 +524,9 @@ void VideoOutputDirectfb::PrepareFrame(VideoFrame *buffer)
         DFBCHECK(tempYV12Surface->Unlock(tempYV12Surface));
         bufferSurface = tempYV12Surface;
     }
-    if(!bufferSurface)
+    if (!bufferSurface)
         return;
-    if((data->cardCapabilities.acceleration_mask & DFXL_BLIT) > 0)
+    if ((data->cardCapabilities.acceleration_mask & DFXL_BLIT) > 0)
     {
         DFBCHECK(data->videoSurface->Blit(data->videoSurface, bufferSurface, NULL, 0, 0));
     }
@@ -595,6 +605,8 @@ void VideoOutputDirectfb::ProcessFrame(VideoFrame *frame, OSD *osd,
         CopyFrame(scratchFrame, &pauseFrame);
     }
 
+    if (m_deinterlacing && m_deintFilter != NULL)
+	m_deintFilter->ProcessFrame(frame);
     if (filterList)
         filterList->ProcessFrame(frame);
 
@@ -654,7 +666,7 @@ void VideoOutputDirectfb::MoveResize(void)
             .arg(dispwoff)
             .arg(disphoff));
     //**FIXME support for zooming when dispwoff > screenwidth || disphoff > screenheight
-    if(data->videoLayerDesc.caps & DLCAPS_SCREEN_LOCATION) {
+    if (data->videoLayerDesc.caps & DLCAPS_SCREEN_LOCATION) {
         DFBCHECK(data->videoLayer->SetScreenLocation(data->videoLayer,
                  (float)dispxoff/(float)data->screen_width,
                  (float)dispyoff/(float)data->screen_height,
@@ -707,7 +719,7 @@ void VideoOutputDirectfb::DeleteDirectfbBuffers(void)
     BufferMap::iterator iter;
     for ( iter = data->buffers.begin() ; iter != data->buffers.end() ; iter++ )
     {
-        if(iter->second) {
+        if (iter->second) {
             DFBCHECK(iter->second->Unlock(iter->second));
             DFBCHECK(iter->second->Release(iter->second));
         }
@@ -763,7 +775,7 @@ DFBEnumerationResult LayerCallback(unsigned int id,
     DFBResult ret;
     IDirectFBSurface *surface;
 
-    if(id == DLID_PRIMARY)
+    if (id == DLID_PRIMARY)
         return DFENUM_OK;
 
     DFBCHECKFAIL(vodata->dfb->GetDisplayLayer(vodata->dfb, id, &(vodata->videoLayer)), DFENUM_OK);
