@@ -12,8 +12,8 @@
 #include <cerrno>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
-#include <sys/user.h>
 #include <sys/poll.h>
+#include <sys/param.h>
 
 #include <map>
 #include <iostream>
@@ -230,7 +230,7 @@ void VideoOutputIvtv::Open(void)
         return;
 
     if ((videofd = open(videoDevice.ascii(), 
-        O_WRONLY | O_LARGEFILE | O_NONBLOCK, 0555)) < 0)
+        O_WRONLY | O_NONBLOCK, 0555)) < 0)
     {
         perror("Cannot open ivtv video out device");
         return;
@@ -296,9 +296,10 @@ void VideoOutputIvtv::StopEmbedding(void)
     VideoOutput::StopEmbedding();
 }
 
-void VideoOutputIvtv::PrepareFrame(VideoFrame *buffer)
+void VideoOutputIvtv::PrepareFrame(VideoFrame *buffer, FrameScanType t)
 {
     (void)buffer;
+    (void)t;
 }
 
 void VideoOutputIvtv::Show(FrameScanType )
@@ -393,6 +394,8 @@ void VideoOutputIvtv::ShowPip(VideoFrame *frame, NuppelVideoPlayer *pipplayer)
         memcpy(frame->buf + (i + yoff) * frame->width + xoff * 4,
                outputbuf + i * pipw * 4, pipw * 4);
     }
+
+    delete [] outputbuf;
 }
 
 void VideoOutputIvtv::ProcessFrame(VideoFrame *frame, OSD *osd,
