@@ -37,7 +37,7 @@ extern "C" {
 #include <X11/extensions/Xinerama.h>
 }
 
-const int kNumBuffers = 4;
+const int kNumBuffers = 7;
 const int kPrebufferFrames = 1;
 const int kNeedFreeFrames = 2;
 const int kKeepPrebuffer = 1;
@@ -52,13 +52,12 @@ struct ViaData
     Display *XJ_disp;
     float display_aspect;
 
-    via_slice_state_t decode_buffers[4];
+    via_slice_state_t decode_buffers[kNumBuffers];
     via_slice_state_t overlay_buffer;
 
     unsigned char *buffer;
     unsigned char *current;
-    unsigned char *subp_buffer;
-   
+
     DDLOCK ddLock;
     VIASUBPICT VIASubPict;
     int fd;
@@ -71,6 +70,7 @@ struct ViaData
 
     VIAMPGSURFACE VIAMPGSurface;
 
+    VideoFrame *display_buffer;
     int display_frame;
 };
 
@@ -173,8 +173,15 @@ bool VideoOutputVIA::Init(int width, int height, float aspect,
     data->XJ_screen = DefaultScreenOfDisplay(data->XJ_disp);
     XJ_screen_num = DefaultScreen(data->XJ_disp);
 
-    w_mm = DisplayWidthMM(data->XJ_disp, XJ_screen_num);
-    h_mm = DisplayHeightMM(data->XJ_disp, XJ_screen_num);
+    if (myth_dsw != 0)
+        w_mm = myth_dsw;
+    else
+        w_mm = DisplayWidthMM(data->XJ_disp, XJ_screen_num);
+
+    if (myth_dsh != 0)
+        h_mm = myth_dsh;
+    else
+        h_mm = DisplayHeightMM(data->XJ_disp, XJ_screen_num);
 
     usingXinerama = (XineramaQueryExtension(data->XJ_disp, &event_base, 
                                             &error_base) &&
@@ -193,9 +200,9 @@ bool VideoOutputVIA::Init(int width, int height, float aspect,
         int gui_w = gContext->GetNumSetting("GuiWidth", w);
         int gui_h = gContext->GetNumSetting("GuiHeight", h);
 
-		if (gui_w)
+                if (gui_w)
             w_mm = w_mm * gui_w / w;
-		if (gui_h)
+                if (gui_h)
             h_mm = h_mm * gui_h / h;
 
         data->display_aspect = (float)w_mm/h_mm;
@@ -231,10 +238,13 @@ bool VideoOutputVIA::Init(int width, int height, float aspect,
     XJ_depth = DefaultDepthOfScreen(data->XJ_screen);
 
     data->current = data->buffer = new unsigned char[1920 * 1080 * 2];
-    data->subp_buffer = new unsigned char[1920 * 1080];
-    memset(data->subp_buffer, 0, 1920 * 1080);
-   
-    VIADriverProc(CREATEDRIVER, NULL);
+
+    if (VIADriverProc(CREATEDRIVER, NULL))
+    {
+        VERBOSE(VB_IMPORTANT, "Unable to initialize VIA CLE266 HW Driver, ");
+        VERBOSE(VB_IMPORTANT, "please check your installation, exiting");
+        exit(-23);
+    }
 
     if (!CreateViaBuffers())
         return false;
@@ -253,18 +263,21 @@ bool VideoOutputVIA::CreateViaBuffers(void)
 {
     data->ddSurfaceDesc.dwWidth = XJ_width;
     data->ddSurfaceDesc.dwHeight = XJ_height;
-    data->ddSurfaceDesc.dwBackBufferCount = 3;
+    data->ddSurfaceDesc.dwBackBufferCount = kNumBuffers;
     data->ddSurfaceDesc.dwFourCC = FOURCC_VIA;
-    VIADriverProc(CREATESURFACE, &data->ddSurfaceDesc);
+    if (VIADriverProc(CREATESURFACE, &data->ddSurfaceDesc))
+        return false;
 
     data->ddSurfaceDesc.dwWidth  = XJ_width;
     data->ddSurfaceDesc.dwHeight = XJ_height;
     data->ddSurfaceDesc.dwBackBufferCount = 1;
     data->ddSurfaceDesc.dwFourCC = FOURCC_SUBP;
-    VIADriverProc(CREATESURFACE, &data->ddSurfaceDesc);
+    if (VIADriverProc(CREATESURFACE, &data->ddSurfaceDesc))
+        return false;
 
     data->ddLock.dwFourCC = FOURCC_SUBP;
-    VIADriverProc(LOCKSURFACE, &data->ddLock);
+    if (VIADriverProc(LOCKSURFACE, &data->ddLock))
+        return false;
 
     data->fd = open("/dev/mem", O_RDWR);
     if (data->fd >= 0)
@@ -275,9 +288,10 @@ bool VideoOutputVIA::CreateViaBuffers(void)
                                            (off_t)data->ddLock.dwPhysicalBase);
         if (data->lpSubSurface == MAP_FAILED)
         {
-            perror("mmap");
-            cerr << "Couldn't map overlay surface\n";
-            data->lpSubSurface = NULL;
+            perror("MMAP ViaSlice Surface failed");
+            VERBOSE(VB_IMPORTANT, "Couldn't map overlay surface");
+            VERBOSE(VB_IMPORTANT, "You should run MythTV as root when using ViaSlice");
+            exit(-24);
         }
 
         data->lpSubSurface += data->ddLock.SubDev.dwSUBPhysicalAddr[0];
@@ -298,22 +312,27 @@ bool VideoOutputVIA::CreateViaBuffers(void)
         VIASUBPicture(&data->VIASubPict);
     }
     else
-        cerr << "Couldn't open /dev/mem to map overlay surface\n";
-     
-    for (int i = 0; i < 4; i++)
     {
-         data->decode_buffers[i].image_number = i;
-	 data->decode_buffers[i].slice_data = NULL;
-	 data->decode_buffers[i].slice_datalen = 0;
-         data->decode_buffers[i].lastcode = 0;
-         data->decode_buffers[i].slicecount = 1;
+        perror("Open /dev/mem failed");
+        VERBOSE(VB_IMPORTANT, "Couldn't open /dev/mem to map overlay surface");
+        VERBOSE(VB_IMPORTANT, "You should run MythTV as root when using ViaSlice");
+        exit(-25);
+    }
+     
+    for (int i = 0; i < kNumBuffers; i++)
+    {
+        data->decode_buffers[i].image_number = i;
+        data->decode_buffers[i].slice_data = NULL;
+        data->decode_buffers[i].slice_datalen = 0;
+        data->decode_buffers[i].lastcode = 0;
+        data->decode_buffers[i].slicecount = 1;
 
-	 vbuffers[i].buf = (unsigned char *)&(data->decode_buffers[i]);
-	 vbuffers[i].height = XJ_height;
-	 vbuffers[i].width = XJ_width;
-	 vbuffers[i].bpp = -1;
-	 vbuffers[i].size = sizeof(via_slice_state_t);
-	 vbuffers[i].codec = FMT_VIA_HWSLICE;
+        vbuffers[i].buf = (unsigned char *)&(data->decode_buffers[i]);
+        vbuffers[i].height = XJ_height;
+        vbuffers[i].width = XJ_width;
+        vbuffers[i].bpp = -1;
+        vbuffers[i].size = sizeof(via_slice_state_t);
+        vbuffers[i].codec = FMT_VIA_HWSLICE;
     }
 
     return true;
@@ -328,12 +347,11 @@ void VideoOutputVIA::Exit(void)
         DeleteViaBuffers();
 
         VIADriverProc(DESTROYDRIVER, NULL);
-	
+        
         XFreeGC(data->XJ_disp, data->XJ_gc);
         XCloseDisplay(data->XJ_disp);
 
         delete [] data->buffer;
-        delete [] data->subp_buffer;
     }
 }
 
@@ -376,12 +394,13 @@ void VideoOutputVIA::StopEmbedding(void)
     pthread_mutex_unlock(&lock);
 }
 
-void VideoOutputVIA::PrepareFrame(VideoFrame *buffer)
+void VideoOutputVIA::PrepareFrame(VideoFrame *buffer, FrameScanType t)
 {
     // pause update
     if (!buffer)
         return;
 
+    data->display_buffer = buffer;
     via_slice_state_t *curdata = (via_slice_state_t *)buffer->buf;
 
     data->display_frame = curdata->image_number;
@@ -396,10 +415,34 @@ void VideoOutputVIA::PrepareFrame(VideoFrame *buffer)
 void VideoOutputVIA::Show(FrameScanType )
 {
     data->VIAMPGSurface.dwTaskType = VIA_TASK_DISPLAY;
-    data->VIAMPGSurface.dwDisplayPictStruct = VIA_PICT_STRUCT_FRAME;
     data->VIAMPGSurface.dwDisplayBuffIndex = data->display_frame;
 
-    VIADisplayControl(DEV_MPEG, &data->VIAMPGSurface);
+    via_slice_state_t *cur = (via_slice_state_t *)data->display_buffer->buf;
+
+    if (1) //cur->progressive_sequence)
+    {
+        data->VIAMPGSurface.dwDisplayPictStruct = VIA_PICT_STRUCT_FRAME;
+        VIADisplayControl(DEV_MPEG, &data->VIAMPGSurface);
+    }
+#if 0  // Needs fixed to work with new bob deint.
+    else
+    {
+        if (cur->top_field_first)
+        {
+            data->VIAMPGSurface.dwDisplayPictStruct = VIA_PICT_STRUCT_BOTTOM;
+            VIADisplayControl(DEV_MPEG, &data->VIAMPGSurface);
+            data->VIAMPGSurface.dwDisplayPictStruct = VIA_PICT_STRUCT_TOP;
+            VIADisplayControl(DEV_MPEG, &data->VIAMPGSurface);
+        }
+        else
+        {
+            data->VIAMPGSurface.dwDisplayPictStruct = VIA_PICT_STRUCT_TOP;
+            VIADisplayControl(DEV_MPEG, &data->VIAMPGSurface);
+            data->VIAMPGSurface.dwDisplayPictStruct = VIA_PICT_STRUCT_BOTTOM;
+            VIADisplayControl(DEV_MPEG, &data->VIAMPGSurface);
+        }
+    }
+#endif
 }
 
 void VideoOutputVIA::DrawSlice(VideoFrame *frame, int x, int y, int w, int h)
@@ -413,8 +456,16 @@ void VideoOutputVIA::DrawSlice(VideoFrame *frame, int x, int y, int w, int h)
 
     if (curdata->lastcode > curdata->code )
     {
+#if 0  // Crashes the decoder?
+
         //Slice must be in next frame so flush last buffer first.
         VIASliceReceiveData(curdata->slicecount - 1, data->buffer);
+
+        data->VIAMPGSurface.dwTaskType = VIA_TASK_DECODE;
+        data->VIAMPGSurface.dwDisplayPictStruct = VIA_PICT_STRUCT_FRAME;
+        data->VIAMPGSurface.dwDisplayBuffIndex = curdata->image_number;
+        VIADisplayControl(DEV_MPEG, &data->VIAMPGSurface);
+#endif
         curdata->slicecount = 1;
     }
 
@@ -448,6 +499,12 @@ void VideoOutputVIA::DrawSlice(VideoFrame *frame, int x, int y, int w, int h)
     {
         // Send slices to HW Decoder
         VIASliceReceiveData(curdata->slicecount, data->buffer);
+
+        data->VIAMPGSurface.dwTaskType = VIA_TASK_DECODE;
+        data->VIAMPGSurface.dwDisplayPictStruct = VIA_PICT_STRUCT_FRAME;
+        data->VIAMPGSurface.dwDisplayBuffIndex = curdata->image_number;
+        VIADisplayControl(DEV_MPEG, &data->VIAMPGSurface);
+
         curdata->slicecount = 1;
         curdata->lastcode = 0;
     } 
@@ -461,10 +518,10 @@ void VideoOutputVIA::DrawSlice(VideoFrame *frame, int x, int y, int w, int h)
 void VideoOutputVIA::DrawUnusedRects(void)
 {
     XSetForeground(data->XJ_disp, data->XJ_gc, XJ_black);
-    XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, dispx, dispy, 
-                   dispw, dispyoff);
-    XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, dispx,
-                   dispyoff + disphoff, dispw, dispyoff);
+
+    // Clear all background, so that the guide also get's nice video.
+    XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc,
+                   dispx, dispy, dispw, disph);
 
     data->ddUpdateOverlay.rDest.left = dispxoff;
     data->ddUpdateOverlay.rDest.top = dispyoff;
@@ -476,7 +533,7 @@ void VideoOutputVIA::DrawUnusedRects(void)
     data->ddUpdateOverlay.rSrc.right = imgx + imgw;
     data->ddUpdateOverlay.rSrc.bottom = imgy + imgh;
 
-    data->ddUpdateOverlay.dwFlags = DDOVER_SHOW;
+    data->ddUpdateOverlay.dwFlags = DDOVER_SHOW | DDOVER_KEYDEST;
 
     VIADriverProc(UPDATEOVERLAY, &data->ddUpdateOverlay);
 
