@@ -3,6 +3,7 @@
 
 struct XvData;
 
+#include <DisplayRes.h>
 #include "videooutbase.h"
 
 class NuppelVideoPlayer;
@@ -15,7 +16,8 @@ class VideoOutputXv : public VideoOutput
 
     bool Init(int width, int height, float aspect, WId winid,
               int winx, int winy, int winw, int winh, WId embedid = 0);
-    void PrepareFrame(VideoFrame *buffer);
+    bool ApproveDeintFilter(const QString& filtername) const;
+    void PrepareFrame(VideoFrame *buffer, FrameScanType);
     void Show(FrameScanType );
 
     void InputChanged(int width, int height, float aspect);
@@ -47,6 +49,8 @@ class VideoOutputXv : public VideoOutput
     void DeleteShmBuffers(void);
     void DeleteXBuffers(void);
 
+    DisplayRes * display_res;
+
     XvData *data;
 
     int XJ_screen_num;
@@ -62,6 +66,11 @@ class VideoOutputXv : public VideoOutput
 
     int use_shm;
 
+    long long framesShown;
+    int showFrame;
+    int fps;
+    time_t stop_time;
+
     unsigned char *scratchspace;
 
     pthread_mutex_t lock;
