@@ -31,10 +31,18 @@
 #include "videoout_dx.h"
 #endif
 
+#ifdef Q_OS_MACX
+#include "videoout_quartz.h"
+#endif
+
+#include "videoout_null.h"
+
 #include "dithertable.h"
 
 #include "../libavcodec/avcodec.h"
 
+#include "filtermanager.h"
+
 VideoOutput *VideoOutput::InitVideoOut(VideoOutputType type)
 {
     (void)type;
@@ -66,8 +74,12 @@ VideoOutput *VideoOutput::InitVideoOut(VideoOutputType type)
     return new VideoOutputXv();
 #endif
 
-    VERBOSE(VB_ALL, "Not compiled with any useable video output method.");
-    exit(-1);
+#ifdef Q_OS_MACX
+    return new VideoOutputQuartz();
+#endif
+
+    VERBOSE(VB_IMPORTANT, "Not compiled with any useable video output method.");
+    exit(-28);
 }
 
 VideoOutput::VideoOutput()
@@ -98,6 +110,14 @@ VideoOutput::VideoOutput()
 
     w_mm = 400;
     h_mm = 300;
+
+    myth_dsw = gContext->GetNumSetting("DisplaySizeWidth", 0);
+    myth_dsh = gContext->GetNumSetting("DisplaySizeHeight", 0);
+
+    m_deinterlacing = false;
+    m_deintFiltMan = NULL;
+    m_deintFilter = NULL;
+    m_deinterlaceBeforeOSD = true;
 }
 
 VideoOutput::~VideoOutput()
@@ -111,6 +131,10 @@ VideoOutput::~VideoOutput()
 
     if (vbuffers)
         delete [] vbuffers;
+    if (m_deintFilter)
+        delete m_deintFilter;
+    if (m_deintFiltMan)
+        delete m_deintFiltMan;
 }
 
 bool VideoOutput::Init(int width, int height, float aspect, WId winid,
@@ -154,6 +178,9 @@ bool VideoOutput::Init(int width, int height, float aspect, WId winid,
     imgx = winx; imgy = winy;
     imgw = XJ_width; imgh = XJ_height;
 
+    if (imgw == 1920 && imgh == 1088)
+        imgh = 1080; // ATSC 1920x1080
+
     brightness = gContext->GetNumSettingOnHost("PlaybackBrightness", 
                                                gContext->GetHostName(), 50);
     contrast = gContext->GetNumSettingOnHost("PlaybackContrast", 
@@ -173,6 +200,82 @@ bool VideoOutput::Init(int width, int height, float aspect, WId winid,
     return true;
 }
 
+bool VideoOutput::SetupDeinterlace(bool interlaced)
+{
+    if (VideoOutputNull *null = dynamic_cast<VideoOutputNull *>(this))
+    {
+        (void)null;
+        // null vidout doesn't deinterlace
+        return false;
+    }
+
+    if (m_deinterlacing == interlaced)
+        return m_deinterlacing;
+
+    m_deinterlacing = interlaced;
+
+    if (m_deinterlacing) 
+    {
+        m_deinterlaceBeforeOSD = true;
+
+        VideoFrameType itmp = FMT_YV12;
+        VideoFrameType otmp = FMT_YV12;
+        int btmp;
+        
+        m_deintfiltername = gContext->GetSetting("DeinterlaceFilter", 
+                                                 "linearblend");
+        m_deintFiltMan = new FilterManager;
+        m_deintFilter = NULL;
+
+        if (ApproveDeintFilter(m_deintfiltername))
+        {
+            m_deintFilter = m_deintFiltMan->LoadFilters(
+                m_deintfiltername, itmp, otmp, XJ_width, XJ_height, btmp);
+        }
+
+        if (m_deintFilter == NULL) 
+        {
+            VERBOSE(VB_IMPORTANT,QString("Couldn't load deinterlace filter %1")
+                    .arg(m_deintfiltername));
+            m_deinterlacing = false;
+            m_deintfiltername = "";
+        }
+
+        VERBOSE(VB_PLAYBACK, QString("Using deinterlace method %1")
+                .arg(m_deintfiltername));
+
+        if (m_deintfiltername == "bobdeint")
+            m_deinterlaceBeforeOSD = false;
+    }
+    else 
+    {
+        if (m_deintFiltMan)
+        {
+            delete m_deintFiltMan;
+            m_deintFiltMan = NULL;
+        }
+        if (m_deintFilter)
+        {
+            delete m_deintFilter;
+            m_deintFilter = NULL;
+        }
+    }
+
+    return m_deinterlacing;
+}
+
+bool VideoOutput::NeedsDoubleFramerate() const
+{
+    // Bob deinterlace requires doubling framerate
+    return (m_deintfiltername == "bobdeint" && m_deinterlacing);
+}
+
+bool VideoOutput::ApproveDeintFilter(const QString& filtername) const
+{
+    // Default to not supporting bob deinterlace
+    return (filtername != "bobdeint");
+}
+
 void VideoOutput::AspectOverride(int override)
 {
     switch(override)
@@ -287,6 +390,8 @@ void VideoOutput::MoveResize(void)
     xoff = img_xoff; yoff = img_yoff;
     dispxoff = dispx; dispyoff = dispy;
     dispwoff = dispw; disphoff = disph;
+    if (imgw == 1920 && imgh == 1088)
+        imgh = 1080; // ATSC 1920x1080
 
 /*
     Here we apply playback over/underscanning and offsetting (if any apply).
@@ -302,8 +407,8 @@ void VideoOutput::MoveResize(void)
     if (img_vscanf > 0) 
     {
         // Veritcal overscan. Move the Y start point in original image.
-        imgy = (int)floor(0.5 + XJ_height * img_vscanf);
-        imgh = (int)floor(0.5 + XJ_height * (1 - 2 * img_vscanf));
+        imgy = (int)floor(0.5 + imgh * img_vscanf);
+        imgh = (int)floor(0.5 + imgh * (1 - 2 * img_vscanf));
 
         // If there is an offset, apply it now that we have a room.
         // To move the image down, move the start point up.
@@ -335,7 +440,7 @@ void VideoOutput::MoveResize(void)
                 xoff = imgx;
             imgx -= xoff;
         }
-        if(xoff < 0) 
+        if (xoff < 0) 
         {
             if (abs(xoff) > imgx) 
                 xoff = 0 - imgx;
@@ -382,7 +487,7 @@ void VideoOutput::MoveResize(void)
             dispxoff += xoff;
         }
 
-        if(xoff < 0) 
+        if (xoff < 0) 
         {
             if (abs(xoff) > dispxoff) 
                 xoff = 0 - dispxoff;
@@ -737,6 +842,8 @@ void VideoOutput::InitBuffers(int numdecode, bool extra_for_pause,
         vbuffers[i].frameNumber = 0;
         vbuffers[i].qscale_table = NULL;
         vbuffers[i].qstride = 0;
+        vbuffers[i].interlaced_frame = -1;
+        vbuffers[i].top_field_first = 1;
     }
 
     numbuffers = numdecode;
