@@ -19,6 +19,7 @@ class NuppelVideoPlayer;
 class OSD;
 class OSDSurface;
 class FilterChain;
+class FilterManager;
 
 extern "C" {
 struct ImgReSampleContext;
@@ -35,10 +36,14 @@ enum VideoOutputType
 
 enum PictureAttribute
 {
-    kPictureAttribute_Brightness = 0,
+    kPictureAttribute_None = 0,
+    kPictureAttribute_MIN = 1,
+    kPictureAttribute_Brightness = 1,
     kPictureAttribute_Contrast,
     kPictureAttribute_Colour,
-    kPictureAttribute_Hue
+    kPictureAttribute_Hue,
+    kPictureAttribute_Volume,
+    kPictureAttribute_MAX
 };
 
 enum PIPLocations
@@ -75,7 +80,8 @@ enum FrameScanType {
     kScan_Ignore      = -1,
     kScan_Progressive =  0,
     kScan_Interlaced  =  1,
-    kScan_Detect      =  2
+    kScan_Intr2ndField=  2,
+    kScan_Detect      =  3
 };
 
 class VideoOutput
@@ -93,7 +99,11 @@ class VideoOutput
                       WId winid, int winx, int winy, int winw, 
                       int winh, WId embedid = 0);
 
-    virtual void PrepareFrame(VideoFrame *buffer) = 0;
+    virtual bool SetupDeinterlace(bool i);
+    virtual bool NeedsDoubleFramerate() const;
+    virtual bool ApproveDeintFilter(const QString& filtername) const;
+
+    virtual void PrepareFrame(VideoFrame *buffer, FrameScanType) = 0;
     virtual void Show(FrameScanType) = 0;
 
     virtual void InputChanged(int width, int height, float aspect);
@@ -190,6 +200,9 @@ class VideoOutput
     int w_mm;
     int h_mm;
 
+    int myth_dsw;
+    int myth_dsh;
+
     int img_xoff, img_yoff;
     float img_hscanf, img_vscanf;
 
@@ -242,6 +255,13 @@ class VideoOutput
     ImgReSampleContext *pipscontext;
 
     bool allowpreviewepg;
+
+    bool m_deinterlacing;
+    QString m_deintfiltername;
+    FilterManager *m_deintFiltMan;
+    FilterChain *m_deintFilter;
+
+    bool m_deinterlaceBeforeOSD;;
 };
 
 #endif
