@@ -7,6 +7,7 @@ class QSqlDatabase;
 #include <qmap.h> 
 #include <list>
 #include <vector>
+#include <qmutex.h>
 #include <qobject.h>
 
 using namespace std;
@@ -35,8 +36,6 @@ class AutoExpire : public QObject
 
     QSqlDatabase *db;
 
-    pthread_mutex_t expirerLock;
-
     bool threadrunning;
     bool isMaster;
 };
