@@ -11,7 +11,7 @@
 #include <unistd.h>
 #include <cstdlib>
 #include <fcntl.h>
-#include <linux/soundcard.h>
+#include <sys/soundcard.h>
 #include <sys/ioctl.h>
 
 #include <list>
@@ -19,10 +19,16 @@
 using namespace std;
 
 #include <sys/stat.h>
+#ifdef linux
 #include <sys/vfs.h>
+#else
+#include <sys/param.h>
+#include <sys/mount.h>
+#endif
 
 #include "libmyth/mythcontext.h"
 #include "libmyth/util.h"
+#include "libmyth/mythdbcon.h"
 
 #include "mainserver.h"
 #include "scheduler.h"
@@ -34,10 +40,11 @@ class ProcessRequestThread : public QThread
   public:
     ProcessRequestThread(MainServer *ms) { parent = ms; }
    
-    void setup(QSocket *sock)
+    void setup(RefSocket *sock)
     {
         lock.lock();
         socket = sock;
+        socket->UpRef();
         lock.unlock();
         waitCond.wakeOne();
     }
@@ -65,6 +72,8 @@ class ProcessRequestThread : public QThread
 
             parent->ProcessRequest(socket);
             parent->MarkUnused(this);
+            socket->DownRef();
+            socket = NULL;
         }
 
         lock.unlock();
@@ -73,7 +82,7 @@ class ProcessRequestThread : public QThread
   private:
     MainServer *parent;
 
-    QSocket *socket;
+    RefSocket *socket;
 
     QMutex lock;
     QWaitCondition waitCond;
@@ -104,10 +113,10 @@ MainServer::MainServer(bool master, int port, int statusport,
     masterBackendOverride = gContext->GetSetting("MasterBackendOverride", 0);
 
     mythserver = new MythServer(port);
-    connect(mythserver, SIGNAL(newConnect(QSocket *)), 
-            SLOT(newConnection(QSocket *)));
-    connect(mythserver, SIGNAL(endConnect(QSocket *)), 
-            SLOT(endConnection(QSocket *)));
+    connect(mythserver, SIGNAL(newConnect(RefSocket *)), 
+            SLOT(newConnection(RefSocket *)));
+    connect(mythserver, SIGNAL(endConnect(RefSocket *)), 
+            SLOT(endConnection(RefSocket *)));
 
     statusserver = new HttpStatus(this, statusport);    
 
@@ -135,14 +144,14 @@ MainServer::~MainServer()
         delete masterServerReconnect;
 }
 
-void MainServer::newConnection(QSocket *socket)
+void MainServer::newConnection(RefSocket *socket)
 {
     connect(socket, SIGNAL(readyRead()), this, SLOT(readSocket()));
 }
 
 void MainServer::readSocket(void)
 {
-    QSocket *socket = (QSocket *)sender();
+    RefSocket *socket = (RefSocket *)sender();
 
     PlaybackSock *testsock = getPlaybackBySock(socket);
     if (testsock && testsock->isExpectingReply())
@@ -173,7 +182,7 @@ void MainServer::readSocket(void)
     readReadyLock.unlock();
 }
 
-void MainServer::ProcessRequest(QSocket *sock)
+void MainServer::ProcessRequest(RefSocket *sock)
 {
     if (sock->bytesAvailable() <= 0)
         return;
@@ -281,6 +290,10 @@ void MainServer::ProcessRequest(QSocket *sock)
     {
         HandleGetFreeRecorder(pbs);
     }
+    else if (command == "GET_FREE_RECORDER_COUNT")
+    {
+        HandleGetFreeRecorderCount(pbs);
+    }
     else if (command == "GET_NEXT_FREE_RECORDER")
     {
         HandleGetNextFreeRecorder(listline, pbs);
@@ -379,7 +392,16 @@ void MainServer::ProcessRequest(QSocket *sock)
         HandleBackendRefresh(sock);
     }
     else
+    {
         VERBOSE(VB_ALL, "Unknown command: " + command);
+
+        QSocket *pbssock = pbs->getSocket();
+
+        QStringList strlist;
+        strlist << "UNKNOWN_COMMAND";
+        
+        SendResponse(pbssock, strlist);
+    }
 }
 
 void MainServer::MarkUnused(ProcessRequestThread *prt)
@@ -432,13 +454,45 @@ void MainServer::customEvent(QCustomEvent *e)
 
     if (sendstuff)
     {
+        bool sendGlobal = false;
+        if (ismaster && broadcast[1].left(7) == "GLOBAL_")
+        {
+            broadcast[1].replace(QRegExp("GLOBAL_"), "LOCAL_");
+            MythEvent me(broadcast[1], broadcast[2]);
+            gContext->dispatch(me);
+
+            sendGlobal = true;
+        }
+
+        QPtrList<PlaybackSock> sentSet;
+
         vector<PlaybackSock *>::iterator iter = playbackList.begin();
         for (; iter != playbackList.end(); iter++)
         {
             PlaybackSock *pbs = (*iter);
-            if (pbs->wantsEvents())
+
+            if (sentSet.containsRef(pbs))
+                continue;
+
+            sentSet.append(pbs);
+
+            RefSocket *sock = pbs->getSocket();
+            sock->UpRef();
+
+            if (sendGlobal)
             {
-                WriteStringList(pbs->getSocket(), broadcast);
+                if (pbs->isSlaveBackend())
+                    WriteStringList(sock, broadcast);
+            }
+            else if (pbs->wantsEvents())
+            {
+                WriteStringList(sock, broadcast);
+            }
+
+            if (sock->DownRef())
+            {
+                // was deleted elsewhere, so the iterator's invalid.
+                iter = playbackList.begin();
             }
         }
     }
@@ -463,7 +517,7 @@ void MainServer::HandleVersion(QSocket *socket,QString version)
 }
 
 void MainServer::HandleAnnounce(QStringList &slist, QStringList commands, 
-                                QSocket *socket)
+                                RefSocket *socket)
 {
     QStringList retlist = "OK";
 
@@ -496,6 +550,11 @@ void MainServer::HandleAnnounce(QStringList &slist, QStringList commands,
         ScheduledRecording::signalChange(m_db);
         dblock.unlock();
 
+        QString message = QString("LOCAL_SLAVE_BACKEND_ONLINE %2")
+                                  .arg(commands[2]);
+        MythEvent me(message);
+        gContext->dispatch(me);
+
         playbackList.push_back(pbs);
     }
     else if (commands[1] == "RingBuffer")
@@ -509,7 +568,7 @@ void MainServer::HandleAnnounce(QStringList &slist, QStringList commands,
         if (iter == encoderList->end())
         {
             VERBOSE(VB_ALL, "Unknown encoder.");
-            exit(1);
+            exit(4);
         }
 
         EncoderLink *enc = iter.data();
@@ -620,6 +679,10 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
     bool islocal = pbs->isLocal();
     QString playbackhost = pbs->getHostname();
 
+    QString fs_db_name = "";
+    MythSqlDatabase *updatefs_mdb = NULL;
+    QSqlDatabase *updatefs_db = NULL;
+
     QString thequery;
 
     QString ip = gContext->GetSetting("BackendServerIP");
@@ -634,7 +697,8 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
                "recorded.autoexpire,editing,bookmark,recorded.category,"
                "recorded.recgroup,record.dupin,record.dupmethod,"
                "record.recordid,outputfilters,"
-               "recorded.seriesid,recorded.programid "
+               "recorded.seriesid,recorded.programid,recorded.filesize, "
+               "recorded.lastmodified "
                "FROM recorded "
                "LEFT JOIN record ON recorded.recordid = record.recordid "
                "LEFT JOIN channel ON recorded.chanid = channel.chanid "
@@ -644,12 +708,12 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
         thequery += " DESC";
     thequery += ";";
 
-    
-    
     QSqlQuery query = m_db->exec(thequery);
 
     QStringList outputlist;
     QString fileprefix = gContext->GetFilePrefix();
+    QMap<QString, QString> backendIpMap;
+    QMap<QString, QString> backendPortMap;
 
     if (query.isActive() && query.numRowsAffected() > 0)
     {
@@ -677,6 +741,10 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
             proginfo->chanOutputFilters = query.value(20).toString();
             proginfo->seriesid = query.value(21).toString();
             proginfo->programid = query.value(22).toString();
+            proginfo->filesize = stringToLongLong(query.value(23).toString());
+            proginfo->lastmodified =
+                      QDateTime::fromString(query.value(24).toString(),
+                                            Qt::ISODate);
 
             if (proginfo->hostname.isEmpty() || proginfo->hostname.isNull())
                 proginfo->hostname = gContext->GetHostName();
@@ -728,13 +796,25 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
                     proginfo->pathname = QString("myth://") + ip + ":" + port
                                          + "/" + proginfo->GetRecordBasename();
 
-                struct stat st;
+                if (proginfo->filesize == 0)
+                {
+                    struct stat st;
 
-                long long size = 0;
-                if (stat(lpath.ascii(), &st) == 0)
-                    size = st.st_size;
+                    long long size = 0;
+                    if (stat(lpath.ascii(), &st) == 0)
+                        size = st.st_size;
 
-                proginfo->filesize = size;
+                    proginfo->filesize = size;
+
+                    if (!updatefs_db)
+                    {
+                        fs_db_name = QString("updatefilesize%1%2")
+                                             .arg(getpid()).arg(rand());
+                        updatefs_mdb = new MythSqlDatabase(fs_db_name);
+                        updatefs_db = updatefs_mdb->db();
+                    }
+                    proginfo->SetFilesize(size, updatefs_db);
+                }
             }
             else
             {
@@ -748,7 +828,35 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
                 }
                 else
                 {
-                    slave->FillProgramInfo(proginfo, playbackhost);
+                    if (proginfo->filesize == 0)
+                    {
+                        slave->FillProgramInfo(proginfo, playbackhost);
+
+                        if (!updatefs_db)
+                        {
+                            fs_db_name = QString("updatefilesize%1%2")
+                                                 .arg(getpid()).arg(rand());
+                            updatefs_mdb = new MythSqlDatabase(fs_db_name);
+                            updatefs_db = updatefs_mdb->db();
+                        }
+                        proginfo->SetFilesize(proginfo->filesize, updatefs_db);
+                    }
+                    else
+                    {
+                        ProgramInfo *p = proginfo;
+                        if (!backendIpMap.contains(p->hostname))
+                            backendIpMap[p->hostname] =
+                                gContext->GetSettingOnHost("BackendServerIp",
+                                                           p->hostname);
+                        if (!backendPortMap.contains(p->hostname))
+                            backendPortMap[p->hostname] =
+                                gContext->GetSettingOnHost("BackendServerPort",
+                                                           p->hostname);
+                        p->pathname = QString("myth://") +
+                                      backendIpMap[p->hostname] + ":" +
+                                      backendPortMap[p->hostname] + "/" +
+                                      p->GetRecordBasename();
+                    }
                 }
             }
 
@@ -762,6 +870,9 @@ void MainServer::HandleQueryRecordings(QString type, PlaybackSock *pbs)
 
     dblock.unlock();
 
+    if (updatefs_mdb)
+        delete updatefs_mdb;
+
     SendResponse(pbssock, outputlist);
 }
 
@@ -864,6 +975,53 @@ static void *SpawnDelete(void *param)
     return NULL;
 }
 
+struct DeleteRecordedMarkupStruct
+{
+    MainServer *ms;
+    QString chanid;
+    QString startts;
+};
+
+void *MainServer::SpawnDeleteRecordedMarkup(void *param)
+{
+    DeleteRecordedMarkupStruct *ds = (DeleteRecordedMarkupStruct *)param;
+    MainServer *ms = ds->ms;
+    ms->DoSpawnDeleteRecordedMarkup(ds->chanid, ds->startts);
+
+    delete ds;
+
+    return NULL;
+}
+
+void MainServer::DoSpawnDeleteRecordedMarkup(QString chanid, QString startts)
+{
+    QString name = QString("recmarkup%1%2").arg(getpid()).arg(rand());
+
+    MythSqlDatabase *recmarkup_db = new MythSqlDatabase(name);
+
+    QString thequery = QString("DELETE FROM recordedmarkup WHERE chanid = %1 "
+                               "AND starttime = %2;")
+                              .arg(chanid).arg(startts);
+
+    if (!recmarkup_db || !recmarkup_db->isOpen())
+    {
+        QString msg = QString("ERROR deleting recordedmarkup for chanid "
+                              "%1 recorded at %2, unable to open DB "
+                              "connection.")
+                              .arg(chanid).arg(startts);
+        VERBOSE(VB_GENERAL, msg);
+        return;
+    }
+
+    QSqlDatabase *db = recmarkup_db->db();
+
+    QSqlQuery query = db->exec(thequery);
+    if (!query.isActive())
+        MythContext::DBError("Recorded program deletion (2)", query);
+
+    delete recmarkup_db;
+}
+
 void MainServer::HandleCheckRecordingActive(QStringList &slist, 
                                             PlaybackSock *pbs)
 {
@@ -1060,6 +1218,13 @@ void MainServer::DoHandleStopRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
         SendResponse(pbssock, outputlist);
     }
 
+    QString message = QString("GLOBAL_COMMFLAG START %1 %2 %3")
+                              .arg(pginfo->chanid)
+                              .arg(pginfo->startts.toString(Qt::ISODate))
+                              .arg(gContext->GetHostName());
+    MythEvent me(message);
+    gContext->dispatch(me);
+
     delete pginfo;
 }
 
@@ -1143,6 +1308,22 @@ void MainServer::DoHandleDeleteRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
 
     MythContext::KickDatabase(m_db);
 
+    if (pginfo->IsCommProcessing(m_db))
+    {
+        QString message = QString("GLOBAL_COMMFLAG STOP %1 %2 master")
+                                 .arg(pginfo->chanid)
+                                 .arg(pginfo->recstartts.toString(Qt::ISODate));
+        MythEvent me(message);
+        gContext->dispatch(me);
+
+        int loop = 0;
+        while ((pginfo->IsCommProcessing(m_db)) && loop < 20)
+        {
+            usleep(100000);
+            loop++;
+        }
+    }
+
     QSqlQuery query(QString::null, m_db);
     query.prepare("DELETE FROM recorded WHERE chanid = :CHANID AND "
                   "title = :TITLE AND starttime = :STARTTIME AND "
@@ -1157,17 +1338,21 @@ void MainServer::DoHandleDeleteRecording(ProgramInfo *pginfo, PlaybackSock *pbs)
     if (!query.isActive())
         MythContext::DBError("Recorded program deletion", query);
 
-    // now delete any markups for this program
-    QString thequery = QString("DELETE FROM recordedmarkup WHERE chanid = %1 "
-                               "AND starttime = %2;")
-                              .arg(pginfo->chanid).arg(startts);
-
-    query = m_db->exec(thequery);
-    if (!query.isActive())
-        MythContext::DBError("Recorded program deletion (2)", query);
-
     dblock.unlock();
 
+    DeleteRecordedMarkupStruct *ds = new DeleteRecordedMarkupStruct;
+    ds->chanid = pginfo->chanid;
+    ds->startts = startts;
+    ds->ms = this;
+
+    pthread_t deleterecordedmarkupthread;
+    pthread_attr_t attr;
+    pthread_attr_init(&attr);
+    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+
+    pthread_create(&deleterecordedmarkupthread, &attr,
+                   SpawnDeleteRecordedMarkup, ds);
+
     QString fileprefix = gContext->GetFilePrefix();
     QString filename = pginfo->GetRecordFilename(fileprefix);
     QFile checkFile(filename);
@@ -1519,33 +1704,42 @@ void MainServer::HandleGetFreeRecorder(PlaybackSock *pbs)
     EncoderLink *encoder = NULL;
     QString enchost;
 
+    bool lastcard = false;
+
+    if (gContext->GetSetting("LastFreeCard", "0") == "1")
+        lastcard = true;
+
     QMap<int, EncoderLink *>::Iterator iter = encoderList->begin();
     for (; iter != encoderList->end(); ++iter)
     {
         EncoderLink *elink = iter.data();
 
-        if (elink->isLocal())
-            enchost = gContext->GetHostName();
-        else
-            enchost = elink->getHostname();
+        if (!lastcard)
+        {
+            if (elink->isLocal())
+                enchost = gContext->GetHostName();
+            else
+                enchost = elink->getHostname();
 
-        if ((enchost == pbshost) &&
-            (elink->isConnected()) &&
-            (!elink->IsBusy()) &&
-            (!elink->isTunerLocked()))
-        {
-            encoder = elink;
-            retval = iter.key();
-            break;
+            if (enchost == pbshost && elink->isConnected() &&
+                !elink->IsBusy() && !elink->isTunerLocked())
+            {
+                encoder = elink;
+                retval = iter.key();
+                VERBOSE(VB_RECORD, QString("Card %1 is local.")
+                        .arg(iter.key()));
+                break;
+            }
         }
-        if ((retval == -1) &&
-            (elink->isConnected()) &&
-            (!elink->IsBusy()) &&
-            (!elink->isTunerLocked()))
+
+        if ((retval == -1 || lastcard) && elink->isConnected() &&
+            !elink->IsBusy() && !elink->isTunerLocked())
         {
             encoder = elink;
             retval = iter.key();
         }
+        VERBOSE(VB_RECORD, QString("Checking card %1. Best card so far %2")
+                .arg(iter.key()).arg(retval));
     }
 
     strlist << QString::number(retval);
@@ -1575,6 +1769,31 @@ void MainServer::HandleGetFreeRecorder(PlaybackSock *pbs)
     SendResponse(pbssock, strlist);
 }
 
+void MainServer::HandleGetFreeRecorderCount(PlaybackSock *pbs)
+{
+    QSocket *pbssock = pbs->getSocket();
+
+    QStringList strlist;
+    int count = 0;
+
+    QMap<int, EncoderLink *>::Iterator iter = encoderList->begin();
+    for (; iter != encoderList->end(); ++iter)
+    {
+        EncoderLink *elink = iter.data();
+
+        if ((elink->isConnected()) &&
+            (!elink->IsBusy()) &&
+            (!elink->isTunerLocked()))
+        {
+            count++;
+        }
+    }
+
+    strlist << QString::number(count);
+        
+    SendResponse(pbssock, strlist);
+}
+
 void MainServer::HandleGetNextFreeRecorder(QStringList &slist, 
                                            PlaybackSock *pbs)
 {
@@ -1588,12 +1807,13 @@ void MainServer::HandleGetNextFreeRecorder(QStringList &slist,
     EncoderLink *encoder = NULL;
     QString enchost;
 
-    VERBOSE(VB_ALL, QString("Getting next free recorder : %1").arg(currrec));
+    VERBOSE(VB_RECORD, QString("Getting next free recorder after : %1")
+            .arg(currrec));
 
     // find current recorder
     QMap<int, EncoderLink *>::Iterator iter, curr = encoderList->find(currrec);
 
-    if (curr != encoderList->end())
+    if (currrec > 0 && curr != encoderList->end())
     {
         // cycle through all recorders
         for (iter = curr;;)
@@ -1608,20 +1828,6 @@ void MainServer::HandleGetNextFreeRecorder(QStringList &slist,
 
             elink = iter.data();
 
-            if (elink->isLocal())
-                enchost = gContext->GetHostName();
-            else
-                enchost = elink->getHostname();
-
-            if ((enchost == pbshost) &&
-                (elink->isConnected()) &&
-                (!elink->IsBusy()) &&
-                (!elink->isTunerLocked()))
-            {
-                encoder = elink;
-                retval = iter.key(); 
-                break;
-            }
             if ((retval == -1) &&
                 (elink->isConnected()) &&
                 (!elink->IsBusy()) &&
@@ -1738,6 +1944,9 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
                 encodeLongLong(retlist, value);
             }
         }
+
+        if (!retlist.size())
+            retlist << "ok";
     }
     else if (command == "SETUP_RING_BUFFER")
     {
@@ -1920,10 +2129,13 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
         QString title = "", subtitle = "", desc = "", category = "";
         QString starttime = "", endtime = "", callsign = "", iconpath = "";
         QString channelname = "", chanid = "", seriesid = "", programid = "";
+        QString chanOutputFilters = "";
+        QString repeat = "", airdate = "", stars = "";
 
         enc->GetChannelInfo(title, subtitle, desc, category, starttime,
                             endtime, callsign, iconpath, channelname, chanid,
-                            seriesid, programid);
+                            seriesid, programid, chanOutputFilters,
+                            repeat, airdate, stars);
 
         if (title == "")
             title = " ";
@@ -1949,6 +2161,14 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
             seriesid = " ";
         if (programid == "")
             programid = " ";
+        if (chanOutputFilters == "")
+            chanOutputFilters = " ";    
+        if (repeat = "" )
+            repeat = "0";
+        if (airdate = "")
+            airdate = starttime;
+        if (stars = "")
+            stars = " ";
 
         retlist << title;
         retlist << subtitle;
@@ -1962,6 +2182,10 @@ void MainServer::HandleRecorderQuery(QStringList &slist, QStringList &commands,
         retlist << chanid;
         retlist << seriesid;
         retlist << programid;
+        retlist << chanOutputFilters;
+        retlist << repeat;
+        retlist << airdate;
+        retlist << stars;
     }
     else if (command == "GET_INPUT_NAME")
     {
@@ -2362,7 +2586,7 @@ void MainServer::HandleBackendRefresh(QSocket *socket)
     SendResponse(socket, retlist);    
 }
 
-void MainServer::endConnection(QSocket *socket)
+void MainServer::endConnection(RefSocket *socket)
 {
     vector<PlaybackSock *>::iterator it = playbackList.begin();
     for (; it != playbackList.end(); ++it)
@@ -2385,8 +2609,13 @@ void MainServer::endConnection(QSocket *socket)
                 dblock.lock();
                 ScheduledRecording::signalChange(m_db);
                 dblock.unlock();
+
+                QString message = QString("LOCAL_SLAVE_BACKEND_OFFLINE %2")
+                                          .arg((*it)->getHostname());
+                MythEvent me(message);
+                gContext->dispatch(me);
             }
-            delete (*it);
+            socket->DownRef();
             playbackList.erase(it);
             return;
         }
@@ -2398,7 +2627,7 @@ void MainServer::endConnection(QSocket *socket)
         QSocket *sock = (*ft)->getSocket();
         if (sock == socket)
         {
-            delete (*ft);
+            socket->DownRef();
             fileTransferList.erase(ft);
             return;
         }
@@ -2433,7 +2662,7 @@ void MainServer::endConnection(QSocket *socket)
                 }
             }
 
-            delete (*rt);
+            socket->DownRef();
             ringBufList.erase(rt);
             return;
         }
@@ -2570,7 +2799,7 @@ void MainServer::masterServerDied(void)
 
 void MainServer::reconnectTimeout(void)
 {
-    QSocket *masterServerSock = new QSocket();
+    RefSocket *masterServerSock = new RefSocket();
 
     QString server = gContext->GetSetting("MasterServerIP", "localhost");
     int port = gContext->GetNumSetting("MasterServerPort", 6543);
@@ -2894,6 +3123,7 @@ void MainServer::PrintStatus(QSocket *socket)
                 QString title, callsign, dummy;
                 elink->GetChannelInfo(title, dummy, dummy, dummy, dummy,
                                       dummy, callsign, dummy, dummy,
+                                      dummy, dummy, dummy, dummy,
                                       dummy, dummy, dummy);
                 os << ": '" << title << "' on "  << callsign;
             }
@@ -2934,9 +3164,26 @@ void MainServer::PrintStatus(QSocket *socket)
     if (m_sched)
         m_sched->getAllPending(&recordingList);
 
+    list<ProgramInfo *>::iterator iterCnt = recordingList.begin();
+
     unsigned int iNum = 10;
-    if (recordingList.size() < iNum) 
-        iNum = recordingList.size();
+
+    // count the number of upcoming recordings
+    unsigned int iNumRecordings = 0;
+    while (iterCnt != recordingList.end())
+    {
+        if (!((*iterCnt)->recstatus > rsWillRecord ||     
+               ((*iterCnt)->recstartts) < QDateTime::currentDateTime())) 
+        {                       
+               iNumRecordings++;
+        }
+
+        if (iNumRecordings > iNum) break;
+        iterCnt++;
+    }
+
+    if (iNumRecordings < iNum) 
+        iNum = iNumRecordings;
 
     if (iNum == 0)
         os << "    There are no shows scheduled for recording.\r\n";
@@ -2947,7 +3194,9 @@ void MainServer::PrintStatus(QSocket *socket)
           << " scheduled for recording:\r\n";
 
        os << "    <div id=\"schedule\">\r\n";
+
        list<ProgramInfo *>::iterator iter = recordingList.begin();
+
        for (unsigned int i = 0; (iter != recordingList.end()) && i < iNum; 
             iter++, i++)
        {
