@@ -36,7 +36,7 @@ class MainServer : public QObject
     bool isClientConnected();
     void ShutSlaveBackendsDown(QString &haltcmd);
 
-    void ProcessRequest(QSocket *sock); 
+    void ProcessRequest(RefSocket *sock); 
     void MarkUnused(ProcessRequestThread *prt);
 
   protected slots:
@@ -44,13 +44,13 @@ class MainServer : public QObject
     void masterServerDied(void);
 
   private slots:
-    void newConnection(QSocket *);
-    void endConnection(QSocket *);
+    void newConnection(RefSocket *);
+    void endConnection(RefSocket *);
     void readSocket();
 
   private:
     void HandleAnnounce(QStringList &slist, QStringList commands, 
-                        QSocket *socket);
+                        RefSocket *socket);
     void HandleDone(QSocket *socket);
 
     void HandleIsActiveBackendQuery(QStringList &slist, PlaybackSock *pbs);
@@ -68,6 +68,7 @@ class MainServer : public QObject
     void HandleGetConflictingRecordings(QStringList &slist, PlaybackSock *pbs);
     void HandleGetNextFreeRecorder(QStringList &slist, PlaybackSock *pbs);
     void HandleGetFreeRecorder(PlaybackSock *pbs);
+    void HandleGetFreeRecorderCount(PlaybackSock *pbs);
     void HandleRecorderQuery(QStringList &slist, QStringList &commands,
                              PlaybackSock *pbs);
     void HandleFileTransferQuery(QStringList &slist, QStringList &commands,
@@ -103,6 +104,9 @@ class MainServer : public QObject
 
     bool isRingBufSock(QSocket *sock);
 
+    static void *SpawnDeleteRecordedMarkup(void *param);
+    void DoSpawnDeleteRecordedMarkup(QString chanid, QString startts);
+
     QMap<int, EncoderLink *> *encoderList;
 
     MythServer *mythserver;
