@@ -16,11 +16,11 @@
 using namespace std;
 
 #include "channelrecpriority.h"
-#include "infodialog.h"
 #include "tv.h"
 
 #include "dialogbox.h"
 #include "mythcontext.h"
+#include "scheduledrecording.h"
 #include "infostructs.h"
 
 ChannelRecPriority::ChannelRecPriority(QSqlDatabase *ldb, 
@@ -70,7 +70,7 @@ ChannelRecPriority::ChannelRecPriority(QSqlDatabase *ldb,
     else
     {
         cerr << "MythFrontEnd: ChannelRecPriority - Failed to get selector.\n";
-        exit(0);
+        exit(19);
     }
 
     bgTransBackup = gContext->LoadScalePixmap("trans-backup.png");
@@ -189,7 +189,7 @@ void ChannelRecPriority::LoadWindow(QDomElement &element)
             else
             {
                 cerr << "Unknown element: " << e.tagName() << endl;
-                exit(0);
+                exit(20);
             }
         }
     }
@@ -359,7 +359,7 @@ void ChannelRecPriority::changeRecPriority(int howMuch)
 
     // inc/dec recording priority
     tempRecPriority = chanInfo->recpriority.toInt() + howMuch;
-    if(tempRecPriority > -100 && tempRecPriority < 100)
+    if (tempRecPriority > -100 && tempRecPriority < 100)
     {
         chanInfo->recpriority = QString::number(tempRecPriority);
 
@@ -419,10 +419,10 @@ void ChannelRecPriority::FillList(void)
             ChannelInfo *chaninfo = new ChannelInfo;
             chaninfo->chanid = result.value(0).toInt();
             chaninfo->chanstr = result.value(1).toString();
-            chaninfo->callsign = result.value(2).toString();
+            chaninfo->callsign = QString::fromUtf8(result.value(2).toString());
             chaninfo->iconpath = result.value(3).toString();
             chaninfo->recpriority = result.value(4).toString();
-            chaninfo->channame = result.value(5).toString();
+            chaninfo->channame = QString::fromUtf8(result.value(5).toString());
             channelData[QString::number(cnt)] = *chaninfo;
 
             // save recording priority value in map so we don't have to save 
@@ -642,7 +642,7 @@ void ChannelRecPriority::updateInfo(QPainter *p)
             itype = (UIImageType *)container->GetType("icon");
             if (itype) {
                 int iconsize = itype->GetSize();
-                if(curitem->iconpath == "none" || curitem->iconpath == "")
+                if (curitem->iconpath == "none" || curitem->iconpath == "")
                     curitem->iconpath = "blankicon.jpg";
                 if (!curitem->icon)
                     curitem->LoadIcon(iconsize);
@@ -656,7 +656,7 @@ void ChannelRecPriority::updateInfo(QPainter *p)
  
             type = (UITextType *)container->GetType("recpriority");
             if (type) {
-                if(curitem->recpriority.toInt() > 0)
+                if (curitem->recpriority.toInt() > 0)
                     type->SetText("+"+curitem->recpriority);
                 else
                     type->SetText(curitem->recpriority);
