@@ -34,6 +34,7 @@ using namespace std;
 #include "xbox.h"
 #include "dbcheck.h"
 #include "mythmediamonitor.h"
+#include "statusbox.h"
 
 #define QUIT     1
 #define HALT     2
@@ -67,14 +68,25 @@ void startSearchTitle(void)
     qApp->lock();
 }
 
-void startSearchDesc(void)
+void startSearchKeyword(void)
 {
-  ProgLister searchDesc(plDescSearch, "",
+  ProgLister searchKeyword(plKeywordSearch, "",
                         QSqlDatabase::database(),
                         gContext->GetMainWindow(), "proglist");
 
     qApp->unlock();
-    searchDesc.exec();
+    searchKeyword.exec();
+    qApp->lock();
+}
+
+void startSearchPeople(void)
+{
+  ProgLister searchPeople(plPeopleSearch, "",
+                         QSqlDatabase::database(),
+                         gContext->GetMainWindow(), "proglist");
+
+    qApp->unlock();
+    searchPeople.exec();
     qApp->lock();
 }
 
@@ -122,6 +134,17 @@ void startSearchNew(void)
     qApp->lock();
 }
 
+void startSearchTime(void)
+{
+  ProgLister searchTime(plTime, "",
+                         QSqlDatabase::database(),
+                         gContext->GetMainWindow(), "proglist");
+
+    qApp->unlock();
+    searchTime.exec();
+    qApp->lock();
+}
+
 void startManaged(void)
 {
     QSqlDatabase *db = QSqlDatabase::database();
@@ -282,91 +305,10 @@ void startTV(void)
 
 void showStatus(void)
 {
-    QString mfdLastRunStart, mfdLastRunEnd, mfdLastRunStatus, Status;
-    QString querytext, DataDirectMessage;
-    int DaysOfData;
-    QDateTime qdtNow, GuideDataThrough;
-    QSqlDatabase *db = QSqlDatabase::database();
-
-    qdtNow = QDateTime::currentDateTime();
-    querytext = QString("SELECT max(endtime) FROM program;");
-
-    QSqlQuery query = db->exec(querytext);
-
-    if (query.isActive() && query.numRowsAffected())
-    {
-        query.next();
-        GuideDataThrough = QDateTime::fromString(query.value(0).toString(),
-                                                 Qt::ISODate);
-    }
-
-    mfdLastRunStart = gContext->GetSetting("mythfilldatabaseLastRunStart");
-    mfdLastRunEnd = gContext->GetSetting("mythfilldatabaseLastRunEnd");
-    mfdLastRunStatus = gContext->GetSetting("mythfilldatabaseLastRunStatus");
-    DataDirectMessage = gContext->GetSetting("DataDirectMessage");
-
-    Status = QObject::tr("Myth version:") + " " + MYTH_BINARY_VERSION + "\n";
-
-    Status += QObject::tr("Last mythfilldatabase guide update:");
-    Status += "\n   ";
-    Status += QObject::tr("Started:   ");
-    Status += mfdLastRunStart;
-    if (mfdLastRunEnd > mfdLastRunStart)  //if end < start, it's still running.
-    {
-        Status += "\n   ";
-        Status += QObject::tr("Finished: ");
-        Status += mfdLastRunEnd;
-    }
-
-    Status += "\n   ";
-    Status += QObject::tr("Result: ");
-    Status += mfdLastRunStatus;
-
-    DaysOfData = qdtNow.daysTo(GuideDataThrough);
-
-    if (GuideDataThrough.isNull())
-    {
-        Status += QObject::tr("There's no guide data available! ");
-        Status += QObject::tr("Have you run mythfilldatabase?");
-        Status += "\n";
-    }
-    else
-    {
-        Status += "\n\n";
-        Status += QObject::tr("There is guide data until ");
-        Status += QDateTime(GuideDataThrough).toString("yyyy-MM-dd hh:mm");
-
-        if (DaysOfData > 0)
-        {
-            Status += QString("\n(%1 ").arg(DaysOfData);
-            if (DaysOfData >1)
-                Status += QObject::tr("days");
-            else
-                Status += QObject::tr("day");
-            Status += ").";
-        }
-        else
-            Status += ".";
-    }
-
-    if (DaysOfData <= 3)
-    {
-        Status += "\n";
-        Status += QObject::tr("WARNING: is mythfilldatabase running?");
-    }
-
-    if (!DataDirectMessage.isNull())
-    {
-        Status += "\n";
-        Status += QObject::tr("DataDirect Status: ");
-        Status += DataDirectMessage;
-    }
-
-    DialogBox *status_dialog = new DialogBox(gContext->GetMainWindow(), Status);
-    status_dialog->AddButton(QObject::tr("OK"));
-    status_dialog->exec();
-
-    delete status_dialog;
+    StatusBox statusbox(gContext->GetMainWindow(), "status box");
+    qApp->unlock();
+    statusbox.exec();
+    qApp->lock();
 }
 
 void TVMenuCallback(void *data, QString &selection)
@@ -394,8 +336,10 @@ void TVMenuCallback(void *data, QString &selection)
         startFinder();
     else if (sel == "tv_search_title")
         startSearchTitle();
-    else if (sel == "tv_search_desc")
-        startSearchDesc();
+    else if (sel == "tv_search_keyword")
+        startSearchKeyword();
+    else if (sel == "tv_search_people")
+        startSearchPeople();
     else if (sel == "tv_search_channel")
         startSearchChannel();
     else if (sel == "tv_search_category")
@@ -404,6 +348,8 @@ void TVMenuCallback(void *data, QString &selection)
         startSearchMovie();
     else if (sel == "tv_search_new")
         startSearchNew();
+    else if (sel == "tv_search_time")
+        startSearchTime();
     else if (sel == "settings appearance") 
     {
         AppearanceSettings settings;
@@ -471,7 +417,7 @@ void TVMenuCallback(void *data, QString &selection)
 
 int handleExit(void)
 {
-    if (gContext->GetNumSetting("NoPromptOnExit") == 1)
+    if (gContext->GetNumSetting("NoPromptOnExit", 1) == 0)
         return QUIT;
 
     // first of all find out, if a backend runs on this host...
@@ -480,7 +426,7 @@ int handleExit(void)
     QStringList strlist = "";
     strlist << "QUERY_IS_ACTIVE_BACKEND";
     strlist << gContext->GetHostName();
-                
+
     gContext->SendReceiveStringList(strlist);
 
     if (QString(strlist[0]) == "FALSE")
@@ -601,31 +547,51 @@ QString RandTheme(QString &themename, QSqlDatabase *db)
     return themename;
 }
 
-int internal_play_media(const char *mrl) 
+int internal_play_media(const char *mrl, const char* plot, const char* title, 
+                        const char* director, int lenMins, const char* year) 
 {
-    int res = -1;
+    int res = -1; 
+    
+   
     TV *tv = new TV();
 
     tv->Init();
+
     ProgramInfo *pginfo = new ProgramInfo();
-    pginfo->endts = QDateTime::currentDateTime().addSecs(-180);
+    pginfo->recstartts = QDateTime::currentDateTime().addSecs((0 - (lenMins + 1)) * 60 );
+    pginfo->recendts = QDateTime::currentDateTime().addSecs(-60);
+    pginfo->recstartts.setDate(QDate::fromString(year));
+    pginfo->lenMins = lenMins;
+    pginfo->isVideo = true;
     pginfo->pathname = mrl;
+    pginfo->description = plot;
+    
+    
+    if (strlen(director))
+        pginfo->subtitle = QString( "%1: %2" ).arg(QObject::tr("Directed By")).arg(director);
+    
+    pginfo->title = title;
 
-    if (tv->Playback(pginfo)) 
+    tv->Playback(pginfo);
+    
+    if (tv->Playback(pginfo))
     {
-       while (tv->GetState() != kState_None) 
-       {
-           qApp->unlock();
-           qApp->processEvents();
-           usleep(50);
-           qApp->lock();
-       }
-       res = 0;
+        while (tv->GetState() != kState_None)
+        {
+            qApp->unlock();
+            qApp->processEvents();
+            usleep(10000);
+            qApp->lock();
+        }
     }
+    
 
+    res = 0;
+    
+    sleep(1);
     delete tv;
     delete pginfo;
-
+    
     return res;
 }
 
@@ -651,6 +617,53 @@ int internal_media_init()
                     internal_play_media);
     return 0;
 }
+
+static void *run_priv_thread(void *data)
+{
+    (void)data;
+    while (true) 
+    {
+        if (gContext->hasPrivRequest()) 
+        {
+            MythPrivRequest req = gContext->popPrivRequest();
+            switch (req.getType()) 
+            {
+            case MythPrivRequest::MythRealtime:
+                {
+                    pthread_t *target_thread = (pthread_t *)(req.getData());
+                    // Raise the given thread to realtime priority
+                    struct sched_param sp = {1};
+                    if (target_thread)
+                    {
+                        int status = pthread_setschedparam(
+                            *target_thread, SCHED_FIFO, &sp);
+                        if (status) {
+                            perror("pthread_setschedparam");
+                            VERBOSE(VB_GENERAL, "Running as SUID root would allow "
+                                    "some threads to run with realtime priority, "
+                                    "improving video smoothness.");
+                        }
+                        else
+                            VERBOSE(VB_GENERAL, "Using realtime priority.");
+                    }
+                    else
+                    {
+                        VERBOSE(VB_IMPORTANT, "Something sus here. NULL thread ptr "
+                                "for MythPrivRequest::MythRealtime");
+                    }
+                }
+                break;
+            case MythPrivRequest::MythExit:
+                pthread_exit(NULL);
+                break;
+            case MythPrivRequest::PrivEnd:
+                break;
+            }
+        }
+        sleep(1);
+    }
+    return NULL; // will never happen
+}
    
 int main(int argc, char **argv)
 {
@@ -708,66 +721,71 @@ int main(int argc, char **argv)
                     for (QStringList::Iterator it = verboseOpts.begin(); 
                          it != verboseOpts.end(); ++it )
                     {
-                        if(!strcmp(*it,"none"))
+                        if (!strcmp(*it,"none"))
                         {
                             print_verbose_messages = VB_NONE;
                             verboseString = "";
                         }
-                        else if(!strcmp(*it,"all"))
+                        else if (!strcmp(*it,"all"))
                         {
                             print_verbose_messages = VB_ALL;
                             verboseString = "all";
                         }
-                        else if(!strcmp(*it,"quiet"))
+                        else if (!strcmp(*it,"quiet"))
                         {
                             print_verbose_messages = VB_IMPORTANT;
                             verboseString = "important";
                         }
-                        else if(!strcmp(*it,"record"))
+                        else if (!strcmp(*it,"record"))
                         {
                             print_verbose_messages |= VB_RECORD;
                             verboseString += " " + *it;
                         }
-                        else if(!strcmp(*it,"playback"))
+                        else if (!strcmp(*it,"playback"))
                         {
                             print_verbose_messages |= VB_PLAYBACK;
                             verboseString += " " + *it;
                         }
-                        else if(!strcmp(*it,"channel"))
+                        else if (!strcmp(*it,"channel"))
                         {
                             print_verbose_messages |= VB_CHANNEL;
                             verboseString += " " + *it;
                         }
-                        else if(!strcmp(*it,"osd"))
+                        else if (!strcmp(*it,"osd"))
                         {
                             print_verbose_messages |= VB_OSD;
                             verboseString += " " + *it;
                         }
-                        else if(!strcmp(*it,"file"))
+                        else if (!strcmp(*it,"file"))
                         {
                             print_verbose_messages |= VB_FILE;
                             verboseString += " " + *it;
                         }
-                        else if(!strcmp(*it,"schedule"))
+                        else if (!strcmp(*it,"schedule"))
                         {
                             print_verbose_messages |= VB_SCHEDULE;
                             verboseString += " " + *it;
                         }
-                        else if(!strcmp(*it,"network"))
+                        else if (!strcmp(*it,"network"))
                         {
                             print_verbose_messages |= VB_NETWORK;
                             verboseString += " " + *it;
                         }
-                        else if(!strcmp(*it,"commflag"))
+                        else if (!strcmp(*it,"commflag"))
                         {
                             print_verbose_messages |= VB_COMMFLAG;
                             verboseString += " " + *it;
                         }
-                        else if(!strcmp(*it,"audio"))
+                        else if (!strcmp(*it,"audio"))
                         {
                             print_verbose_messages |= VB_AUDIO;
                             verboseString += " " + *it;
                         }
+                        else if (!strcmp(*it,"libav"))
+                        {
+                            print_verbose_messages |= VB_LIBAV;
+                            verboseString += " " + *it;
+                        }
                         else
                         {
                             cerr << "Unknown argument for -v/--verbose: "
@@ -801,7 +819,7 @@ int main(int argc, char **argv)
                     "-v or --verbose debug-level    Prints more information" << endl <<
                     "                               Accepts any combination (separated by comma)" << endl << 
                     "                               of all,none,quiet,record,playback,channel," << endl <<
-                    "                               osd,file,schedule,network,commflag,audio" << endl <<
+                    "                               osd,file,schedule,network,commflag,audio,libav" << endl <<
                     "--version                      Version information" << endl <<
                     "<plugin>                       Initialize and run this plugin" << endl;
             return -1;
@@ -813,14 +831,14 @@ int main(int argc, char **argv)
     if (logfile != "")
     {
         logfd = open(logfile.ascii(), O_WRONLY|O_CREAT|O_APPEND, 0664);
-         
+ 
         if (logfd < 0)
         {
             perror("open(logfile)");
             return -1;
         }
     }
-    
+
     if (logfd != -1)
     {
         // Send stdout and stderr to the logfile
@@ -838,6 +856,17 @@ int main(int argc, char **argv)
         dir.mkdir(fileprefix);
 
     gContext = new MythContext(MYTH_BINARY_VERSION);
+   
+    // Create priveleged thread, then drop privs
+    pthread_t priv_thread;
+
+    int status = pthread_create(&priv_thread, NULL, run_priv_thread, NULL);
+    if (status) 
+    {
+        perror("pthread_create");
+        priv_thread = 0;
+    }
+    setuid(getuid());
 
     QSqlDatabase *db = QSqlDatabase::addDatabase("QMYSQL3");
     if (!db)
@@ -871,7 +900,7 @@ int main(int argc, char **argv)
                     ++acttry;
                 }
             }
-            
+
             if (WOLsqlCommand.isEmpty() || (acttry > retries))
             {
                 printf("Sorry, couldn't open db\nExiting.\n");
@@ -889,6 +918,14 @@ int main(int argc, char **argv)
 
     VERBOSE(VB_ALL, QString("Enabled verbose msgs :%1").arg(verboseString));
 
+    lcd_host = gContext->GetSetting("LCDHost", "localhost");
+    lcd_port = gContext->GetNumSetting("LCDPort", 13666);
+    if (lcd_host.length() > 0 && lcd_port > 1024 && gContext->GetLCDDevice())
+    {
+        gContext->GetLCDDevice()->connectToHost(lcd_host, lcd_port);
+        gContext->GetLCDDevice()->setupLEDs(RemoteGetRecordingMask);
+    }
+
     translator->load(PREFIX + QString("/share/mythtv/i18n/mythfrontend_") + 
                      QString(gContext->GetSetting("Language").lower()) + 
                      QString(".qm"), ".");
@@ -906,22 +943,11 @@ int main(int argc, char **argv)
     if (themedir == "")
     {
         cerr << "Couldn't find theme " << themename << endl;
-        exit(0);
+        exit(21);
     }
 
     gContext->LoadQtConfig();
 
-    bool switchMode = gContext->GetNumSetting("UseVideoModes", 0);
-    if (switchMode) 
-    {
-        // xrandr to gui size
-        char buf[128];
-        int screenwidth = gContext->GetNumSetting("GuiVidModeWidth", 0);
-        int screenheight = gContext->GetNumSetting("GuiVidModeHeight", 0);
-        sprintf(buf, "xrandr -s %dx%d", screenwidth, screenheight);
-        system(buf);
-    }
-
     MythMainWindow *mainWindow = new MythMainWindow();
     mainWindow->Show();
     mainWindow->Init();
@@ -936,15 +962,6 @@ int main(int argc, char **argv)
 
     gContext->UpdateImageCache();
 
-    lcd_host = gContext->GetSetting("LCDHost", "localhost");
-    lcd_port = gContext->GetNumSetting("LCDPort", 13666);
-
-    if (lcd_host.length() > 0 && lcd_port > 1024 && gContext->GetLCDDevice())
-    {
-        gContext->GetLCDDevice()->connectToHost(lcd_host, lcd_port);
-        gContext->GetLCDDevice()->setupLEDs(RemoteGetRecordingMask);
-    }
-
     if (gContext->GetNumSetting("EnableXbox") == 1)
     {
         xbox = new XBox();
@@ -968,18 +985,16 @@ int main(int argc, char **argv)
             if (pmanager->run_plugin(pluginname))
                 return 0;
         }
-    }            
+    }
 
     qApp->lock();
     qApp->unlock();
 
 #ifndef _WIN32
     MediaMonitor *mon = NULL;
-
-    if (gContext->GetNumSetting("MonitorDrives") == 1)
+    mon = MediaMonitor::getMediaMonitor();
+    if (mon)
     {
-        mon = new MediaMonitor(NULL, 500, true);
-        mon->addFSTab();
         VERBOSE(VB_ALL, QString("Starting media monitor."));
         mon->startMonitoring();
     }
@@ -998,6 +1013,13 @@ int main(int argc, char **argv)
     }
 #endif
 
+    if (priv_thread != 0)
+    {
+        void *value;
+        gContext->addPrivRequest(MythPrivRequest::MythExit, NULL);
+        pthread_join(priv_thread, &value);
+    }
+
     delete mainWindow;
     delete gContext;
     return exitstatus;
