@@ -18,6 +18,7 @@ using namespace std;
 #include "programrecpriority.h"
 #include "scheduledrecording.h"
 #include "infodialog.h"
+#include "proglist.h"
 #include "tv.h"
 
 #include "dialogbox.h"
@@ -131,7 +132,7 @@ ProgramRecPriority::ProgramRecPriority(QSqlDatabase *ldb, MythMainWindow *parent
     else
     {
         cerr << "MythFrontEnd: ProgramRecPriority - Failed to get selector object.\n";
-        exit(0);
+        exit(27);
     }
 
     bgTransBackup = gContext->LoadScalePixmap("trans-backup.png");
@@ -224,6 +225,11 @@ void ProgramRecPriority::keyPressEvent(QKeyEvent *e)
                 saveRecPriority();
                 edit();
             }
+            else if (action == "UPCOMING")
+            {
+                saveRecPriority();
+                upcoming();
+            }
             else
                 handled = false;
         }
@@ -252,7 +258,7 @@ void ProgramRecPriority::LoadWindow(QDomElement &element)
             else
             {
                 cerr << "Unknown element: " << e.tagName() << endl;
-                exit(0);
+                exit(28);
             }
         }
     }
@@ -419,22 +425,12 @@ void ProgramRecPriority::edit(void)
     if (rec)
     {
         int recid = 0;
-
-        if ((gContext->GetNumSetting("AdvancedRecord", 0)) ||
-            (rec->GetProgramRecordingStatus(db) > kAllRecord))
-        {
-            ScheduledRecording record;
+        ScheduledRecording record;
+        record.loadByID(db, rec->recordid);
+        if (record.getSearchType() == kNoSearch)
             record.loadByProgram(db, rec);
-            record.exec(db);
-            recid = record.getRecordID();
-        }
-        else
-        {
-            InfoDialog diag(rec, gContext->GetMainWindow(), "Program Info");
-            diag.exec();
-        }
-
-        ScheduledRecording::signalChange(db);
+        record.exec(db);
+        recid = record.getRecordID();
 
         // We need to refetch the recording priority values since the Advanced
         // Recording Options page could've been used to change them 
@@ -519,6 +515,20 @@ void ProgramRecPriority::edit(void)
     }
 }
 
+void ProgramRecPriority::upcoming(void)
+{
+    if (!curitem)
+        return;
+
+    ProgramRecPriorityInfo *rec = curitem;
+
+    ProgLister *pl = new ProgLister(plTitle, rec->title,
+                                   QSqlDatabase::database(),
+                                   gContext->GetMainWindow(), "proglist");
+    pl->exec();
+    delete pl;
+}
+
 void ProgramRecPriority::changeRecPriority(int howMuch) 
 {
     int tempRecPriority, cnt;
@@ -533,7 +543,7 @@ void ProgramRecPriority::changeRecPriority(int howMuch)
 
     // inc/dec recording priority
     tempRecPriority = progInfo->recpriority + howMuch;
-    if(tempRecPriority > -100 && tempRecPriority < 100) 
+    if (tempRecPriority > -100 && tempRecPriority < 100) 
     {
         progInfo->recpriority = tempRecPriority;
 
@@ -559,7 +569,6 @@ void ProgramRecPriority::saveRecPriority(void)
         if (progInfo->recpriority != origRecPriorityData[key])
             progInfo->ApplyRecordRecPriorityChange(db, progInfo->recpriority);
     }
-    ScheduledRecording::signalChange(db);
 }
 
 void ProgramRecPriority::FillList(void)
@@ -656,7 +665,7 @@ void ProgramRecPriority::FillList(void)
             {
                 ProgramRecPriorityInfo *progInfo = &(it.data());
                 QString keyB = progInfo->MakeUniqueKey();
-                if(keyA == keyB)
+                if (keyA == keyB)
                 {
                     progInfo->channelRecPriority = channelRecPriority;
                     progInfo->recTypeRecPriority = recTypeRecPriority;
@@ -845,6 +854,9 @@ void ProgramRecPriority::updateList(QPainter *p)
                         ltype->SetItemText(cnt, 5, 
                                            QString::number(abs(finalRecPriority)));
 
+                        if (progInfo->rectype == kDontRecord)
+                            ltype->EnableForcedFont(cnt, "inactive");
+
                         cnt++;
                         listCount++;
                     }
@@ -964,7 +976,7 @@ void ProgramRecPriority::updateInfo(QPainter *p)
             }
             type = (UITextType *)container->GetType("typesign");
             if (type) {
-                if(rectyperecpriority >= 0)
+                if (rectyperecpriority >= 0)
                     type->SetText("+");
                 else
                     type->SetText("-");
@@ -972,7 +984,7 @@ void ProgramRecPriority::updateInfo(QPainter *p)
 
             type = (UITextType *)container->GetType("channel");
             if (type) {
-                if(rectype != kAllRecord && rectype != kFindOneRecord)
+                if (rectype != kAllRecord && rectype != kFindOneRecord)
                     type->SetText(curitem->ChannelText(channelFormat));
                 else
                     type->SetText(tr("Any"));
@@ -980,7 +992,7 @@ void ProgramRecPriority::updateInfo(QPainter *p)
 
             type = (UITextType *)container->GetType("longchannel");
             if (type) {
-                if(rectype != kAllRecord && rectype != kFindOneRecord)
+                if (rectype != kAllRecord && rectype != kFindOneRecord)
                     type->SetText(curitem->ChannelText(longChannelFormat));
                 else
                     type->SetText(tr("Any"));
@@ -993,7 +1005,7 @@ void ProgramRecPriority::updateInfo(QPainter *p)
 
             type = (UITextType *)container->GetType("channelsign");
             if (type) {
-                if(chanrecpriority >= 0)
+                if (chanrecpriority >= 0)
                     type->SetText("+");
                 else
                     type->SetText("-");
@@ -1001,7 +1013,7 @@ void ProgramRecPriority::updateInfo(QPainter *p)
 
             type = (UITextType *)container->GetType("recpriority");
             if (type) {
-                if(curitem->recpriority >= 0)
+                if (curitem->recpriority >= 0)
                     type->SetText("+"+QString::number(curitem->recpriority));
                 else
                     type->SetText(QString::number(curitem->recpriority));
