@@ -21,6 +21,8 @@ class ViewScheduled : public MythDialog
 
   protected slots:
     void edit();
+    void upcoming();
+    void details();
     void selected();
     void cursorDown(bool page = false);
     void cursorUp(bool page = false);
@@ -36,8 +38,6 @@ class ViewScheduled : public MythDialog
     void FillList(void);
     void setShowAll(bool all);
 
-    void fillUpdateAll();
-
     void updateBackground(void);
     void updateList(QPainter *);
     void updateConflict(QPainter *);
@@ -70,7 +70,12 @@ class ViewScheduled : public MythDialog
 
     bool showAll;
 
-    PGInfoCon recList;
+    bool inEvent;
+    bool inFill;
+    bool needFill;
+
+    int listPos;
+    ProgramList recList;
 };
 
 #endif
