@@ -1,6 +1,6 @@
 #include <qapplication.h>
 #include <qsqldatabase.h>
-#include <stdlib.h>
+#include <cstdlib>
 
 #include "libmyth/mythcontext.h"
 #include "libmyth/settings.h"
@@ -8,8 +8,6 @@
 #include "libmythtv/progfind.h"
 #include "libmythtv/tv.h"
 
-MythContext *gContext;
-
 int main(int argc, char **argv)
 {
     QApplication a(argc, argv);
