@@ -42,9 +42,9 @@ class SetBits
 #define READ_PTS ((GETBITS(0, 3)<<30) | (marker_bit() & 0) | (GETBITS(0, 15) << 15) | \
                   (marker_bit() & 0) | GETBITS(0, 15) | (marker_bit() & 0))
 #define READ_PTS_EXT (GETBITS(0, 9) | (marker_bit() & 0))
-#define PTS2FLOAT(pts) ((float)(pts).val * (outputContext->pts_num) / (outputContext->pts_den))
-#define PTS2INT(pts) ((int64_t)(pts).val * AV_TIME_BASE * (outputContext->pts_num) / (outputContext->pts_den))
-#define INT2PTS(val) ((val) * (outputContext->pts_den) / (outputContext->pts_num) / AV_TIME_BASE)
+#define PTS2FLOAT(pts) ((float)(pts).val * (outputContext->time_base.num) / (outputContext->time_base.den))
+#define PTS2INT(pts, st) ((int64_t)(pts).val * AV_TIME_BASE * (st->time_base.num) / (st->time_base.den))
+#define INT2PTS(val, st) ((val) * (st->time_base.den) / (st->time_base.num) / AV_TIME_BASE)
 
 const int ac3_ratetable[] = { 32,  40,  48,  56,  64,  80,  96, 112,
                          128, 160, 192, 224, 256, 320, 384, 448,
@@ -64,7 +64,7 @@ void DPRINTF(int level, const char *format, ...)
 
 void FrameBuffer::setPkt(AVPacket *newpkt, int64_t del)
 {
-    if(newpkt->size + pkt.size > size)
+    if (newpkt->size + pkt.size > size)
     {
         pkt.data = (uint8_t *)realloc(pkt.data, pkt.size + newpkt->size);
         size = pkt.size + newpkt->size;
@@ -72,6 +72,7 @@ void FrameBuffer::setPkt(AVPacket *newpkt, int64_t del)
     memcpy(pkt.data + pkt.size, newpkt->data, newpkt->size);
     pkt.size += newpkt->size;
     pkt.pts = newpkt->pts;
+    pkt.flags = newpkt->flags;
     delta = del;
 }
 
@@ -105,7 +106,7 @@ void GetBits::MarkerBit(void)
     if (! GetNext(1))
     {
         cerr <<  "Marker bit was not valid!\n";
-        exit(1);
+        exit(31);
     }
 }
 
@@ -203,7 +204,7 @@ void MPEG2trans::write_muxed_frame(AVStream *stream, AVPacket *pkt, int advance,
         if (audioPool.isEmpty() )
         {
             cerr << "Ran out of audio buffers!\n";
-            exit(1);
+            exit(32);
         }
         DPRINTF(1, "AUDMUX : %d buffers left\n", audioPool.count());
     }
@@ -222,7 +223,7 @@ void MPEG2trans::write_muxed_frame(AVStream *stream, AVPacket *pkt, int advance,
         if (videoPool.isEmpty())
         {
             cerr << "Ran out of video buffers!\n";
-            exit(1);
+            exit(33);
         }
         DPRINTF(1, "VIDMUX : %d buffers left\n", videoPool.count());
     }
@@ -249,12 +250,12 @@ void MPEG2trans::write_muxed_frame(AVStream *stream, AVPacket *pkt, int advance,
         if (av_set_parameters(outputContext, NULL) < 0)
         {
             cerr << "Invalid output format parameters\n";
-            exit(1);
+            exit(34);
         }
         /* write the stream header, if any */
         av_write_header(outputContext);
         dump_format(outputContext, 0, outputContext->filename, 1);
-        videoout_st->pts.val = INT2PTS(videoQueue.head()->getPTS());
+        videoout_st->pts.val = INT2PTS(videoQueue.head()->getPTS(), videoout_st);
     }
 
     while ((vidpts < audpts && ! videoQueue.isEmpty()) ||
@@ -266,18 +267,16 @@ void MPEG2trans::write_muxed_frame(AVStream *stream, AVPacket *pkt, int advance,
         if (audpts <= vidpts) {
             FrameBuffer *buf = audioQueue.dequeue();
             int64_t bufpts = buf->getPTS() - buf->getDelta();
-            audioout_st->pts.val = INT2PTS(bufpts);
-//            long long orig = PTS2INT(audioout_st->pts);
+            audioout_st->pts.val = INT2PTS(bufpts, audioout_st);
+//            long long orig = PTS2INT(audioout_st->pts, audioout_st);
             audioout_st->codec.coded_frame= &avframe;
-            if (av_write_frame(outputContext, audioout_st->index, 
-                               buf->getData(), buf->getSize()) != 0)
+            if (av_write_frame(outputContext, buf->getPkt()) != 0)
             {
                 cerr << "Error while writing audio frame\n";
-                exit(1);
+                exit(35);
             }
-            DPRINTF(1, "AUDIO PTS (initial): %lld (stored): %lld\n", buf->getPTS(), PTS2INT(audioout_st->pts));
-//            DPRINTF(1, "AUDIO PTS (initial): %lld (stored): %lld %d\n", orig, PTS2INT(audioout_st->pts), outputContext->pts_den);
-            audpts += PTS2INT(audioout_st->pts) - bufpts;
+            DPRINTF(1, "AUDIO PTS (initial): %lld (stored): %lld\n", buf->getPTS(), PTS2INT(audioout_st->pts, audioout_st));
+            audpts += PTS2INT(audioout_st->pts, audioout_st) - bufpts;
             buf->Reset();
             audioPool.push(buf);
         }
@@ -285,17 +284,16 @@ void MPEG2trans::write_muxed_frame(AVStream *stream, AVPacket *pkt, int advance,
         {
             FrameBuffer *buf = videoQueue.dequeue();
             int64_t bufpts = buf->getPTS() - buf->getDelta();
-            videoout_st->pts.val = INT2PTS(bufpts);
-//            int64_t oldpts = PTS2INT(videoout_st->pts);
+            videoout_st->pts.val = INT2PTS(bufpts, videoout_st);
+//            int64_t oldpts = PTS2INT(videoout_st->pts, videoout_st);
             videoout_st->codec.coded_frame= &avframe;
-            if (av_write_frame(outputContext, videoout_st->index, buf->getData(), buf->getSize()) != 0)
+            if (av_write_frame(outputContext, buf->getPkt()) != 0)
             {
                 cerr << "Error while writing audio frame\n";
-                exit(1);
+                exit(36);
             }
-            DPRINTF(1, "VIDEO PTS (initial): %lld (stored): %lld\n", buf->getPTS(), PTS2INT(videoout_st->pts));
-            vidpts += PTS2INT(videoout_st->pts) - bufpts;
-//            vidpts += PTS2INT(videoout_st->pts) - oldpts;
+            DPRINTF(1, "VIDEO PTS (initial): %lld (stored): %lld\n", buf->getPTS(), PTS2INT(videoout_st->pts, videoout_st));
+            vidpts += PTS2INT(videoout_st->pts, videoout_st) - bufpts;
             buf->Reset();
             videoPool.push(buf);
         }
@@ -371,7 +369,7 @@ uint32_t MPEG2trans::process_mp2_audio(AVPacket *pkt)
     if (check_mp2_header(buf))
     {
         uint8_t pad;
-        if(buf[1] != 0xfd)
+        if (buf[1] != 0xfd)
         {
             cerr << "Only MP2 audio is currently supported\n";
             assert(0);
@@ -524,7 +522,7 @@ bool MPEG2trans::process_video(AVPacket *pkt, bool gopsearch)
             }
             else
                 frame_len = 4 + parse_seq_header(vidblkptr + 4, false);
-            if(! chopping && cutlistIter != cutlistMap.end() && 
+            if (! chopping && cutlistIter != cutlistMap.end() && 
                last_frame_number >= cutlistIter.key())
             {
                 chopping = 1;
@@ -549,17 +547,17 @@ bool MPEG2trans::process_video(AVPacket *pkt, bool gopsearch)
             found_gop = true;
             last_gop_pts = thispkt->pts;
             frame_start = vidblkptr;
-            if(chop_end)
+            if (chop_end)
                 chop_end = false;
             last_frame_number += pic_frame_count;
-            if(! chopping && cutlistIter != cutlistMap.end() && 
+            if (! chopping && cutlistIter != cutlistMap.end() && 
                last_frame_number >= cutlistIter.key() && 
                last_frame_number < cutlistIter.data())
             {
                 chopping = 1;
                 chop_start_pts = thispkt->pts;
             }
-            else if(chopping && cutlistIter != cutlistMap.end() && 
+            else if (chopping && cutlistIter != cutlistMap.end() && 
                last_frame_number >= cutlistIter.data())
             {
                 chopping = 0;
@@ -587,7 +585,7 @@ bool MPEG2trans::process_video(AVPacket *pkt, bool gopsearch)
                             gop_ts.sec ) * AV_TIME_BASE +
                            gop_ts.pic * AV_TIME_BASE / video_frame_rate - 
                            goppts;
-            if(chop_end)
+            if (chop_end)
             {
                 closed = 0;
                 broken = 1;
@@ -623,11 +621,11 @@ bool MPEG2trans::process_video(AVPacket *pkt, bool gopsearch)
                 default: pictype = 'X'; break;
             }
             if (0 && chop_end) {
-                if(pic_frame_count < 2 && pictype == 'B') {
+                if (pic_frame_count < 2 && pictype == 'B') {
                     skip_pic_frame = 1;
                     skipped_frames++;
                 } else {
-                    if(pic_frame_count == 0)
+                    if (pic_frame_count == 0)
                         num = 0;
                     else
                         num -= skipped_frames;
@@ -654,9 +652,9 @@ bool MPEG2trans::process_video(AVPacket *pkt, bool gopsearch)
         return false;
     if ((uint32_t)(vidblkptr - thispkt->data) == videoFrame.getSize())
     {
-        if(! chopping && !skip_pic_frame && frame_start)
+        if (! chopping && !skip_pic_frame && frame_start)
         {
-            if(found_gop)
+            if (found_gop)
                 write_muxed_frame(videoout_st, seqFrame.getPkt(), false);
             AVPacket picpkt;
             picpkt.pts = thispkt->pts;
@@ -684,11 +682,11 @@ int MPEG2trans::DoTranscode(QString inputFilename, QString outputFilename)
        < 0) {
      cerr << "failed to open file '" << inputFilename 
           << "' errcode: " << err << "\n";
-     exit(1);
+     exit(37);
    }
    if (av_find_stream_info(inputContext) < 0) {
      cerr << "Couldn't find stream paramters for ' " << inputFilename << "'\n";
-     exit(1);
+     exit(38);
    }
    for(i = 0; i < inputContext->nb_streams; i++) {
      AVCodecContext *enc = &inputContext->streams[i]->codec;
@@ -720,14 +718,14 @@ int MPEG2trans::DoTranscode(QString inputFilename, QString outputFilename)
     fmt = guess_format("vob", NULL, NULL);
     if (!fmt) {
         cerr << "Could not find suitable output format\n";
-        exit(1);
+        exit(39);
     }
    
     /* allocate the output media context */
     outputContext = (AVFormatContext *)av_mallocz(sizeof(AVFormatContext));
     if (!outputContext) {
         cerr << "Memory error\n";
-        exit(1);
+        exit(40);
     }
     outputContext->oformat = fmt;
     snprintf(outputContext->filename,
@@ -747,7 +745,7 @@ int MPEG2trans::DoTranscode(QString inputFilename, QString outputFilename)
     /* open the output file, if needed */
     if (url_fopen(&outputContext->pb, outputFilename, URL_WRONLY) < 0) {
             cerr << "Could not open '" << outputFilename << "'\n";
-            exit(1);
+            exit(41);
     }
     
    }
@@ -805,12 +803,12 @@ uint32_t MPEG2trans::parse_seq_header(uint8_t *PES_data, bool store)
         gb->SkipNext(50);
     }
     gb->SkipNext(12);
-    if(gb->GetNext(1))
+    if (gb->GetNext(1))
     {
         gb->SkipNext(8*64);
         len += 64;
     }
-    if(gb->GetNext(1))
+    if (gb->GetNext(1))
     {
         gb->SkipNext(8*64);
         len += 64;
@@ -969,11 +967,11 @@ void MPEG2trans::BuildKeyframeIndex(QString filename, QMap <long long, long long
     if ((err = av_open_input_file(&inputContext, filename, NULL, 0, ap)) < 0) {
         cerr << "failed to open file '" << filename 
              << "' errcode: " << err << "\n";
-        exit(1);
+        exit(42);
     }
     if (av_find_stream_info(inputContext) < 0) {
         cerr << "Couldn't find stream paramters for ' " << filename << "'\n";
-        exit(1);
+        exit(43);
     }
     for(i = 0; i < inputContext->nb_streams; i++) {
         AVCodecContext *enc = &inputContext->streams[i]->codec;
